/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orx.h
 * @date 02/09/2005
 * @author
 *
 * @todo
 '/

/'*
 * @addtogroup Orx
 *
 * Main orx include
 *
 * @{
 '/

#inclib "orx"
#inclib "stdc++"

#ifndef _orx_BI
#define _orx_BI


#define __orxEXTERN__ /' Not compiling orx library '/


#include "orxInclude.bi"
#include "orx/orxKernel.bi"
#include "orx/orxUtils.bi"


/'**************************************************************************
 * Static variables                                                        *
 **************************************************************************'/

/'* Should stop execution by default event handling?
 '/
Dim Shared sbStopByEvent As orxBOOL  = 0


/'**************************************************************************
 * Public functions                                                        *
 **************************************************************************'/

/'* Orx default basic event handler
 * @param[in]   _pstEvent                     Sent event
 * @return      orxSTATUS_SUCCESS if handled / orxSTATUS_FAILURE otherwise
 '/

Declare Function orx_DefaultEventHandler(ByVal _pstEvent As Const orxEVENT Ptr) As orxSTATUS

Function orx_DefaultEventHandler(ByVal _pstEvent As Const orxEVENT Ptr) As orxSTATUS

  Dim As orxSTATUS eResult = orxSTATUS_SUCCESS

  ' Checks
  orxASSERT(_pstEvent->eType = orxEVENT_TYPE_SYSTEM)

  'Depending on event ID
  Select Case (_pstEvent->eID)
  
    ' Close event
    Case orxSYSTEM_EVENT_CLOSE
    
      ' Updates status
      sbStopByEvent = orxTRUE
    Case Else
       'Do nothing
    End Select 

  'Done!
  Return eResult

End Function

/'* Default main setup (module dependencies)
'/
Declare Sub orx_MainSetup()

/''* Default main setup (module dependencies)
''/
Sub orx_MainSetup()

  ' Adds module dependencies
  orxModule_AddDependency(orxMODULE_ID_MAIN, orxMODULE_ID_PARAM)
  orxModule_AddDependency(orxMODULE_ID_MAIN, orxMODULE_ID_CLOCK)
  orxModule_AddDependency(orxMODULE_ID_MAIN, orxMODULE_ID_CONFIG)
  orxModule_AddDependency(orxMODULE_ID_MAIN, orxMODULE_ID_INPUT)
  orxModule_AddDependency(orxMODULE_ID_MAIN, orxMODULE_ID_EVENT)
  orxModule_AddDependency(orxMODULE_ID_MAIN, orxMODULE_ID_FILE)
  orxModule_AddDependency(orxMODULE_ID_MAIN, orxMODULE_ID_FILESYSTEM)
  orxModule_AddDependency(orxMODULE_ID_MAIN, orxMODULE_ID_LOCALE)
  orxModule_AddDependency(orxMODULE_ID_MAIN, orxMODULE_ID_PLUGIN)
  orxModule_AddDependency(orxMODULE_ID_MAIN, orxMODULE_ID_OBJECT)
  orxModule_AddOptionalDependency(orxMODULE_ID_MAIN, orxMODULE_ID_SCREENSHOT)	
  
End Sub


/'* Orx main execution function
 * @param[in]   _u32NbParams                  Main function parameters number (argc)
 * @param[in]   _azParams                     Main function parameter list (argv)
 * @param[in]   _pfnInit                      Main init function (should init all the main stuff and register the main event handler to override the default one)
 * @param[in]   _pfnRun                       Main run function (will be called once per frame, should return orxSTATUS_SUCCESS to continue processing)
 * @param[in]   _pfnExit                      Main exit function (should clean all the main stuff)
 '/

Declare Sub orx_Execute(ByVal _u32NbParams As orxU32,_azParams As orxSTRING Ptr,_
                        ByVal _pfnInit As Const orxMODULE_INIT_FUNCTION, _
								        ByVal _pfnRun As Const orxMODULE_RUN_FUNCTION,_
								        ByVal _pfnExit As Const orxMODULE_EXIT_FUNCTION _
							         )

Sub orx_Execute(ByVal _u32NbParams As orxU32,_azParams As orxSTRING Ptr,_
                ByVal _pfnInit As Const orxMODULE_INIT_FUNCTION, _
								ByVal _pfnRun As Const orxMODULE_RUN_FUNCTION,_
								ByVal _pfnExit As Const orxMODULE_EXIT_FUNCTION _
							 )


   /' Inits the Debug System '/
		orxDEBUG_INIT()

		' Checks
		orxASSERT(_u32NbParams > 0)
		orxASSERT(_azParams <> orxNULL)
		orxASSERT(_pfnInit <> orxNULL)
		orxASSERT(_pfnRun <> orxNULL)
		orxASSERT(_pfnExit <> orxNULL)
    
		
		' Registers main module
		orxModule_Register(orxMODULE_ID_MAIN, @orx_MainSetup, _pfnInit, _pfnExit)
		Print "orxModule_Register(orxMODULE_ID_MAIN, @orx_MainSetup, _pfnInit, _pfnExit)"
		' Registers all other modules
		orxModule_RegisterAll()
    Print "orxModule_RegisterAll()"
		'Calls all modules setup
		orxModule_SetupAll()
    Print "orxModule_SetupAll()"
		'Sends the command line arguments to orxParam module
		If(orxParam_SetArgs(_u32NbParams, _azParams) <> orxSTATUS_FAILURE) Then
		  
			' Inits the engine
			Print "parameters set"
			If(orxModule_Init(orxMODULE_ID_MAIN) <> orxSTATUS_FAILURE) Then
			  Print "orxModule_Init succeeds"
				
				Dim As orxSTATUS eClockStatus
				Dim As orxSTATUS eMainStatus
				Dim As orxBOOL   bStop

				' Registers default event handler
				
				orxEvent_AddHandler(orxEVENT_TYPE_SYSTEM, @orx_DefaultEventHandler)
				Print "orxEvent_AddHandler(orxEVENT_TYPE_SYSTEM, @orx_DefaultEventHandler)"
        
				' Displays help
				If(orxParam_DisplayHelp() <> orxSTATUS_FAILURE) Then
				
				'Main loop
				 bStop = orxFALSE 
				 Print "bStop = orxFALSE "
				 
				  While (1)'(bStop)
					
						'Runs the engine
						eMainStatus = _pfnRun()
						

						'Updates clock system
						eClockStatus = orxClock_Update()
					  'bstop = Iif(((sbStopByEvent <> orxFALSE) OrElse (eMainStatus = orxSTATUS_FAILURE) OrElse (eClockStatus = orxSTATUS_FAILURE)), orxTRUE,orxFALSE)
					Wend
				End If

				'Removes event handler
				orxEvent_RemoveHandler(orxEVENT_TYPE_SYSTEM, @orx_DefaultEventHandler)

				' Exits from engine
				orxModule_Exit(orxMODULE_ID_MAIN)
			End If
      
			'Exits from all modules
			orxModule_ExitAll()
		End If
    
		'Exits from the Debug system
		''orxDEBUG_EXIT()


End Sub


'#ifdef __orxWINDOWS__

'#include "windows.bi"

'/''* Orx main execution function (console-less windows application)
 '* @param[in]   _pfnInit                      Main init function (should init all the main stuff and register the main event handler to override the default one)
 '* @param[in]   _pfnRun                       Main run function (will be called once per frame, should return orxSTATUS_SUCCESS to continue processing)
 '* @param[in]   _pfnExit                      Main exit function (should clean all the main stuff)
 '''/
'#macro orx_WinExecute(_pfnInit,_pfnRun,_pfnExit)

'Scope
  '#define orxMAX_ARGS 256

  'Dim As Integer argc
  'Dim As ZString Ptr argv(orxMAX_ARGS)
  'Dim As ZString Ptr pcToken
	'Dim As ZString Ptr pcNextToken
	'Dim As ZString Ptr pcFirstDelimiters
  'Dim As LPSTR lpFullCmdLine
  'Dim As LPSTR pcTmp
  '' Gets full command line
  'lpFullCmdLine = GetCommandLine()

  ''Starts with a double quote?
  'orxString_SkipWhiteSpaces(lpFullCmdLine,pcTmp)
	'If(pcTmp[0][0] = Asc("""")) Then  
    ''Gets first delimiters
    'pcFirstDelimiters = !"\""  
  'Else  
    ''Gets first delimiters
    'pcFirstDelimiters = " "
  'End If

  ''Process command line
  'argc = 0:pcNextTOken = NULL:pcToken=strtok_s((lpFullCmdLine, pcFirstDelimiters, @pcNextToken))
	'While (pcToken AndAlso argc < orxMAX_ARGS)    
		'argv(argc) = pcToken
		'argc += 1
		'pcToken = strtok_s(NULL, " ", @pcNextToken)
  'Wend

  ''Inits and executes orx
  'orx_Execute(argc, argv, _pfnInit, _pfnRun, _pfnExit)

'End Scope
'#endmacro

'#endif /' __orxWINDOWS__ '/

#endif /'_orx_BI'/



/'* @} '/
