/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxAnim.h
 * @date 11/02/2004
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxAnim
 *
 * Animation (Data) Module.
 * Allows to creates and handle Animations data.
 * It consists of a structure containing data for a single animation
 * and functions for handling and accessing them.
 * Animations are structures.
 * They thus can be referenced by Animation Sets (orxAnimSet) Module.
 *
 * @{
 '/


#ifndef _orxANIM_BI
#define _orxANIM_BI


#include "orxInclude.bi"
#include "orx/object/orxStructure.bi"


/'* Anim flags
 '/
#define orxANIM_KU32_FLAG_NONE                &H00000000  /'*< No flags '/

#define orxANIM_KU32_FLAG_2D                  &H00000001  /'*< 2D type animation ID flag '/

#define orxANIM_KU32_MASK_USER_ALL            &H0000000F  /'*< User all ID mask '/

/'* Anim defines
 '/
#define orxANIM_KU32_KEY_MAX_NUMBER           255         /'*< Maximum number of keys for an animation structure '/
#define orxANIM_KU32_EVENT_MAX_NUMBER         15          /'*< Maximum number of events for an animation structure '/


/'* Internal Anim structure
 '/
Type __orxANIM_t As Any
Type orxANIM As __orxANIM_t


/'* Event enum
 '/
Enum orxANIM_EVENT

  orxANIM_EVENT_START = 0,                    /'*< Event sent when an animation starts '/
  orxANIM_EVENT_STOP,                         /'*< Event sent when an animation stops '/
  orxANIM_EVENT_CUT,                          /'*< Event sent when an animation is cut '/
  orxANIM_EVENT_LOOP,                         /'*< Event sent when an animation has looped '/
  orxANIM_EVENT_CUSTOM_EVENT,                 /'*< Event sent when a custom event is reached '/

  orxANIM_EVENT_NUMBER,

  orxANIM_EVENT_NONE = orxENUM_NONE

End Enum

/'* Anim event payload
 '/
Type __orxANIM_EVENT_PAYLOAD_t
  As orxANIM  Ptr pstAnim                          /'*< Animation reference : 4 '/
  As orxSTRING zAnimName                        /'*< Animation name : 8 '/
  As orxSTRING zCustomEventName                 /'*< Custom event name : 12 '/
  As orxFLOAT  fCustomEventValue                /'*< Custom event value : 16 '/
  As orxFLOAT  fCustomEventTime                 /'*< Custom event time : 20 '/

End Type 

Type orxANIM_EVENT_PAYLOAD As __orxANIM_EVENT_PAYLOAD_t

/'* Anim custom event
 '/
Type __orxANIM_CUSTOM_EVENT_t

  As orxSTRING     zName                        /'*< Event name : 4 '/
  As orxFLOAT      fValue                       /'*< Event value : 8 '/
  As orxFLOAT      fTimeStamp                   /'*< Timestamo : 12 '/

End Type

Type orxANIM_CUSTOM_EVENT As __orxANIM_CUSTOM_EVENT_t


Extern "Windows"

/'* Anim module setup
 '/

Declare Sub orxAnim_Setup()

/'* Inits the Anim module
 '/
Declare Function orxAnim_Init() As orxSTATUS 

/'* Exits from the Anim module
 '/
Declare Sub orxAnim_Exit()


/'* Creates an empty animation
 * @param[in]   _u32Flags       Flags for created animation
 * @param[in]   _u32KeyNumber   Number of keys for this animation
 * @param[in]   _u32EventNumber Number of events for this animation
 * @return      Created orxANIM / orxNULL
 '/
Declare Function orxAnim_Create(ByVal _u32Flags As orxU32, _
                                                      ByVal _u32KeyNumber As orxU32, _
                                                      ByVal _u32EventNumber As orxU32 _
																										 ) As orxANIM Ptr

/'* Creates an animation from config
 * @param[in]   _zConfigID                    Config ID
 * @return      orxANIMSET / orxNULL
 '/
Declare Function orxAnim_CreateFromConfig(ByVal _zConfigID As orxSTRING) As orxAnim Ptr

/'* Deletes an animation
 * @param[in]   _pstAnim        Anim to delete
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnim_Delete(ByVal _pstAnim As orxANIM Ptr) As orxSTATUS 


/'* Adds a key to an animation
 * @param[in]   _pstAnim        Concerned animation
 * @param[in]   _pstData        Key data to add
 * @param[in]   _fTimeStamp     Timestamp for this key
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnim_AddKey(ByVal _pstAnim As orxANIM Ptr, _
                                Byval _pstData As orxSTRUCTURE Ptr, _
																ByVal _fTimeStamp As orxFLOAT _
															) As orxSTATUS 

/'* Removes last added key from an animation
 * @param[in]   _pstAnim        Concerned animation
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function   orxAnim_RemoveLastKey(ByVal _pstAnim As orxANIM Ptr) As orxSTATUS 

/'* Removes all keys from an animation
 * @param[in]   _pstAnim        Concerned animation
 '/
Declare Sub orxAnim_RemoveAllKeys(ByVal _pstAnim As orxANIM Ptr)


/'* Adds an event to an animation
 * @param[in]   _pstAnim        Concerned animation
 * @param[in]   _zEventName     Event name to add
 * @param[in]   _fTimeStamp     Timestamp for this event
 * @param[in]   _fValue         Value for this event
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnim_AddEvent(ByVal _pstAnim As orxANIM Ptr, _
																	ByVal _zEventName As Const orxSTRING, _
																	ByVal _fTimeStamp As orxFLOAT, _
																	ByVal _fValue As orxFLOAT _
																 ) As  orxSTATUS 

/'* Removes last added event from an animation
 * @param[in]   _pstAnim        Concerned animation
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnim_RemoveLastEvent(ByVal _pstAnim As orxANIM Ptr) As orxSTATUS

/'* Removes all events from an animation
 * @param[in]   _pstAnim        Concerned animation
 '/
Declare Sub orxAnim_RemoveAllEvents(ByVal _pstAnim As orxANIM Ptr)

/'* Gets next event after given timestamp
 * @param[in]   _pstAnim        Concerned animation
 * @param[in]   _fTimeStamp     Time stamp, excluded
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnim_GetNextEvent(ByVal _pstAnim As Const orxANIM Ptr, _
																			ByVal _fTimeStamp As orxFLOAT _
																		) As Const orxANIM_CUSTOM_EVENT Ptr


/'* Updates anim given a timestamp
 * @param[in]   _pstAnim        Concerned animation
 * @param[in]   _fTimeStamp     TimeStamp for animation update
 * @param[out]  _pu32CurrentKey Current key as a result of update
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnim_Update(ByVal _pstAnim As orxANIM Ptr, _
                                ByVal _fTimeStamp As orxFLOAT, _
																ByVal _pu32CurrentKey As orxU32 Ptr _
															  ) As orxSTATUS 

/'* Anim key data accessor
 * @param[in]   _pstAnim        Concerned animation
 * @param[in]   _u32Index       Index of desired key data
 * @return      Desired orxSTRUCTURE / orxNULL
 '/
Declare Function orxAnim_GetKeyData(ByVal _pstAnim As Const orxANIM Ptr, _
                                    ByVal _u32Index As orxU32 _
																	 ) As orxSTRUCTURE Ptr


/'* Anim key storage size accessor
 * @param[in]   _pstAnim        Concerned animation
 * @return      Anim key storage size
 '/
Declare Function orxAnim_GetKeyStorageSize(ByVal _pstAnim As Const orxANIM Ptr) As orxU32 

/'* Anim key counter accessor
 * @param[in]   _pstAnim        Concerned animation
 * @return      Anim key counter
 '/
Declare Function orxAnim_GetKeyCounter(ByVal _pstAnim As Const orxANIM Ptr) As orxU32 


/'* Anim event storage size accessor
 * @param[in]   _pstAnim        Concerned animation
 * @return      Anim event storage size
 '/
Declare Function orxAnim_GetEventStorageSize(ByVal _pstAnim As Const orxANIM Ptr) As orxU32

/'* Anim event counter accessor
 * @param[in]   _pstAnim        Concerned animation
 * @return      Anim event counter
 '/
Declare Function orxAnim_GetEventCounter(ByVal _pstAnim As Const orxANIM Ptr) As orxU32


/'* Anim time length accessor
 * @param[in]   _pstAnim        Concerned animation
 * @return      Anim time length
 '/
Declare Function orxAnim_GetLength(ByVal _pstAnim As Const orxANIM Ptr) As orxFLOAT 

/'* Anim ID get accessor
 * @param[in]   _pstAnim        Concerned animation
 * @return      Anim ID / orxU32_UNDEFINED
 '/
Declare Function orxAnim_GetID(ByVal _pstAnim As Const orxANIM Ptr) As orxU32

/'* Anim name get accessor
 * @param[in]   _pstAnim        Concerned animation
 * @return      orxSTRING / orxSTRING_EMPTY
 '/
Declare Function  orxAnim_GetName(ByVal _pstAnim As orxANIM Ptr) As Const orxSTRING

End Extern

#endif /' _orxANIM_BI '/

/'* @} '/
