/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxAnimPointer.h
 * @date 03/03/2004
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxAnimPointer
 *
 * Animation (Pointer) Module.
 * Allows to creates and handle Animation Set Pointers.
 * It consists of a structure containing pointers and counters referenced to an Animation Set.
 * It also contains functions for handling and accessing animations of the referenced Animation Set.
 * Animation Pointers are structures.
 * They thus can be referenced by Graphics.
 *
 * @{
 '/


#ifndef _orxANIMPOINTER_BI
#define _orxANIMPOINTER_BI


#include "orxInclude.bi"
#include "orx/anim/orxAnimSet.bi"


/'* Internal AnimPointer structure
 '/
Type __orxANIMPOINTER_t As Any Ptr
Type orxANIMPOINTER As __orxANIMPOINTER_t

Extern "Windows"

/'* AnimPointer module setup
 '/
Declare Sub orxAnimPointer_Setup()

/'* Inits the AnimPointer module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimPointer_Init() As orxSTATUS

/'* Exits from the AnimPointer module
 '/
Declare Sub orxAnimPointer_Exit()


/'* Creates an empty AnimPointer
 * @param[in]   _pstOwner                     AnimPointer's owner used for event callbacks (usually an orxOBJECT)
 * @param[in]   _pstAnimSet                   AnimSet reference
 * @return      orxANIMPOINTER / orxNULL
 '/
Declare Function orxAnimPointer_Create(ByVal _pstOwner As Const orxSTRUCTURE Ptr, _
                                       ByVal _pstAnimSet As orxANIMSET Ptr _
																			) As orxANIMPOINTER Ptr

/'* Creates an animation pointer from config
 * @param[in]   _pstOwner                     AnimPointer's owner used for event callbacks (usually an orxOBJECT)
 * @param[in]   _zConfigID                    Config ID
 * @return      orxANIMPOINTER / orxNULL
 '/
Declare Function orxAnimPointer_CreateFromConfig(ByVal _pstOwner As Const orxSTRUCTURE Ptr, _
                                                 ByVal _zConfigID As COnst orxSTRING _
																								) As orxANIMPOINTER Ptr

/'* Deletes an AnimPointer
 * @param[in]   _pstAnimPointer               AnimPointer to delete
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimPointer_Delete(ByVal _pstAnimPointer As orxANIMPOINTER Ptr) As orxSTATUS

/'* Gets an AnimPointer owner
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @return      orxSTRUCTURE / orxNULL
 '/
Declare Function orxAnimPointer_GetOwner(ByVal _pstAnimPointer As Const orxANIMPOINTER Ptr) As orxSTRUCTURE Ptr

/'* Gets the referenced AnimSet
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @return      Referenced orxANIMSET
 '/
Declare Function orxAnimPointer_GetAnimSet(ByVal _pstAnimPointer As const orxANIMPOINTER Ptr) As orxANIMSET Ptr


/'* AnimPointer current Animation get accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @return      Current Animation handle
 '/
Declare Function orxAnimPointer_GetCurrentAnimHandle(ByVal _pstAnimPointer As Const orxANIMPOINTER Ptr) As orxHANDLE

/'* AnimPointer target Animation get accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @return      Target Animation handle
 '/
Declare Function orxAnimPointer_GetTargetAnimHandle(ByVal _pstAnimPointer As Const orxANIMPOINTER Ptr) As orxHANDLE 

/'* AnimPointer current Animation ID get accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @return      Current Animation ID
 '/
Declare Function orxAnimPointer_GetCurrentAnim(ByVal _pstAnimPointer As Const orxANIMPOINTER Ptr) As orxU32

/'* AnimPointer target Animation ID get accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @return      Target Animation ID
 '/
Declare Function orxAnimPointer_GetTargetAnim(ByVal _pstAnimPointer As Const orxANIMPOINTER Ptr) As orxU32

/'* AnimPointer current anim data get accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @return      Current anim data / orxNULL
 '/
Declare Function orxAnimPointer_GetCurrentAnimData(ByVal _pstAnimPointer As Const orxANIMPOINTER Ptr) As orxSTRUCTURE Ptr

/'* AnimPointer current Time get accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @return      Current time
 '/
Declare Function orxAnimPointer_GetCurrentTime(ByVal _pstAnimPointer As Const orxANIMPOINTER Ptr) As orxFLOAT 

/'* AnimPointer Frequency get accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @return      AnimPointer frequency
 '/
Declare Function orxAnimPointer_GetFrequency(ByVal _pstAnimPointer As Const orxANIMPOINTER Ptr) As orxFLOAT 

/'* AnimPointer current Animation set accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @param[in]   _hAnimHandle                  Animation handle to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimPointer_SetCurrentAnimHandle(ByVal _pstAnimPointer As orxANIMPOINTER Ptr, _
                                                     ByVal _hAnimHandle_ As orxHANDLE _
																										) As orxSTATUS 

/'* AnimPointer target Animation set accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @param[in]   _hAnimHandle                  Animation handle to set / orxHANDLE_UNDEFINED
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimPointer_SetTargetAnimHandle(ByVal _pstAnimPointer As orxANIMPOINTER Ptr, _
																										ByVal _hAnimHandle As orxHANDLE _
														   										 ) As orxSTATUS

/'* AnimPointer current Animation set accessor using ID
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @param[in]   _u32AnimID                    Animation ID (config's name CRC) to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimPointer_SetCurrentAnim(ByVal _pstAnimPointer As orxANIMPOINTER Ptr, _
																							 ByVal _u32AnimID As orxU32 _
																							) As orxSTATUS

/'* AnimPointer target Animation set accessor using ID
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @param[in]   _u32AnimID                    Animation ID (config's name CRC) to set / orxU32_UNDEFINED
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimPointer_SetTargetAnim(ByVal _pstAnimPointer As orxANIMPOINTER Ptr, _
                                              ByVal _u32AnimID As orxU32 _
																						 ) As orxSTATUS

/'* AnimPointer current Time accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @param[in]   _fTime                        Time to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimPointer_SetTime(ByVal _pstAnimPointer As orxANIMPOINTER Ptr,_
                                        ByVal _fTime As orxFLOAT _
																			 ) As orxSTATUS

/'* AnimPointer Frequency set accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @param[in]   _fFrequency                   Frequency to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimPointer_SetFrequency(ByVal _pstAnimPointer As orxANIMPOINTER Ptr, _
                                             ByVal _fFrequency As orxFLOAT _
																						) As orxSTATUS

/'* AnimPointer pause accessor
 * @param[in]   _pstAnimPointer               Concerned AnimPointer
 * @param[in]   _bPause                       Pause / Unpause
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxAnimPointer_Pause(ByVal _pstAnimPointer As orxANIMPOINTER Ptr, _
                                      ByVal _bPause As orxBOOL _
																			) As orxSTATUS

End Extern

#endif /' _orxANIMPOINTER_BI '/


/'* @} '/
