/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxDecl.h
 * @date 14/04/2005
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxDecl
 *
 * Base declarations
 * Allows to creates and handle Sets of Anims.
 * It consists of a structure containing Anims and their relations.
 * It also contains functions for handling and accessing them.
 * AnimSets are structures.
 * They thus can be referenced by Anim Pointers.
 *
 * @{
 '/


#ifndef _orxDECL_BI
#define _orxDECL_BI


/' *** Platform dependent base declarations '/

/' No compiler defines? '/


/' No platform defines? '/
#if Not(defined(__orxWINDOWS__)) And Not(defined(__orxMAC__)) And Not(defined(__orxLINUX__)) And Not(defined(__orxGP2X__)) And Not(defined(__orxWII__)) And Not(defined(__orxIPHONE__))

  /' Windows? '/
  #if defined(__FB_WIN32__)

    #define __orxWINDOWS__

    /' Linux? '/
  #elseif defined(__FB_LINUX__)

    #define __orxLINUX__  

  #else

    #error "Couldn't guess platform define. Please provide it (__orxWINDOWS__/__orxLINUX__/__orxMAC__/__orxGP2X__/__orxWII__/__orxIPHONE__)"

  #endif

#endif /' Not __orxWINDOWS__ And Not __orxMAC__ And Not __orxLINUX__ And Not __orxGP2X__ AndAlso Not __orxWII__ And Not __orxIPHONE__ '/


/' Windows '/
#ifdef __orxWINDOWS__

  /'* The null adress. '/
  #define orxNULL               (0)

#else /' __orxWINDOWS__ '/

  /' Linux / Mac / GP2X / Wii '/
  #if defined(__orxLINUX__) Or defined(__orxMAC__) Or defined(__orxGP2X__) Or defined(__orxWII__) Or defined(__orxIPHONE__)
		
    /'* The null adress. '/
    #define orxNULL             (0)   

  #endif /' __orxLINUX__ Or __orxMAC__ Or __orxGP2X__ Or __orxWII__ Or __orxIPHONE__ '/

#endif /' __orxWINDOWS__ '/



/'* Memory alignment macros '/
#define _orxALIGN(ADDRESS, BLOCK_SIZE)  ((ADDRESS + (BLOCK_SIZE - 1)) And (Not(BLOCK_SIZE - 1)))

#define orxALIGN16(ADDRESS)             _orxALIGN(ADRESS, 16)
#define orxALIGN32(ADDRESS)             _orxALIGN(ADRESS, 32)
#define orxALIGN64(ADDRESS)             _orxALIGN(ADRESS, 64)





/'* Fast in-place swap macros for two 32-bits objects. '/
#macro orxSWAP32(A, B)
Do
  *(Cast(orxU32 Ptr,@(A))) = *(Cast(orxU32 Ptr,@(A))) Xor *(Cast(orxU32 Ptr,@(B)))
  *(Cast(orxU32 Ptr,@(B))) = *(Cast(orxU32 Ptr,@(A))) Xor *(Cast(orxU32 Ptr,@(B)))
  *(Cast(orxU32 Ptr,@(A))) = *(Cast(orxU32 Ptr,@(A))) Xor *(Cast(orxU32 Ptr,@(B)))
Loop While(0)
#endmacro

/'* Tests all flags
 * @param[in] X Flag container
 * @param[in] F Flags to test
 * @return true if flags are all presents
 '/
#define orxFLAG_TEST_ALL(X, F) (((X) And (F)) = (F))

/'* Tests flags
 * @param[in] X Flag container
 * @param[in] F Flags to test
 * @return true if at least one flag is present
 '/
#define orxFLAG_TEST(X, F) (((X) And (F)) <> 0)

/'* Gets flags
 * @param[in] X Flag container
 * @param[in] M Filtering mask
 * @return Masked flags
 '/
#define orxFLAG_GET(X, M) ((X) And (M))

/'* Sets / unsets flags
 * @param[in] X Flag container
 * @param[in] A Flags to add
 * @param[in] R Flags to remove
 '/
#macro orxFLAG_SET(X, A, R) 
Do
  (X) And= Not(R)
  (X) Or= (A)
Loop While(0)
#endmacro

#endif /'_orxDECL_BI'/

/'* @} '/
