/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxModule.h
 * @date 12/09/2005
 * @author iarwain@orx-project.org
 *
 * @todo
 * - Add internal/external dependency system
 * - Optimize dependencies storage
 '/

/'*
 * @addtogroup orxModule
 *
 * Module file
 * Code that handles modules and their dependencies
 *
 * @{
 '/


#ifndef _orxMODULE_BI
#define _orxMODULE_BI


#include "orx/base/orxType.bi"


/'* Module enum '/
Enum orxMODULE_ID
  orxMODULE_ID_ANIM = 0,
  orxMODULE_ID_ANIMPOINTER,
  orxMODULE_ID_ANIMSET,
  orxMODULE_ID_BANK,
  orxMODULE_ID_BODY,
  orxMODULE_ID_CAMERA,
  orxMODULE_ID_CLOCK,
  orxMODULE_ID_CONFIG,
  orxMODULE_ID_DISPLAY,
  orxMODULE_ID_EVENT,
  orxMODULE_ID_FILE,
  orxMODULE_ID_FILESYSTEM,
  orxMODULE_ID_FPS,
  orxMODULE_ID_FRAME,
  orxMODULE_ID_FX,
  orxMODULE_ID_FXPOINTER,
  orxMODULE_ID_GRAPHIC,
  orxMODULE_ID_INPUT,
  orxMODULE_ID_JOYSTICK,
  orxMODULE_ID_KEYBOARD,
  orxMODULE_ID_LOCALE,
  orxMODULE_ID_MAIN,
  orxMODULE_ID_MEMORY,
  orxMODULE_ID_MOUSE,
  orxMODULE_ID_OBJECT,
  orxMODULE_ID_PARAM,
  orxMODULE_ID_PHYSICS,
  orxMODULE_ID_PLUGIN,
  orxMODULE_ID_RENDER,
  orxMODULE_ID_SCREENSHOT,
  orxMODULE_ID_SHADER,
  orxMODULE_ID_SHADERPOINTER,
  orxMODULE_ID_SOUND,
  orxMODULE_ID_SOUNDPOINTER,
  orxMODULE_ID_SOUNDSYSTEM,
  orxMODULE_ID_SPAWNER,
  orxMODULE_ID_STRUCTURE,
  orxMODULE_ID_SYSTEM,
  orxMODULE_ID_TEXT,
  orxMODULE_ID_TEXTURE,
  orxMODULE_ID_VIEWPORT,

  orxMODULE_ID_NUMBER,

  orxMODULE_ID_MAX_NUMBER = 64,

  orxMODULE_ID_NONE = orxENUM_NONE

End Enum



/' *** setup/init/exit/run function prototypes *** '/
Type orxMODULE_INIT_FUNCTION As Function stdcall () As orxSTATUS
Type orxMODULE_EXIT_FUNCTION As Sub stdcall
Type orxMODULE_RUN_FUNCTION As Function stdcall () As orxSTATUS
Type orxMODULE_SETUP_FUNCTION As Sub stdcall


/'* Registers a module
 * @param[in]   _eModuleID                Concerned module ID
 * @param[in]   _pfnSetup                 Module setup callback
 * @param[in]   _pfnInit                  Module init callback
 * @param[in]   _pfnExit                  Module exit callback
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/

Extern "Windows"

Declare Sub orxModule_Register(ByVal _eModuleID As orxMODULE_ID, _
                               ByVal _pfnSetup As orxMODULE_SETUP_FUNCTION, _
															 ByVal _pfnInit As orxMODULE_INIT_FUNCTION, _
															 ByVal _pfnExit As orxMODULE_EXIT_FUNCTION _
															)

/'* Registers all modules. ! Needs to be updated for all new modules ! '/
Declare Sub orxModule_RegisterAll()

/'* Adds dependencies between 2 modules
 * @param[in]   _eModuleID                Concerned module ID
 * @param[in]   _eDependID                Module ID of the needed module
 '/
Declare Sub orxModule_AddDependency(ByVal _eModuleID As orxMODULE_ID, _
                                    ByVal _eDependID As orxMODULE_ID _
																	 )

/'* Adds optional dependencies between 2 modules
 * @param[in]   _eModuleID                Concerned module ID
 * @param[in]   _eDependID                Module ID of the optionally needed module
 '/
Declare Sub orxModule_AddOptionalDependency(ByVal _eModuleID As orxMODULE_ID, _
                                            ByVal _eDependID As orxMODULE_ID )

/'* Updates dependencies for all modules '/
Declare Sub orxModule_UpdateDependencies()

/'* Calls a module setup callback
 * @param[in]   _eModuleID                Concerned module ID
 '/
Declare Sub orxModule_Setup(ByVal _eModuleID As orxMODULE_ID)

/'* Calls all module setups '/
Declare Sub orxModule_SetupAll()

/'* Inits a module
 * @param[in]   _eModuleID                Concerned module ID
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxModule_Init(ByVal _eModuleID As orxMODULE_ID) As orxSTATUS 

/'* Inits all modules
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxModule_InitAll() As orxSTATUS 

/'* Exits from a module
 * @param[in]   _eModuleID                Concerned module ID
 '/
Declare Sub orxModule_Exit(ByVal _eModuleID As orxMODULE_ID)

/'* Exits from all modules.'/
Declare Sub orxModule_ExitAll()

/'* Is module initialized?
 * @param[in]   _eModuleID                Concerned module ID
 * @return      orxTRUE / orxFALSE
 '/
Declare Function orxModule_IsInitialized(ByVal _eModuleID As orxMODULE_ID) As orxBOOL 

End Extern

#endif /' _orxMODULE_BI '/

/'* @} '/
