/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxType.h
 * @date 12/04/2005
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxType
 * 
 * Base types
 *
 * @{
 '/


#ifndef _orxTYPE_BI
#define _orxTYPE_BI

/' *** Uses orx declarations *** '/
#include "orx/base/orxDecl.bi"

/' *** Platform dependent base types '/

/' Windows '/
#ifdef __orxWINDOWS__

  Type orxHANDLE As Any Ptr

  Type orxU32 As ULong
  Type orxU16 As UShort
  Type orxU8 As UByte

  Type orxS32 As Long
  Type orxS16 As Short
  Type orxS8 As Byte

  Type orxBOOL As ULong

  Type orxFLOAT As Single
  Type orxDOUBLE As Double

  Type orxCHAR As Byte
  Type orxSTRING As ZString Ptr

  Type orxENUM As orxU32

  #define orx2F(V)                (Cast(orxFLOAT,(V)))

  #define orxENUM_NONE            &HFFFFFFFFL 
  

  Type orxU64 As ULongInt
  Type orxS64 As LongInt
  

#else /' __orxWINDOWS__ '/

  /' Linux / Mac '/
  #if defined(__orxLINUX__) Or defined(__orxMAC__) Or defined(__orxGP2X__) Or defined(__orxWII__) Or defined(__orxIPHONE__)

    Type orxHANDLE As Any Ptr

		Type orxU64 As ULongInt
		Type orxU32 As ULong
		Type orxU16 As UShort
		Type orxU8 As UByte

		Type orxS64 As LongInt
		Type orxS32 As Long
		Type orxS16 As Short
		Type orxS8 As Byte

		Type orxBOOL As UByte

		Type orxFLOAT As Single
		Type orxDOUBLE As Double

		Type orxCHAR As Byte
		Type orxSTRING As ZString Ptr

		Type orxENUM As orxU32
		
    #define orx2F(V)              (Cast(orxFLOAT,(V)))

    #define orxENUM_NONE          &HFFFFFFFFL

  #endif /' __orxLINUX__ OrElse __orxMAC__ OrElse __orxGP2X__ OrElse __orxWII__ OrElse __orxIPHONE__ '/

#endif /' __orxWINDOWS__ '/

/' *** Misc constants *** '/

/' *** Boolean constants *** '/
Const orxFALSE As orxBOOL = 0
Const orxTRUE As orxBOOL = 1

/' *** Undefined constants *** '/
Extern orxU64_UNDEFINED As orxU64
Extern orxU32_UNDEFINED As orxU32
Extern orxU16_UNDEFINED As orxU16
Extern orxU8_UNDEFINED As orxU8
Extern orxHANDLE_UNDEFINED As orxHANDLE

Extern orxSTRING_EMPTY As orxSTRING
Extern orxSTRING_TRUE As orxSTRING
Extern orxSTRING_FALSE As orxSTRING
Extern orxCHAR_NULL As orxSTRING
Extern orxCHAR_CR As orxSTRING
Extern orxCHAR_LF As orxSTRING

Extern orxCHAR_EOL As orxCHAR
Extern orxSTRING_EOL As orxSTRING


/' *** Directory separators *** '/

Extern orxCHAR_DIRECTORY_SEPARATOR_WINDOWS As orxCHAR
Extern orxCHAR_DIRECTORY_SEPARATOR_LINUX As orxCHAR
Extern orxSTRING_DIRECTORY_SEPARATOR_WINDOWS As orxCHAR
Extern orxSTRING_DIRECTORY_SEPARATOR_LINUX As orxCHAR

Extern orxCHAR_DIRECTORY_SEPARATOR As orxCHAR
Extern orxSTRING_DIRECTORY_SEPARATOR As orxCHAR


/' *** Float constants *** '/
Extern orxFLOAT_0 As orxFLOAT
Extern orx_FLOAT_1 As orxFLOAT


/' *** Status defines *** '/
Enum orxSTATUS
  orxSTATUS_FAILURE = 0,          /'*< Failure status, the operation has failed '/
  orxSTATUS_SUCCESS,              /'*< Success status, the operation has worked has expected '/

  orxSTATUS_NUMBER,               /'*< Sentinel : Number of status '/

  orxSTATUS_NONE = orxENUM_NONE   /'*< Invalid status '/

End Enum


#endif /'_orxTYPE_BI'/

/'* @} '/
