/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxConfig.bi
 * @date 02/02/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 * - Need to refactor value getters (lot of duplicated code to handle random and lists)
 '/

/'*
 * @addtogroup orxConfig
 * 
 * Config module
 * Module that handles configuration files
 *
 * @{
 '/


#ifndef _orxCONFIG_BI
#define _orxCONFIG_BI


#include "orxInclude.bi"
#include "orx/math/orxVector.bi"


Extern "Windows"

/'* Config callback function type to use with save function '/
Type orxCONFIG_SAVE_FUNCTION As Function(ByVal _zSectionName As Const orxSTRING, _
                                         ByVal _zKeyName As Const orxSTRING, _
																				 ByVal _busEncryption As orxBOOL _
																				 ) As orxBOOL



/'* Config module setup
 '/
Declare Sub orxConfig_Setup()

/'* Initializes the Config Module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_Init() As orxSTATUS

/'* Exits from the Config Module
 '/
Declare Sub orxConfig_Exit()


/'* Sets encryption key
 * @param[in] _zEncryptionKey  Encryption key to use, orxNULL to clear
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_SetEncryptionKey(ByVal _zEncryptionKey As Const orxSTRING) As orxSTATUS

/'* Sets config base name
 * @param[in] _zBaseName        Base name used for default config file
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_SetBaseName(ByVal _zBaseName As Const orxSTRING) As orxSTATUS

/'* Gets config main file name
 * @return Config main file name / orxSTRING_EMPTY
 '/
Declare Function orxConfig_GetMainFileName() As Const orxSTRING 

/'* Loads config file from source
 * @param[in] _zFileName        File name
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_Load(ByVal _zFileName As Const orxSTRING) As orxSTATUS

/'* Reloads config files from history
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_ReloadHistory() As orxSTATUS

/'* Writes config to given file. Will overwrite any existing file, including all comments.
 * @param[in] _zFileName        File name, if null or empty the default file name will be used
 * @param[in] _bUseEncryption   Use file encryption to make it human non-readable?
 * @param[in] _pfnSaveCallback  Callback used to filter section/key to save. If NULL is passed, all section/keys will be saved
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_Save(ByVal _zFileName As Const orxSTRING, _
                                ByVal _busEncryption As orxBOOL, _
																ByVal _pfnSaveCallback As Const orxCONFIG_SAVE_FUNCTION _
															 ) As orxSTATUS

/'* Selects current working section
 * @param[in] _zSectionName     Section name to select
 '/
Declare Function orxConfig_SelectSection(ByVal _zSectionName As Const orxSTRING) As orxSTATUS

/'* Sets a section's parent
 * @param[in] _zSectionName     Concerned section, if the section doesn't exist, it will be created
 * @param[in] _zParentName      Parent section's name, if the section doesn't exist, it will be created, if orxNULL is provided, the former parent will be erased
 '/
Declare Function orxConfig_SetParent(ByVal _zSectionName As Const orxSTRING, _
                                     ByVal _zParentName As Const orxSTRING _
																		) As orxSTATUS

/'* Gets a section's parent
 * @param[in] _zSectionName     Concerned section, if the section doesn't exist, it will be created
 * @return Section's parent name / orxNULL
 '/
Declare Function orxConfig_GetParent(ByVal _zSectionName As Const orxSTRING) As Const orxSTRING

/'* Gets current working section
 * @return Current selected section
 '/
Declare Function orxConfig_GetCurrentSection() As Const orxString

/'* Pushes a section (storing the current one on section stack)
 * @param[in] _zSectionName     Section name to push
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_PushSection(ByVal _zSectionName As Const orxSTRING) As orxSTATUS

/'* Pops last section from section stack
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_PopSection() As orxSTATUS

/'* Has section for the given section name?
 * @param[in] _zSectionName     Section name
 * @return orxTRUE / orxFALSE
 '/
Declare Function orxConfig_HasSection(ByVal _zSectionName As Const orxSTRING ) As orxBOOL

/'* Protects/unprotects a section from deletion (content might still be changed or deleted, but the section itself will resist delete/clear calls)
 * @param[in] _zSectionName     Section name to protect
 * @param[in] _bProtect         orxTRUE for protecting the section, orxFALSE to remove the protection
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_ProtectSection(ByVal _zSectionName As Const orxSTRING, _
                                          ByVal _bProtect As orxBOOL _
																				 ) As orxSTATUS

/'* Clears all config info
* @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_Clear() As orxSTATUS

/'* Clears section
 * @param[in] _zSectionName     Section name to clear
 '/
Declare Function orxConfig_ClearSection(ByVal _zSectionName As Const orxSTRING) As orxSTATUS

/'* Clears a value from current selected section
 * @param[in] _zKey             Key name
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_ClearValue(ByVal _zKey As Const orxSTRING) As orxSTATUS


/'* Has specified value for the given key?
 * @param[in] _zKey             Key name
 * @return orxTRUE / orxFALSE
 '/
Declare Function orxConfig_HasValue(ByVal _zKey As Const orxSTRING) As orxBOOL

/'* Reads a signed integer value from config (will take a random value if a list is provided for this key)
 * @param[in] _zKey             Key name
 * @return The value
 '/
Declare Function orxConfig_GetS32(ByVal _zKey As Const orxSTRING) As orxS32

/'* Reads an unsigned integer value from config (will take a random value if a list is provided for this key)
 * @param[in] _zKey             Key name
 * @return The value
 '/
Declare Function orxConfig_GetU32(ByVal _zKey As Const orxSTRING) As orxU32

/'* Reads a float value from config (will take a random value if a list is provided for this key)
 * @param[in] _zKey             Key name
 * @return The value
 '/
Declare Function orxConfig_GetFloat(ByVal _zKey As Const orxSTRING) As orxFLOAT

/'* Reads a string value from config (will take a random value if a list is provided for this key)
 * @param[in] _zKey             Key name
 * @return The value
 '/
Declare Function orxConfig_GetString(ByVal _zKey As Const orxSTRING) As Const orxSTRING

/'* Reads a boolean value from config (will take a random value if a list is provided for this key)
 * @param[in] _zKey             Key name
 * @return The value
 '/
Declare Function orxConfig_GetBool(ByVal _zKey As Const orxSTRING) As orxBOOL

/'* Reads a vector value from config (will take a random value if a list is provided for this key)
 * @param[in]   _zKey             Key name
 * @param[out]  _pvVector         Storage for vector value
 * @return The value
 '/
Declare Function orxConfig_GetVector(ByVal _zKey As Const orxSTRING, _
                                     ByVal _pvVector As orxVECTOR Ptr _
																		) As orxVECTOR Ptr


/'* Writes a signed integer value to config
 * @param[in] _zKey             Key name
 * @param[in] _s32Value         Value
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_SetS32(ByVal _zKey As Const orxSTRING, _
                                  ByVal _s32Value As orxS32 _
																	) As orxSTATUS

/'* Writes an unsigned integer value to config
 * @param[in] _zKey             Key name
 * @param[in] _u32Value         Value
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_SetU32(ByVal _zKey As Const orxSTRING, _
                                                       ByVal _u32Value As orxU32 _
																                       ) As orxSTATUS 

/'* Writes a float value to config
 * @param[in] _zKey             Key name
 * @param[in] _fValue           Value
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_SetFloat(ByVal _zKey As Const orxSTRING, _
                                                        ByVal _fValue As orxFLOAT _
																	                      ) As orxSTATUS

/'* Writes a string value to config
 * @param[in] _zKey             Key name
 * @param[in] _zValue           Value
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_SetString(ByVal _zKey As Const orxSTRING, _
                                                          ByVal _zValue As Const orxSTRING _
																		                     ) As orxSTATUS

/'* Writes a boolean value to config
 * @param[in] _zKey             Key name
 * @param[in] _bValue           Value
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_SetBool(ByVal _zKey As Const orxSTRING, _
                                                        ByVal _bValue As orxBOOL _
																	                     ) As orxSTATUS 

/'* Writes a vector value to config
 * @param[in] _zKey             Key name
 * @param[in] _pvValue          Value
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_SetVector(ByVal _zKey As Const orxSTRING, _
                                                           ByVal _pvValue As Const orxVECTOR Ptr _
																		                      ) As orxSTATUS 


/'* Is value a list for the given key?
 * @param[in] _zKey             Key name
 * @return orxTRUE / orxFALSE
 '/
Declare Function orxConfig_IsList(ByVal _zKey As Const orxSTRING) As orxBOOL

/'* Gets list counter for a given key
 * @param[in] _zKey             Key name
 * @return List counter if it's a valid list, 0 otherwise
 '/
Declare Function orxConfig_GetListCounter(ByVal _zKey As Const orxSTRING) As orxS32

/'* Reads a signed integer value from config list
 * @param[in] _zKey             Key name
 * @param[in] _s32ListIndex     Index of desired item in list / -1 for random
 * @return The value
 '/
Declare Function orxConfig_GetListS32(ByVal _zKey As Const orxSTRING, _
                                                             ByVal _s32ListIndex As orxS32 _
																		                        ) As orxS32

/'* Reads an unsigned integer value from config list
 * @param[in] _zKey             Key name
 * @param[in] _s32ListIndex     Index of desired item in list / -1 for random
 * @return The value
 '/
Declare Function orxConfig_GetListU32(ByVal _zKey As Const orxSTRING, _
                                                            ByVal _s32ListIndex As orxS32 _
																		                         ) As orxU32

/'* Reads a float value from config list
 * @param[in] _zKey             Key name
 * @param[in] _s32ListIndex     Index of desired item in list / -1 for random
 * @return The value
 '/
Declare Function orxConfig_GetListFloat(ByVal _zKey As Const orxSTRING, _
                                                              ByVal _s32ListIndex As orxS32 _
																			                       ) As orxFLOAT

/'* Reads a string value from config list
 * @param[in] _zKey             Key name
 * @param[in] _s32ListIndex     Index of desired item in list / -1 for random
 * @return The value
 '/
Declare Function orxConfig_GetListString(ByVal _zKey As Const orxSTRING, _
                                                                ByVal _s32ListIndex As orxS32 _
																				                       ) As Const orxSTRING

/'* Reads a boolean value from config list
 * @param[in] _zKey             Key name
 * @param[in] _s32ListIndex     Index of desired item in list / -1 for random
 * @return The value
 '/
Declare Function orxConfig_GetListBool(ByVal _zKey As Const orxSTRING,_
                                                             ByVal _s32ListIndex As orxS32 _
																			                      ) As orxBOOL

/'* Reads a vector value from config list
 * @param[in]   _zKey             Key name
 * @param[in]   _s32ListIndex     Index of desired item in list / -1 for random
 * @param[out]  _pvVector         Storage for vector value
 * @return The value
 '/
Declare Function orxConfig_GetListVector(ByVal _zKey As Const orxSTRING, _
                                                                ByVal _s32ListIndex As orxS32, _
																				                        ByVal _pvVector As orxVECTOR Ptr _
																				                        ) As orxVECTOR Ptr

/'* Writes a list of string values to config
 * @param[in] _zKey             Key name
 * @param[in] _azValue          Values
 * @param[in] _u32Number        Number of values
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxConfig_SetStringList(ByVal _zKey As Const orxSTRING, _
                                                               _azValue() As Const orxSTRING, _
																				                       ByVal _u32Number As orxU32 _
																				                        ) As orxSTATUS

/'* Gets key counter for the current section
 * @return Key counter the current section if valid, 0 otherwise
 '/
Declare Function orxConfig_GetKeyCounter() As orxS32

/'* Gets key for the current section at the given index
 * @param[in] _s32KeyIndex      Index of the desired key
 * @return orxSTRING if exist, orxSTRING_EMPTY otherwise
 '/
Declare Function orxConfig_GetKey(ByVal _s32KeyIndex As orxS32) As Const orxSTRING

End Extern

#endif /'_orxCONFIG_BI'/

/'* @} '/
