/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxEvent.h
 * @date 20/05/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxEvent
 * 
 * Event module
 * Module that handles internal events
 * @{
 '/


#ifndef _orxEVENT_BI
#define _orxEVENT_BI

#include "orxInclude.bi"


/'* Helper defines
 '/
#macro orxEVENT_INIT_(EVENT, tTYPE, ID, SENDER, RECIPIENT, PAYLOAD)
Do
  EVENT.eType       = Cast(orxEVENT_TYPE,tTYPE)
  EVENT.eID         = Cast(orxENUM,ID)
  EVENT.hSender     = Cast(orxHANDLE,SENDER)
  EVENT.hRecipient  = Cast(orxHANDLE,RECIPIENT)
  EVENT.pstPayload  = Cast(Any Ptr,PAYLOAD)
Loop While(0)
#endmacro

#macro orxEVENT_SEND_(tTYPE, ID, SENDER, RECIPIENT, PAYLOAD)
Do
  Dim As orxEVENT stEvent
  orxEVENT_INIT(stEvent, tTYPE, ID, SENDER, RECIPIENT, PAYLOAD)
  orxEvent_Send(@stEvent)
Loop While(0)
#endmacro

/'* Event type enum
 '/
Enum orxEVENT_TYPE
  orxEVENT_TYPE_ANIM = 0,
  orxEVENT_TYPE_CLOCK,
  orxEVENT_TYPE_FX,
  orxEVENT_TYPE_INPUT,
  orxEVENT_TYPE_JOYSTICK,
  orxEVENT_TYPE_KEYBOARD,
  orxEVENT_TYPE_LOCALE,
  orxEVENT_TYPE_MOUSE,
  orxEVENT_TYPE_OBJECT,
  orxEVENT_TYPE_RENDER,
  orxEVENT_TYPE_PHYSICS,
  orxEVENT_TYPE_SHADER,
  orxEVENT_TYPE_SOUND,
  orxEVENT_TYPE_SPAWNER,
  orxEVENT_TYPE_SYSTEM,

  orxEVENT_TYPE_CORE_NUMBER,

  orxEVENT_TYPE_FIRST_RESERVED = orxEVENT_TYPE_CORE_NUMBER,

  orxEVENT_TYPE_LAST_RESERVED = 127,

  orxEVENT_TYPE_USER_DEFINED,

  orxEVENT_TYPE_NONE = orxENUM_NONE,

End Enum


/'* Public event structure
 '/
Type __orxEVENT_t

  As orxEVENT_TYPE     eType                           /'*< Event type : 4 '/
  As orxENUM           eID                            /'*< Event ID : 8 '/
  As orxHANDLE         hSender                        /'*< Sender handle : 12 '/
  As orxHANDLE         hRecipient                    /'*< Recipient handle : 16 '/
  As Any Ptr           pstPayload                       /'*< Event payload : 20 '/

End Type

Type orxEVENT As __orxEVENT_t

Extern "Windows"

/'*
 * Event handler type / return orxSTATUS_FAILURE if events processing should be stopped for the current event, orxSTATUS_FAILURE otherwise
 '/
Type orxEVENT_HANDLER As Function(ByVal _pstEvent As Const orxEVENT Ptr) As orxSTATUS


/'* Event module setup
 '/
Declare Sub orxEvent_Setup()

/'* Initializes the event Module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxEvent_Init() As orxSTATUS

/'* Exits from the event Module
 '/
Declare Sub orxEvent_Exit()

/'* Adds an event handler
 * @param[in] _eEventType           Concerned type of event
 * @param[in] _pfnEventHandler      Event handler to add
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxEvent_AddHandler(ByVal _eEventType As orxEVENT_TYPE, _
                                                           ByVal _pfnEventHandler As orxEVENT_HANDLER _
																													 ) As orxSTATUS

/'* Removes an event handler
 * @param[in] _eEventType           Concerned type of event
 * @param[in] _pfnEventHandler      Event handler to remove
 * return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxEvent_RemoveHandler(ByVal _eEventType As orxEVENT_TYPE , _
                                                                 ByVal _pfnEventHandler As orxEVENT_HANDLER _
																														     ) As orxSTATUS

/'* Sends an event
 * @param[in] _pstEvent             Event to send
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxEvent_Send(ByVal _pstEvent As Const orxEVENT Ptr) As orxSTATUS

/'* Sends a simple event
 * @param[in] _eEventType           Event type
 * @param[in] _eEventID             Event ID
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxEvent_SendShort(ByVal _eEventType As orxEVENT_TYPE, _
                                                          ByVal _eEventID As orxENUM _
																												 ) As orxSTATUS
																												 
End Extern

#endif /'_orxEVENT_BI '/

/'* @} '/
