/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxLocale.h
 * @date 15/07/2009
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxLocale
 * 
 * Localization module
 * Module that handles localized strings
 *
 * @{
 '/


#ifndef __orxLOCALE_BI
#define __orxLOCALE_BI


#include "orxInclude.bi"


/'* Event enum
 '/
Enum orxLOCALE_EVENT

  orxLOCALE_EVENT_SELECT_LANGUAGE = 0,                  /'*< Event sent when selecting a language '/
  orxLOCALE_EVENT_SET_STRING,                           /'*< Event sent when setting a string '/

  orxLOCALE_EVENT_NUMBER,

  orxLOCALE_EVENT_NONE = orxENUM_NONE

End Enum


/'* Locale event payload
 '/
Type __orxLOCALE_EVENT_PAYLOAD_t

  As orxSTRING zLanguage                                  /'*< Current language : 4 '/
  As orxSTRING zStringKey                                 /'*< String key : 8 '/
  As orxSTRING zStringValue                               /'*< String value : 12 '/

End Type

Type orxLOCALE_EVENT_PAYLOAD As __orxLOCALE_EVENT_PAYLOAD_t

Extern "Windows"

/'* Locale module setup
 '/
Declare Sub orxLocale_Setup()

/'* Initializes the Locale Module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxLocale_Init() As orxSTATUS

/'* Exits from the Locale Module
 '/
Declare Sub orxLocale_Exit()


/'* Selects current working language
 * @param[in] _zLanguage        Language to select
 '/
Declare Function orxLocale_SelectLanguage(ByVal _zLanguage As Const orxSTRING) As orxSTATUS

/'* Gets current language
 * @return Current selected language
 '/
Declare Function orxLocale_GetCurrentLanguage() As Const orxSTRING

/'* Has given language? (if not correctly defined, false will be returned)
 * @param[in] _zLanguage        Concerned language
 * @return orxTRUE / orxFALSE
 '/
Declare Function orxLocale_HasLanguage(ByVal _zLanguage As Const orxSTRING) As orxBOOL

/'* Gets language counter
 * @return Number of languages defined
 '/
Declare Function orxLocale_GetLanguageCounter() As orxS32 

/'* Gets language at the given index
 * @param[in] _s32LanguageIndex Index of the desired language
 * @return orxSTRING if exist, orxSTRING_EMPTY otherwise
 '/
Declare Function orxLocale_GetLanguage(ByVal _s32LanguageIndex As orxS32) As Const orxSTRING


/'* Has string for the given key?
 * @param[in] _zKey             Key name
 * @return orxTRUE / orxFALSE
 '/
Declare Function orxLocale_HasString(ByVal _zKey As Const orxSTRING) As orxBOOL

/'* Reads a string in the current language for the given key
 * @param[in] _zKey             Key name
 * @return The value
 '/
Declare Function orxLocale_GetString(ByVal _zKey As Const orxSTRING) As orxSTRING

/'* Writes a string in the current language for the given key
 * @param[in] _zKey             Key name
 * @param[in] _zValue           Value
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxLocale_SetString(ByVal _zKey As Const orxSTRING, _
                                                           ByVal _zValue As Const orxSTRING _
                                                          ) As orxSTATUS

/'* Gets key counter for the current language
 * @return Key counter the current language if valid, 0 otherwise
 '/
Declare Function orxLocale_GetKeyCounter() As orxS32

/'* Gets key for the current language at the given index
 * @param[in] _s32KeyIndex      Index of the desired key
 * @return orxSTRING if exist, orxNULL otherwise
 '/
Declare Function orxLocale_GetKey(ByVal _s32KeyIndex As orxS32) As  Const orxSTRING

End Extern

#endif /'__orxLOCALE_H_'/

/'* @} '/
