/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxDisplay.h
 * @date 23/04/2003
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxDisplay
 * 
 * Display plugin module
 * Module that handles display
 *
 * @{
 '/


#ifndef _orxDISPLAY_BI
#define _orxDISPLAY_BI

#include "orxInclude.bi"
'#include "orx/plugin/orxPluginCore.bi"
#include "orx/math/orxVector.bi"
#include "orx/utils/orxString.bi"
#include "orx/utils/orxLinkList.bi"


/'* Misc defines
 '/
Type orxRGBA As orxU32
#define orx2RGBA(R, G, B, A)        ((((R) And &HFF) Shl 24) Or (((G) And &HFF) Shl 16) Or (((B) And &HFF) Shl 8) And ((A) And &HFF))
#define orxRGBA_R(RGBA)             Cast(orxU8,(((RGBA) Shr 24) And &HFF))
#define orxRGBA_G(RGBA)             Cast(orxU8,(((RGBA) Shr 16) And &HFF))
#define orxRGBA_B(RGBA)             Cast(orxU8,(((RGBA) Shr 8) And &HFF))
#define orxRGBA_A(RGBA)             Cast(orxU8,((RGBA) And &HFF))

#define orxCOLOR_NORMALIZER         (orx2F(1.0f / 255.0f))
#define orxCOLOR_DENORMALIZER       (orx2F(255.0f))


Type __orxBITMAP_t  As Any
Type orxBITMAP As __orxBITMAP_t

Type __orxDISPLAY_TEXT_t As Any
Type orxDISPLAY_TEXT As __orxDISPLAY_TEXT_t

/'* Transform structure
 '/
Type __orxDISPLAY_TRANSFORM_t

  As orxFLOAT  fSrcX
	As orxFLOAT  fSrcY
	As orxFLOAT fDstX
	As orxFLOAT fDstY
  As orxFLOAT  fRotation
  As orxFLOAT  fScaleX
  As orxFLOAT  fScaleY

End Type

Type orxDISPLAY_TRANSFORM As __orxDISPLAY_TRANSFORM_t

/'* Video mode structure
 '/
Type __orxDISPLAY_VIDEO_MODE_t

  As orxU32 u32Width
	As orxU32 u32Height
	As orxU32 u32Depth

End Type

Type orxDISPLAY_VIDEO_MODE As __orxDISPLAY_VIDEO_MODE_t

/'* Bitmap smoothing enum
 '/
Enum orxDISPLAY_SMOOTHING

  orxDISPLAY_SMOOTHING_DEFAULT = 0,
  orxDISPLAY_SMOOTHING_ON,
  orxDISPLAY_SMOOTHING_OFF,

  orxDISPLAY_SMOOTHING_NUMBER,

  orxDISPLAY_SMOOTHING_NONE = orxENUM_NONE

End Enum

/'* Bitmap blend enum
 '/
Enum orxDISPLAY_BLEND_MODE

  orxDISPLAY_BLEND_MODE_ALPHA = 0,
  orxDISPLAY_BLEND_MODE_MULTIPLY,
  orxDISPLAY_BLEND_MODE_ADD,

  orxDISPLAY_BLEND_MODE_NUMBER,

  orxDISPLAY_BLEND_MODE_NONE = orxENUM_NONE

End Enum


/'* Color structure
 '/
Type __orxCOLOR_t

  As orxVECTOR vRGB                       /'*< RGB components: 12 '/
  As orxFLOAT  fAlpha                     /'*< Alpha component: 16 '/

End Type

Type orxCOLOR As __orxCOLOR_t


#define orxDISPLAY_KZ_CONFIG_SECTION    "Display"
#define orxDISPLAY_KZ_CONFIG_WIDTH      "ScreenWidth"
#define orxDISPLAY_KZ_CONFIG_HEIGHT     "ScreenHeight"
#define orxDISPLAY_KZ_CONFIG_DEPTH      "ScreenDepth"
#define orxDISPLAY_KZ_CONFIG_FULLSCREEN "FullScreen"
#define orxDISPLAY_KZ_CONFIG_DECORATION "Decoration"
#define orxDISPLAY_KZ_CONFIG_FONT       "Font"
#define orxDISPLAY_KZ_CONFIG_TITLE      "Title"
#define orxDISPLAY_KZ_CONFIG_SMOOTH     "Smoothing"
#define orxDISPLAY_KZ_CONFIG_VSYNC      "VSync"


/'**************************************************************************
 * Functions directly implemented by orx core
 **************************************************************************'/

Extern "Windows"

/'* Display module setup
 '/
Declare Sub orxDisplay_Setup()

End Extern

/'* Sets all components from an orxRGBA
 * @param[in]   _pstColor       Concerned color
 * @param[in]   _stRGBA         RGBA values to set
 * @return      orxCOLOR
 '/
#macro orxColor_SetRGBA(_pstColor,_stRGBA,_result)

Scope
  Dim As orxVECTOR Ptr vTEMP
  _result = _pstColor

  /' Checks '/
  orxASSERT(_pstColor <> orxNULL)

  /' Stores RGB '/
  orxVector_Set(@(_pstColor->vRGB), orxCOLOR_NORMALIZER * orxU2F(orxRGBA_R(_stRGBA)), orxCOLOR_NORMALIZER * orxU2F(orxRGBA_G(_stRGBA)), orxCOLOR_NORMALIZER * orxU2F(orxRGBA_B(_stRGBA)),vTEMP)

  /' Stores alpha '/
  _pstColor->fAlpha = orxCOLOR_NORMALIZER * orxRGBA_A(_stRGBA)

  /' Done! '/
  

#endmacro

/'* Sets all components
 * @param[in]   _pstColor       Concerned color
 * @param[in]   _pvRGB          RGB components
 * @param[in]   _fAlpha         Normalized alpha component
 * @return      orxCOLOR
 '/
#macro orxColor_Set(_pstColor,_pvRGB,_fAlpha,_result)

Scope
   Dim As orxVECTOR Ptr vTEMP
  _result = _pstColor

  /' Checks '/
  orxASSERT(_pstColor <> orxNULL)

  /' Stores RGB '/
  orxVector_Copy(@(_pstColor->vRGB), _pvRGB,vTEMP)

  /' Stores alpha '/
  _pstColor->fAlpha = _fAlpha

  /' Done! '/
End Scope
#endmacro


/'* Sets RGB components
 * @param[in]   _pstColor       Concerned color
 * @param[in]   _pvRGB          RGB components
 * @return      orxCOLOR
 '/
#macro orxColor_SetRGB(_pstColor,_pvRGB,_result)
Scope
  Dim As orxVECTOR Ptr vTEMP
	_result = _pstColor

  /' Checks '/
  orxASSERT(_pstColor <> orxNULL)
  orxASSERT(_pvRGB <> orxNULL)

  /' Stores components '/
  orxVector_Copy(@(_pstColor->vRGB), _pvRGB,vTEMP)

  /' Done! '/
End Scope
#endmacro

/'* Sets alpha component
 * @param[in]   _pstColor       Concerned color
 * @param[in]   _fAlpha         Normalized alpha component
 * @return      orxCOLOR / orxNULL
 '/
#macro orxColor_SetAlpha(_pstColor,_fAlpha,_result)

  _result = _pstColor

  /' Checks '/
  orxASSERT(_pstColor <> orxNULL)

  /' Stores it '/
  _pstColor->fAlpha = _fAlpha

  /' Done! '/  

#endmacro

/'* Gets orxRGBA from an orxCOLOR
 * @param[in]   _pstColor       Concerned color
 * @return      orxRGBA
 '/
#macro orxColor_ToRGBA(_pstColor,_result)

  Scope
		
		Dim As orxRGBA   stResult
		Dim As orxVECTOR vColor
		Dim As orxFLOAT  fAlpha
    Dim As orxVECTOR Ptr vTEMP
		/' Checks '/
		orxASSERT(_pstColor <> orxNULL)

		/' Clamps RGB components '/
		orxVector_Clamp(@vColor, @(_pstColor->vRGB), @orxVECTOR_0, @orxVECTOR_WHITE,vTEMP)

		/' De-normalizes vector '/
		orxVector_Mulf(@vColor, @vColor, orxCOLOR_DENORMALIZER,vTEMP)

		/' Clamps alpha '/
		fAlpha = orxCLAMP(_pstColor->fAlpha, orxFLOAT_0, orxFLOAT_1)

		/' Updates result '/
		stResult = orx2RGBA(orxF2U(vColor.fR), orxF2U(vColor.fG), orxF2U(vColor.fB), orxF2U(orxCOLOR_DENORMALIZER * fAlpha))

		/' Done! '/
		_result = stResult
	End Scope
#endmacro

/'* Copies an orxCOLOR into another one
 * @param[in]   _pstDst         Destination color
 * @param[in]   _pstSrc         Source color
 * @return      orxCOLOR
 '/

#macro orxColor_Copy(_pstDst,_pstSrc,_result)

  /' Checks '/
  orxASSERT(_pstDst <> orxNULL)
  orxASSERT(_pstSrc <> orxNULL)

  /' Copies it '/
  orxMemory_Copy(_pstDst, _pstSrc, SizeOf(orxCOLOR))

  /' Done! '/
  _result = _pstDst

#endmacro


/'**************************************************************************
 * Functions extended by plugins
 **************************************************************************'/
Extern "Windows"

/'* Inits the display module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_Init() As orxSTATUS

/'* Exits from the display module
 '/
Declare Sub orxDisplay_Exit()


/'* Swaps/flips bufers (display on screen the current frame)
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function  orxDisplay_Swap() As orxSTATUS


/'* Creates an empty text
 * @return orxDISPLAY_TEXT / orxNULL
 '/
Declare Function orxDisplay_CreateText() As orxDISPLAY_TEXT Ptr

/'* Deletes a text
 * @param[in]   _pstText                              Concerned text
 '/
Declare Sub orxDisplay_DeleteText(ByVal _pstText As orxDISPLAY_TEXT Ptr)

/'* Transforms a text (on a bitmap)
 * @param[in]   _pstDst                               Destination bitmap
 * @param[in]   _pstText                              Text to transform (display)
 * @param[in]   _pstTransform                         Transformation info (positions, scale, rotation, ...)
 * @param[in]   _stColor                              Color to use
 * @param[in]   _eBlendMode                           Blend mode
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_TransformText(Byval _pstDst As orxBITMAP Ptr, _
																											ByVal _pstText As Const orxDISPLAY_TEXT Ptr, _
																											ByVal _pstTransform As const orxDISPLAY_TRANSFORM Ptr, _
																											ByVal _stColor As orxRGBA, _
																											ByVal _eBlendMode As orxDISPLAY_BLEND_MODE _
																										 ) As orxSTATUS

/'* Sets a text string
 * @param[in]   _pstText                              Concerned text
 * @param[in]   _zString                              String to set
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_SetTextString(ByVal _pstText As orxDISPLAY_TEXT Ptr, _
																																ByVal _zString As Const orxSTRING _
																															 ) As orxSTATUS

/'* Sets a text font
 * @param[in]   _pstText                              Concerned text
 * @param[in]   _zFont                                Font to set
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_SetTextFont(ByVal _pstText As orxDISPLAY_TEXT Ptr, _
																									  ByVal _zFont As Const orxSTRING _
																									 ) As orxSTATUS

/'* Gets a text string
 * @param[in]   _pstText                              Concerned text
 * @return orxSTRING / orxNULL
 '/
Declare Function orxDisplay_GetTextString(ByVal _pstText As Const orxDISPLAY_TEXT Ptr) As Const orxSTRING 

/'* Gets a text font
 * @param[in]   _pstText                              Concerned text
 * @return orxSTRING / orxNULL
 '/
Declare Function orxDisplay_GetTextFont(ByVal _pstText As Const orxDISPLAY_TEXT Ptr) As Const orxSTRING

/'* Gets a text size
 * @param[in]   _pstText                              Concerned text
 * @param[out]  _pfWidth                              Text's width
 * @param[out]  _pfHeight                             Text's height
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_GetTextSize(ByVal _pstText As Const orxDISPLAY_TEXT Ptr, _
																										ByVal _pfWidth As orxFLOAT Ptr, _
																										ByVal _pfHeight As orxFLOAT Ptr _
																										) As orxSTATUS 


/'* Prints a string
 * @param[in]   _pstBitmap                            Concerned bitmap
 * @param[in]   _zString                              String to display
 * @param[in]   _pstTransform                         Transformation info (positions, scale, rotation, ...)
 * @param[in]   _stColor                              Color to use for the text
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_PrintString(ByVal _pstBitmap As Const orxBITMAP Ptr, _
																										ByVal _zString As Const orxSTRING,  _
																										ByVal _pstTransformConst As Const orxDISPLAY_TRANSFORM Ptr, _
																										ByVal _stColor As orxRGBA _
																									 ) As orxSTATUS


/'* Creates a bitmap
 * @param[in]   _u32Width                             Bitmap width
 * @param[in]   _u32Height                            Bitmap height
 * @return orxBITMAP / orxNULL
 '/
Declare Function orxDisplay_CreateBitmap(ByVal _u32Width As orxU32, _
																										 ByVal _u32Height As orxU32 _
																										) As orxBITMAP Ptr

/'* Deletes a bitmap
 * @param[in]   _pstBitmap                            Concerned bitmap
 '/
Declare Sub orxDisplay_DeleteBitmap(ByVal _pstBitmap As orxBITMAP Ptr)


/'* Gets screen bitmap
 * @return orxBITMAP / orxNULL
 '/
Declare Function orxDisplay_GetScreenBitmap() As orxBITMAP Ptr

/'* Gets screen size
 * @param[out]   _pfWidth                             Screen width
 * @param[out]   _pfHeight                            Screen height
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_GetScreenSize(ByVal _pfWidth As orxFLOAT Ptr, _
																											ByVal _pfHeight As orxFLOAT Ptr) As orxSTATUS


/'* Clears a bitmap
 * @param[in]   _pstBitmap                            Concerned bitmap
 * @param[in]   _stColor                              Color to clear the bitmap with
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_ClearBitmap(ByVal _pstBitmap As orxBITMAP Ptr, _
																										ByVal _stColor As orxRGBA _
																									 ) As orxSTATUS

/'* Transforms (and blits onto another) a bitmap
 * @param[in]   _pstDst                               Bitmap where to blit the result, can be screen
 * @param[in]   _pstSrc                               Bitmap to transform and draw
 * @param[in]   _pstTransform                         Transformation info (positions, scale, rotation, ...)
 * @param[in]   _eSmoothing                           Bitmap smoothing type
 * @param[in]   _eBlendMode                           Blend mode
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_TransformBitmap(ByVal _pstDst As orxBITMAP Ptr, _
																												ByVal _pstSrc As Const orxBITMAP Ptr, _
																												ByVal _pstTransform As Const orxDISPLAY_TRANSFORM Ptr, _
																												ByVal _eSmoothing As orxDISPLAY_SMOOTHING, _
																												ByVal _eBlendMode As orxDISPLAY_BLEND_MODE  _
																												) As orxSTATUS


/'* Sets a bitmap color key (used with non alpha transparency)
 * @param[in]   _pstBitmap                            Concerned bitmap
 * @param[in]   _stColor                              Color to use as transparent one
 * @param[in]   _bEnable                              Enable / disable transparence for this color
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_SetBitmapColorKey(ByVal _pstBitmap As orxBITMAP Ptr, _
																													ByVal _stColor As orxRGBA, _
																													ByVal _bEnable As orxBOOL _
																												 ) As orxSTATUS

/'* Sets a bitmap color (lighting/hue)
 * @param[in]   _pstBitmap                            Concerned bitmap
 * @param[in]   _stColor                              Color to apply on the bitmap
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_SetBitmapColor(ByVal _pstBitmap As orxBITMAP Ptr, _
																											 ByVal _stColor As orxRGBA _																											 
																											 ) As orxSTATUS

/'* Sets a bitmap clipping for blitting (both as source and destination)
 * @param[in]   _pstBitmap                            Concerned bitmap
 * @param[in]   _u32TLX                               Top left X coord in pixels
 * @param[in]   _u32TLY                               Top left Y coord in pixels
 * @param[in]   _u32BRX                               Bottom right X coord in pixels
 * @param[in]   _u32BRY                               Bottom right Y coord in pixels
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_SetBitmapClipping(ByVal _pstBitmap As orxBITMAP Ptr,_
																													ByVal _u32TLX As orxU32, _
																													ByVal _u32TLY As orxU32, _
																													ByVal _u32BRX As orxU32, _
																													ByVal _u32BRY As orxU32 _ 
																													) As orxSTATUS


/'* Blits a bitmap (no transformation)
 * @param[in]   _pstDst                               Bitmap where to blit
 * @param[in]   _pstSrc                               Bitmap to blit (will begin at top left corner)
 * @param[in]   _fPosX                                X-axis value of the position where to blit the source bitmap
 * @param[in]   _fPosY                                Y-axis value of the position where to blit the source bitmap
 * @param[in]   _eBlendMode                           Blend mode
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_BlitBitmap(ByVal _pstDst As orxBITMAP Ptr, _
																									 ByVal _pstSrc As Const orxBITMAP Ptr, _
																									 ByVal _fPosX As orxFLOAT, _
																									 ByVal _fPosY As orxFLOAT, _
																									 ByVal _eBlendMode As orxDISPLAY_BLEND_MODE _
																									) As orxSTATUS 


/'* Saves a bitmap to file
 * @param[in]   _pstBitmap                            Concerned bitmap
 * @param[in]   _zFileName                            Name of the file where to store the bitmap
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_SaveBitmap(ByVal _pstBitmap As Const orxBITMAP Ptr, _
																									              ByVal _zFileName As Const orxSTRING _
																															 ) As orxSTATUS

/'* Loads a bitmap from file
 * @param[in]   _zFileName                            Name of the file to load
 * @return orxBITMAP * / orxNULL
 '/
Declare Function orxDisplay_LoadBitmap(ByVal _zFileName As Const orxSTRING ) As orxBITMAP Ptr


/'* Gets a bitmap size
 * @param[in]   _pstBitmap                            Concerned bitmap
 * @param[out]   _pfWidth                             Bitmap width
 * @param[out]   _pfHeight                            Bitmap height
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_GetBitmapSize(ByVal _pstBitmap As Const orxBITMAP Ptr, _
																																                      ByVal _pfWidth As orxFLOAT Ptr, _
																																                      ByVal _pfHeight As orxFLOAT Ptr _
																																											) As orxSTATUS

/'* Gets bitmap color (lighting/hue)
 * @param[in]   _pstBitmap                            Concerned bitmap
 * @return orxRGBA
 '/
Declare Function orxDisplay_GetBitmapColor(ByVal _pstBitmap As Const orxBITMAP Ptr) As orxRGBA


/'* Creates (compiles) a shader
 * @param[in]   _zCode                                Shader code to compile
 * @param[in]   _pstParamList                         Shader parameters (should be a link list of orxSHADER_PARAM)
 * @return orxHANDLE of the compiled shader is successful, orxHANDLE_UNDEFINED otherwise
 '/
Declare Function orxDisplay_CreateShader(ByVal _zCode As Const orxSTRING , _
																																	ByVal _pstParamList As Const orxLINKLIST Ptr _
																																	) As orxHANDLE

/'* Deletes a compiled shader
 * @param[in]   _hShader                              Shader to delete
 '/
Declare Sub orxDisplay_DeleteShader(ByVal _hShader As orxHANDLE)

/'* Renders a shader
 * @param[in]   _hShader                              Shader to render
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_RenderShader(ByVal _hShader As Const orxHANDLE ) As orxSTATUS

/'* Sets a shader parameter (orxBITMAP)
 * @param[in]   _hShader                              Concerned shader
 * @param[in]   _zParam                               Parameter to set
 * @param[in]   _pstValue                             Value (orxBITMAP) for this parameter
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_SetShaderBitmap(ByVal _hShader As orxHANDLE, _
																						                           ByVal _zParam As Const orxSTRING, _
																						                           ByVal _pstValue As orxBITMAP Ptr _
																																			 ) As orxSTATUS

/'* Sets a shader parameter (orxFLOAT)
 * @param[in]   _hShader                              Concerned shader
 * @param[in]   _zParam                               Parameter to set
 * @param[in]   _fValue                               Value (orxFLOAT) for this parameter
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_SetShaderFloat(ByVal _hShader As orxHANDLE, _
																											              ByVal _zParam As Const orxSTRING , _
																											              ByVal _fValue As orxFLOAT _
																											              ) As orxSTATUS

/'* Sets a shader parameter (orxVECTOR)
 * @param[in]   _hShader                              Concerned shader
 * @param[in]   _zParam                               Parameter to set
 * @param[in]   _pvValue                              Value (orxVECTOR) for this parameter
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_SetShaderVector(ByVal _hShader As orxHANDLE, _
																																	     ByVal _zParam As Const orxSTRING , _
																																	     ByVal _pvValue As Const orxVECTOR Ptr _
																																       ) As orxSTATUS


/'* Gets application input manager (if embedded with display)
 * @return orxHANDLE / orxHANDLE_UNDEFINED
 '/
Declare Function orxDisplay_GetApplicationInput() As orxHANDLE


/'* Enables / disables vertical synchro
 * @param[in]   _bEnable                              Enable / disable
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_EnableVSync(ByVal _bEnable As orxBOOL) As orxSTATUS

/'* Is vertical synchro enabled?
 * @return orxTRUE if enabled, orxFALSE otherwise
 '/
Declare Function orxDisplay_IsVSyncEnabled() As orxBOOL


/'* Sets full screen mode
 * @param[in]   _bFullScreen                          orxTRUE / orxFALSE
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_SetFullScreen(ByVal _bFullScreen As orxBOOL ) As orxSTATUS 

/'* Is in full screen mode?
 * @return orxTRUE if full screen, orxFALSE otherwise
 '/
Declare Function orxDisplay_IsFullScreen() As orxBOOL


/'* Gets available video mode counter
 * @return Available video mode counter
 '/
Declare Function orxDisplay_GetVideoModeCounter() As orxU32 

/'* Gets an available video mode
 * @param[in]   _u32Index                             Video mode index, must be lesser than orxDisplay_GetVideoModeCounter()
 * @param[out]  _pstVideoMode                         Storage for the video mode
 * @return orxDISPLAY_VIDEO_MODE / orxNULL if invalid
 '/
Declare Function orxDisplay_GetVideoMode(ByVal _u32Index As orxU32, _
																															     ByVal _pstVideoMode As orxDISPLAY_VIDEO_MODE Ptr _
																															     ) As orxDISPLAY_VIDEO_MODE Ptr

/'* Gets an available video mode
 * @param[in]  _pstVideoMode                          Video mode to set
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxDisplay_SetVideoMode(ByVal _pstVideoMode As Const orxDISPLAY_VIDEO_MODE Ptr) As orxSTATUS

/'* Is video mode available
 * @param[in]  _pstVideoMode                          Video mode to test
 * @return orxTRUE is available, orxFALSE otherwise
 '/
Declare Function orxDisplay_IsVideoModeAvailable(ByVal _pstVideoMode As Const orxDISPLAY_VIDEO_MODE Ptr) As orxBOOL

End Extern

#endif /' _orxDISPLAY_BI '/

/'* @} '/
