/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxGraphic.h
 * @date 08/12/2003
 * @author iarwain@orx-project.org
 *
 * @todo
 * - Add 3D support
 '/

/'*
 * @addtogroup orxGraphic
 *
 * Graphic Module.
 * Allows to creates and handle 2D/3D Graphics.
 * They are used as texture/animation container, with associated properties.
 * 2D Graphics are used by objects.
 * They thus can be referenced by objects as structures.
 *
 * @{
 '/


#ifndef _orxGRAPHIC_BI
#define _orxGRAPHIC_BI

#include "orxInclude.bi"
#include "orx/object/orxStructure.bi"
#include "orx/display/orxDisplay.bi"


/'* Graphic flags
 '/
#define orxGRAPHIC_KU32_FLAG_NONE             &H00000000  /'*< No flags '/

#define orxGRAPHIC_KU32_FLAG_2D               &H00000001  /'*< 2D type graphic flag  '/
#define orxGRAPHIC_KU32_FLAG_TEXT             &H00000002  /'*< Text type graphic flag  '/

#define orxGRAPHIC_KU32_MASK_TYPE             &H00000003  /'*< Type mask '/

#define orxGRAPHIC_KU32_FLAG_FLIP_X           &H00000004  /'*< X axis flipping flag '/
#define orxGRAPHIC_KU32_FLAG_FLIP_Y           &H00000008  /'*< Y axis flipping flag '/
#define orxGRAPHIC_KU32_MASK_FLIP_BOTH        &H0000000C  /'*< Both axis flipping mask '/

#define orxGRAPHIC_KU32_FLAG_ALIGN_CENTER     &H00000000  /'*< Center vertical / horizontal alignment '/
#define orxGRAPHIC_KU32_FLAG_ALIGN_LEFT       &H00000010  /'*< Left horizontal alignment '/
#define orxGRAPHIC_KU32_FLAG_ALIGN_RIGHT      &H00000020  /'*< Right horizontal alignment '/
#define orxGRAPHIC_KU32_FLAG_ALIGN_TOP        &H00000040  /'*< Top vertical alignment '/
#define orxGRAPHIC_KU32_FLAG_ALIGN_BOTTOM     &H00000080  /'*< Bottom vertical alignment '/
#define orxGRAPHIC_KU32_FLAG_ALIGN_TRUNCATE   &H00000100  /'*< Truncate alignment value '/
#define orxGRAPHIC_KU32_FLAG_ALIGN_ROUND      &H00000200  /'*< Round alignment value '/

#define orxGRAPHIC_KU32_MASK_USER_ALL         &H00000FFF  /'*< User all ID mask '/


/'* Internal Graphic structure
 '/
Type __orxGRAPHIC As Any
Type orxGRAPHIC As __orxGRAPHIC_t

Extern "Windows"

/'* Graphic module setup
 '/
Declare Sub orxGraphic_Setup()

/'* Inits the Graphic module
 '/
Declare Function orxGraphic_Init() As orxSTATUS

/'* Exits from the Graphic module
 '/
Declare Sub orxGraphic_Exit()


/'* Creates an empty graphic
 * @return      Created orxGRAPHIC / orxNULL
 '/
Declare Function orxGraphic_Create() As orxGRAPHIC Ptr

/'* Creates a graphic from config
 * @param[in]   _zConfigID            Config ID
 * @ return orxGRAPHIC / orxNULL
 '/
Declare Function orxGraphic_CreateFromConfig(ByVal _zConfigID As Const orxSTRING) As orxGRAPHIC Ptr

/'* Deletes a graphic
 * @param[in]   _pstGraphic       Graphic to delete
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_Delete(ByVal _pstGraphic As orxGRAPHIC Ptr) As orxSTATUS


/'* Sets graphic data
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[in]   _pstData        Data structure to set / orxNULL
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_SetData(ByVal _pstGraphic As orxGRAPHIC Ptr,ByVal _pstData As orxSTRUCTURE Ptr) As orxSTATUS

/'* Gets graphic data
 * @param[in]   _pstGraphic     Concerned graphic
 * @return      OrxSTRUCTURE / orxNULL
 '/
Declare Function orxGraphic_GetData(ByVal _pstGraphic As Const orxGRAPHIC Ptr) As orxSTRUCTURE Ptr


/'* Sets graphic flipping
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[in]   _bFlipX         Flip it on X axis
 * @param[in]   _bFlipY         Flip it on Y axis
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_SetFlip(ByVal _pstGraphic As orxGRAPHIC Ptr,ByVal _bFlipX As orxBOOL,ByVal _bFlipY As orxBOOL) As orxSTATUS

/'* Gets graphic flipping
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[in]   _pbFlipX        X axis flipping
 * @param[in]   _pbFlipY        Y axis flipping
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_GetFlip(ByVal _pstGraphic As Const orxGRAPHIC Ptr,ByVal _pbFlipX As orxBOOL Ptr,ByVal _pbFlipY As orxBOOL Ptr) As orxSTATUS


/'* Sets graphic pivot
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[in]   _pvPivot        Pivot to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_SetPivot(ByVal _pstGraphic As orxGRAPHIC Ptr,ByVal _pvPivot As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets relative graphic pivot
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[in]   _u32AlignFlags  Alignment flags
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_SetRelativePivot(ByVal _pstGraphic As orxGRAPHIC Ptr,ByVal _u32AlignFlags As orxU32) As orxSTATUS

/'* Gets graphic pivot
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[out]  _pvPivot        Graphic pivot
 * @return      orxPIVOT / orxNULL
 '/
Declare Function orxGraphic_GetPivot(ByVal _pstGraphic As Const orxGRAPHIC Ptr,ByVal _pvPivot As orxVECTOR Ptr) As orxVECTOR Ptr


/'* Sets graphic size
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[in]   _pvSize         Size to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_SetSize(ByVal _pstGraphic As orxGRAPHIC Ptr,ByVal _pvSize As Const orxVECTOR Ptr) As orxSTATUS

/'* Gets graphic size
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[out]  _pvSize         Object's size
 * @return      orxVECTOR / orxNULL
 '/
Declare Function orxGraphic_GetSize(ByVal _pstGraphic As Const orxGRAPHIC Ptr,ByVal _pvSize As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Sets graphic color
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[in]   _pstColor       Color to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_SetColor(ByVal _pstGraphic As orxGRAPHIC Ptr,ByVal _pstColor As Const orxCOLOR Ptr) As orxSTATUS

/'* Sets graphic repeat (wrap) value
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[in]   _fRepeatX       X-axis repeat value
 * @param[in]   _fRepeatY       Y-axis repeat value
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_SetRepeat(ByVal _pstGraphic As orxGRAPHIC Ptr,ByVal _fRepeatX As orxFLOAT,ByVal _fRepeatY As orxFLOAT) As orxSTATUS

/'* Clears graphic color
 * @param[in]   _pstGraphic     Concerned graphic
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_ClearColor(ByVal _pstGraphic As orxGRAPHIC Ptr) As orxSTATUS

/'* Graphic has color accessor
 * @param[in]   _pstGraphic     Concerned graphic
 * @return      orxTRUE / orxFALSE
 '/
Declare Function orxGraphic_HasColor(ByVal _pstGraphic As Const orxGRAPHIC Ptr) As orxBOOL

/'* Gets graphic color
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[out]  _pstColor       Object's color
 * @return      orxCOLOR / orxNULL
 '/
Declare Function orxGraphic_GetColor(ByVal _pstGraphic As Const orxGRAPHIC Ptr,ByVal _pstColor As orxCOLOR Ptr) As orxCOLOR Ptr

/'* Gets graphic repeat (wrap) values
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[out]  _pfRepeatX      X-axis repeat value
 * @param[out]  _pfRepeatY      Y-axis repeat value
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_GetRepeat(ByVal _pstGraphic As Const orxGRAPHIC Ptr,ByVal _pfRepeatX As orxFLOAT Ptr,ByVal _pfRepeatY As orxFLOAT Ptr) As orxSTATUS

/'* Sets graphic origin
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[in]   _pvOrigin       Origin coordinates
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_SetOrigin(ByVal _pstGraphic As orxGRAPHIC Ptr,ByVal _pvOrigin As Const orxVECTOR Ptr) As orxSTATUS

/'* Gets graphic origin
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[out]  _pvOrigin       Origin coordinates
 * @return      Origin coordinates
 '/
Declare Function orxGraphic_GetOrigin(ByVal _pstGraphic As Const orxGRAPHIC Ptr,ByVal _pvOrigin As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Updates graphic size (recompute)
 * @param[in]   _pstGraphic     Concerned graphic
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_UpdateSize(ByVal _pstGraphic As orxGRAPHIC Ptr) As orxSTATUS

/'* Sets graphic smoothing
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[in]   _eSmoothing     Smoothing type (enabled, default or none)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_SetSmoothing(ByVal _pstGraphic As orxGRAPHIC Ptr,ByVal _eSmoothing As orxDISPLAY_SMOOTHING) As orxSTATUS

/'* Gets graphic smoothing
 * @param[in]   _pstGraphic     Concerned graphic
 * @return Smoothing type (enabled, default or none)
 '/
Declare Function orxGraphic_GetSmoothing(ByVal _pstGraphic As Const orxGRAPHIC Ptr) As orxDISPLAY_SMOOTHING


/'* Sets object blend mode
 * @param[in]   _pstGraphic     Concerned graphic
 * @param[in]   _eBlendMode     Blend mode (alpha, multiply, add or none)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxGraphic_SetBlendMode(ByVal _pstGraphic As orxGRAPHIC Ptr,ByVal _eBlendMode As orxDISPLAY_BLEND_MODE) As orxSTATUS

/'* Gets graphic blend mode
 * @param[in]   _pstGraphic     Concerned graphic
 * @return Blend mode (alpha, multiply, add or none)
 '/
Declare Function orxGraphic_GetBlendMode(ByVal _pstGraphic As Const orxGRAPHIC Ptr) As orxDISPLAY_BLEND_MODE

End Extern

#endif /' _orxGRAPHIC_BI '/

/'* @} '/
