/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxText.h
 * @date 02/12/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxText
 *
 * Text module
 * Module that handles texts
 *
 * @{
 '/


#ifndef _orxTEXT_BI
#define _orxTEXT_BI

#include "orxInclude.bi"
#include "orx/display/orxDisplay.bi"


/'* Internal text structure '/
Type __orxTEXT_t As Any
Type orxTEXT As __orxTEXT_t

Extern "Windows"

/'* Setups the text module
 '/
Declare Sub orxText_Setup()

/'* Inits the text module
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxText_Init() As orxSTATUS

/'* Exits from the text module
 '/
Declare Sub orxText_Exit()


/'* Creates an empty text
 * @return      orxTEXT / orxNULL
 '/
Declare Function orxText_Create() As orxTEXT Ptr

/'* Creates a text from config
 * @param[in]   _zConfigID    Config ID
 * @return      orxTEXT / orxNULL
 '/
Declare Function orxText_CreateFromConfig(ByVal _zConfigID As Const orxSTRING) As orxTEXT Ptr

/'* Deletes a text
 * @param[in]   _pstText      Concerned text
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxText_Delete(ByVal _pstText As orxTEXT Ptr) As orxSTATUS


/'* Gets text size
 * @param[in]   _pstText      Concerned text
 * @param[out]  _pfWidth      Text's width
 * @param[out]  _pfHeight     Text's height
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxText_GetSize(ByVal _pstText As Const orxTEXT Ptr,ByVal _pfWidth As orxFLOAT Ptr,ByVal _pfHeight As orxFLOAT Ptr) As orxSTATUS

/'* Gets text name
 * @param[in]   _pstText      Concerned text
 * @return      Text name / orxNULL
 '/
Declare Function orxText_GetName(ByVal _pstText As Const orxTEXT Ptr) As Const orxSTRING


/'* Gets text string 
 * @param[in]   _pstText      Concerned text
 * @return      Text string / orxSTRING_EMPTY
 '/
Declare Function orxText_GetString(ByVal _pstText As Const orxTEXT Ptr) As Const orxSTRING

/'* Gets text font
 * @param[in]   _pstText      Concerned text
 * @return      Text font / orxNULL
 '/
Declare Function orxText_GetFont(ByVal _pstText As Const orxTEXT Ptr) As Const orxSTRING

/'* Gets text data
 * @param[in]   _pstText      Concerned text
 * @return      orxDISPLAY_TEXT / orxNULL
 '/
Declare Function orxText_GetData(ByVal _pstText As Const orxTEXT Ptr) As orxDISPLAY_TEXT Ptr

/'* Sets text string 
 * @param[in]   _pstText      Concerned text
 * @param[in]   _zString      String to contain
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxText_SetString(ByVal _pstText As orxTEXT Ptr,ByVal _zString As Const orxSTRING) As orxSTATUS

/'* Sets text font
 * @param[in]   _pstText      Concerned text
 * @param[in]   _zFont        Font name / orxNULL to use default
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxText_SetFont(ByVal _pstText As orxTEXT Ptr,ByVal _zFont As Const orxSTRING) As orxSTATUS

End Extern

#endif /' _orxTEXT_BI '/

/'* @} '/
