/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxTexture.h
 * @date 07/12/2003
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxTexture
 * 
 * Texture module
 * Module that handles textures
 *
 * @{
 '/


#ifndef _orxTEXTURE_BI
#define _orxTEXTURE_BI

#include "orxInclude.bi"
#include "orx/display/orxDisplay.bi"
#include "orx/math/orxMath.bi"


/'* Defines
 '/
#define orxTEXTURE_KZ_SCREEN_NAME         "-=SCREEN=-"


/'* Internal texture structure '/
Type __orxTEXTURE_t As Any
Type orxTEXTURE As __orxTEXTURE_t


Extern "Windows"

/'* Setups the texture module
 '/
Declare Sub orxTexture_Setup()

/'* Inits the texture module
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTexture_Init() As orxSTATUS

/'* Exits from the texture module
 '/
Declare Sub orxTexture_Exit()


/'* Creates an empty texture
 * @return      orxTEXTURE / orxNULL
 '/
Declare Function orxTexture_Create() As orxTEXTURE Ptr

/'* Creates a texture from a bitmap file
 * @param[in]   _zBitmapFileName  Name of the bitmap
 * @return      orxTEXTURE / orxNULL
 '/
Declare Function orxTexture_CreateFromFile(ByVal _zBitmapFileName As Const orxSTRING) As orxTEXTURE Ptr

/'* Deletes a texture (and its referenced bitmap)
 * @param[in]   _pstTexture     Concerned texture
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTexture_Delete(ByVal _pstTexture As orxTEXTURE Ptr) As orxSTATUS

/'* Links a bitmap
 * @param[in]   _pstTexture     Concerned texture
 * @param[in]   _pstBitmap      Bitmap to link
 * @param[in]   _zDataName      Name associated with the bitmap (usually filename)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTexture_LinkBitmap(ByVal _pstTexture As orxTEXTURE Ptr,ByVal _pstBitmap As Const orxBITMAP Ptr,ByVal _zDataName As Const orxSTRING) As orxSTATUS

/'* Unlinks (and deletes if not used anymore) a bitmap
 * @param[in]   _pstTexture     Concerned texture
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTexture_UnlinkBitmap(ByVal _pstTexture As orxTEXTURE Ptr) As orxSTATUS


/'* Gets texture bitmap
 * @param[in]   _pstTexture     Concerned texture
 * @return      orxBITMAP / orxNULL
 '/
Declare Function orxTexture_GetBitmap(ByVal _pstTexture As Const orxTEXTURE Ptr) As orxBITMAP Ptr

/'* Gets texture size
 * @param[in]   _pstTexture     Concerned texture
 * @param[out]  _pfWidth        Texture's width
 * @param[out]  _pfHeight       Texture's height
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTexture_GetSize(ByVal _pstTexture As Const orxTEXTURE Ptr,ByVal _pfWidth As orxFLOAT Ptr,ByVal _pfHeight As orxFLOAT Ptr) As orxSTATUS

/'* Gets texture name
 * @param[in]   _pstTexture   Concerned texture
 * @return      Texture name / orxNULL
 '/
Declare Function orxTexture_GetName(ByVal _pstTexture As Const orxTEXTURE Ptr) As Const orxSTRING

/'* Sets texture color
 * @param[in]   _pstTexture     Concerned texture
 * @param[in]   _pstColor       Color to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTexture_SetColor(ByVal _pstTexture As orxTEXTURE Ptr,ByVal _pstColor As Const orxCOLOR Ptr) As orxSTATUS

/'* Gets screen texture
 * @return      Screen texture / orxNULL
 '/
Declare Function orxTexture_GetScreenTexture() As orxTEXTURE Ptr

End Extern

#endif /' _orxTEXTURE_BI '/

/'* @} '/
