/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxFile.h
 * @date 01/05/2005
 * @author bestel@arcallians.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxFile
 * 
 * File plugin module
 * Module that handles file access
 *
 * @{
 '/


#ifndef _orxFILE_BI
#define _orxFILE_BI
 
#include "orxInclude.bi"


#define orxFILE_KU32_FLAG_OPEN_READ   &H00000001 /'*< opened for read '/
#define orxFILE_KU32_FLAG_OPEN_WRITE  &H00000002 /'*< opened for write '/
#define orxFILE_KU32_FLAG_OPEN_APPEND &H00000004 /'*< descriptor positioned at the end of file '/
#define orxFILE_KU32_FLAG_OPEN_BINARY &H00000008 /'*< binary file accessing '/


/'* Internal File structure
 '/
Type __orxFILE_t As Any
Type orxFILE As __orxFILE_t

Extern "Windows"

/'* File module setup '/
Declare Sub orxFile_Setup()

/'* Inits the File Module
 '/
Declare Function orxFile_Init() As orxSTATUS

/'* Exits from the File Module
 '/
Declare Sub orxFile_Exit()

/'* Opens a file for later read or write operation
 * @param[in] _zPath               Full file's path to open
 * @param[in] _u32OpenFlags        List of used flags when opened
 * @return a File pointer (or orxNULL if an error has occured)
 '/
Declare Function orxFile_Open(ByVal _zPath As Const orxSTRING,ByVal _u32OpenFlags As orxU32) As orxFILE Ptr

/'* Reads data from a file
 * @param[out] _pReadData          Pointer where will be stored datas
 * @param[in] _u32ElemSize         Size of 1 element
 * @param[in] _u32NbElem           Number of elements
 * @param[in] _pstFile             Pointer on the file descriptor
 * @return Returns the number of read elements (not bytes)
 '/
Declare Function orxFile_Read(ByVal _pReadData As Any Ptr,ByVal _u32ElemSize As orxU32,ByVal _u32NbElem As orxU32,ByVal _pstFile As orxFILE Ptr) As orxU32

/'* writes data to a file
 * @param[in] _pDataToWrite        Pointer where will be stored datas
 * @param[in] _u32ElemSize         Size of 1 element
 * @param[in] _u32NbElem           Number of elements
 * @param[in] _pstFile             Pointer on the file descriptor
 * @return Returns the number of written elements (not bytes)
 '/
Declare Function orxFile_Write(ByVal _pDataToWrite As Any Ptr,ByVal _u32ElemSize As orxU32,ByVal _u32NbElem As orxU32,ByVal _pstFile As orxFILE Ptr) As orxU32

/'* Prints a formatted string to a file
 * @param[in] _pstFile             Pointer on the file descriptor
 * @param[in] _zString             Formatted string
 * @return Returns the number of written characters
 '/
 
End Extern

Extern "C"

Declare Function orxFile_Print (ByVal _pstFile As orxFILE Ptr,ByVal _zString As orxSTRING,...) As orxS32

End Extern

Extern "Windows"

/'* Gets text line from a file
 * @param[out] _zBuffer             Pointer where will be stored datas
 * @param[in] _u32Size              Size of buffer
 * @param[in] _pstFile              Pointer on the file descriptor
 * @return Returns orxTRUE if a line has been read, else returns orxFALSE.
 '/
Declare Function orxFile_ReadLine(ByVal _zBuffer As orxSTRING,ByVal _u32Size As orxU32,ByVal _pstFile As orxFILE Ptr) As orxBOOL

/'* Closes an oppened file
 * @param[in] _pstFile             File's pointer to close
 * @return Returns the status of the operation
 '/
Declare Function orxFile_Close(ByVal _pstFile As orxFILE Ptr) As orxSTATUS

End Extern

#endif /' _orxFILE_BI '/

/'* @} '/
