/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxFileSystem.h
 * @date 01/05/2005
 * @author bestel@arcallians.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxFileSystem
 * 
 * File system module
 * Module that handles file system access
 *
 * @{
 '/


#ifndef _orxFILESYSTEM_BI
#define _orxFILESYSTEM_BI
 
#include "orxInclude.bi"


#define orxFILESYSTEM_KU32_FLAG_INFO_NORMAL &H00000001 /'*< normal file '/
#define orxFILESYSTEM_KU32_FLAG_INFO_RDONLY &H00000002 /'*< read-only file '/
#define orxFILESYSTEM_KU32_FLAG_INFO_HIDDEN &H00000004 /'*< hidden file '/
#define orxFILESYSTEM_KU32_FLAG_INFO_DIR    &H00000008 /'*< directory '/

#define orxFILESYSTEM_KU32_FLAG_OPEN_READ   &H00000001 /'*< opened for read '/
#define orxFILESYSTEM_KU32_FLAG_OPEN_WRITE  &H00000002 /'*< opened for write '/
#define orxFILESYSTEM_KU32_FLAG_OPEN_APPEND &H00000004 /'*< descriptor positioned at the end of file '/


/'* Store datas about the current file. '/
Type __orxFILESYSTEM_INFO_t
  As orxU32 u32Flags          /'*< File attributes (see list of availables flags) '/
  As orxU32 u32TimeStamp      /'*< Timestamp of the last modification '/
  As orxU32 u32Size           /'*< File's size (in bytes) '/
  As orxHANDLE hInternal      /'*< Internal use handle '/
  As orxCHAR zName(256)       /'*< File's name '/
  As orxCHAR zPattern(256)    /'*< Search pattern '/
  As orxCHAR zPath(1024)      /'*< Directory's name where is stored the file '/
  As orxCHAR zFullName(1280)  /'*< Full file name '/
End Type

Type orxFILESYSTEM_INFO As __orxFILESYSTEM_INFO_t


/'* Internal FileSystem structure '/
Type __orxFILESYSTEM_t As Any
Type  orxFILESYSTEM As __orxFILESYSTEM_t

Extern "Windows"

/'* FileSystem module setup '/
Declare Sub orxFileSystem_Setup()

/'* Inits the FileSystem Module
 '/
Declare Function orxFileSystem_Init() As orxSTATUS

/'* Exits from the FileSystem Module
 '/
Declare Sub orxFileSystem_Exit()

/'* Returns orxTRUE if a file exists, else orxFALSE.
 * @param[in] _zFileName           Full File's name to test
 * @return orxFALSE if _zFileName doesn't exist, else orxTRUE
 '/
Declare Function orxFileSystem_Exists(ByVal _zFileName As Const orxSTRING) As orxBOOL

/'* Starts a new search. Find the first file that will match to the given pattern (e.g : /bin/toto* or c:\*.*)
 * @param[in] _zSearchPattern      Pattern to find
 * @param[out] _pstFileInfo        Informations about the first file found
 * @return orxTRUE if a file has been found, else orxFALSE
 '/
Declare Function orxFileSystem_FindFirst(ByVal _zSearchPattern As Const orxSTRING,ByVal _pstFileInfo As orxFILESYSTEM_INFO Ptr) As orxBOOL

/'* Continues a search. Find the next occurence of a pattern. The search has to be started with orxFileSystem_FindFirst
 * @param[in,out] _pstFileInfo    (IN/OUT) Informations about the found file
 * @return orxTRUE, if the next file has been found, else returns orxFALSE
 '/
Declare Function orxFileSystem_FindNext(ByVal _pstFileInfo As orxFILESYSTEM_INFO Ptr) As orxBOOL

/'* Closes a search (free the memory allocated for this search).
 * @param[in] _pstFileInfo         Informations returned during search
 '/
Declare Sub orxFileSystem_FindClose(ByVal _pstFileInfo As orxFILESYSTEM_INFO Ptr)

/'* Retrieves informations about a file
 * @param[in] _zFileName            Files used to get informations
 * @param[out] _pstFileInfo         Returned file's informations
 * @return Returns the status of the operation
 '/
Declare Function orxFileSystem_Info(ByVal _zFileName As Const orxSTRING,ByVal _pstFileInfo As orxFILESYSTEM_INFO Ptr) As orxSTATUS

/'* Copies a file
 * @param[in] _zSource             Source file's name
 * @param[in] _zDest               Destination file's name
 * @return The status of the operation
 '/
Declare Function orxFileSystem_Copy(ByVal _zSource As Const orxSTRING,ByVal _zDest As Const orxSTRING) As orxSTATUS

/'* Renames a file
 * @param[in] _zSource             Source file's name
 * @param[in] _zDest               Destination file's name
 * @return The status of the operation
 '/
Declare Function orxFileSystem_Rename(ByVal _zSource As Const orxSTRING,ByVal _zDest As Const orxSTRING) As orxSTATUS

/'* Deletes a file
 * @param[in] _zFileName           File's name to delete
 * @return The status of the operation
 '/
Declare Function orxFileSystem_Delete(ByVal _zFileName As Const orxSTRING) As orxSTATUS

/'* Creates a directory
 * @param[in] _zDirName            New directory's name
 * @return The status of the operation
 '/
Declare Function orxFileSystem_CreateDir(ByVal _zDirName As Const orxSTRING) As orxSTATUS

/'* Removes an empty directory
 * @param[in] _zDirName            Directory's name to delete
 * @return The status of the operation
 '/
Declare Function orxFileSystem_DeleteDir(ByVal _zDirName  As Const orxSTRING ) As orxSTATUS

End Extern

#endif /' _orxFILESYSTEM_BI '/

/'* @} '/
