/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxInput.h
 * @date 04/11/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxInput
 * 
 * Input module
 * Module that handles generalized inputs (keyboard, mouse, joystick, ...)
 *
 * @{
 '/


#ifndef _orxINPUT_BI
#define _orxINPUT_BI


#include "orxInclude.bi"
#include "orx/io/orxJoystick.bi"
#include "orx/io/orxKeyboard.bi"
#include "orx/io/orxMouse.bi"


/'* Misc defines
 '/
#define orxINPUT_KU32_BINDING_NUMBER    4


/'* Input type enum
 '/
Enum orxINPUT_TYPE
  orxINPUT_TYPE_KEYBOARD_KEY = 0,
  orxINPUT_TYPE_MOUSE_BUTTON,
  orxINPUT_TYPE_JOYSTICK_BUTTON,
  orxINPUT_TYPE_JOYSTICK_AXIS,
  orxINPUT_TYPE_EXTERNAL,

  orxINPUT_TYPE_NUMBER,

  orxINPUT_TYPE_NONE = orxENUM_NONE

End Enum


/'* Event enum
 '/
Enum orxINPUT_EVENT

  orxINPUT_EVENT_ON = 0,
  orxINPUT_EVENT_OFF,

  orxINPUT_EVENT_NUMBER,

  orxINPUT_EVENT_NONE = orxENUM_NONE

End Enum



/'* Input event payload
 '/
Type __orxINPUT_EVENT_PAYLOAD_t

  As orxSTRING     zSetName                               /'*< Set name : 4 '/
  As orxSTRING     zInputName                             /'*< Input name : 8 '/
  As orxINPUT_TYPE aeType(orxINPUT_KU32_BINDING_NUMBER)   /'*< Input binding type : 16 '/
  As orxENUM       aeID(orxINPUT_KU32_BINDING_NUMBER)     /'*< Input binding ID : 24 '/
  As orxFLOAT      afValue(orxINPUT_KU32_BINDING_NUMBER)  /'*< Input binding value : 32 '/

End Type 

Type orxINPUT_EVENT_PAYLOAD As __orxINPUT_EVENT_PAYLOAD_t

Extern "Windows"

/'* Input module setup
 '/
Declare Sub orxInput_Setup()

/'* Initializes Input module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxInput_Init() As orxSTATUS

/'* Exits from Input module
 '/
Declare Sub orxInput_Exit()

/'* Loads inputs from config
 * @param[in] _zFileName        File name to load, will use current loaded config if orxSTRING_EMPTY/orxNULL
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxInput_Load(ByVal _zFileName As Const orxSTRING) As orxSTATUS

/'* Saves inputs to config
 * @param[in] _zFileName        File name
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxInput_Save(ByVal _zFileName As Const orxSTRING) As orxSTATUS


/'* Selects current working set
 * @param[in] _zSetName         Set name to select
 '/
Declare Function orxInput_SelectSet(ByVal _zSetName As Const orxSTRING) As orxSTATUS

/'* Gets current working set
 * @return Current selected set
 '/
Declare Function orxInput_GetCurrentSet() As Const orxSTRING


/'* Is input active?
 * @param[in] _zInputName       Concerned input name
 * @return orxTRUE if active, orxFALSE otherwise
 '/
Declare Function orxInput_IsActive(ByVal _zInputName As Const orxSTRING) As orxBOOL

/'* Has a new active status since this frame?
 * @param[in] _zInputName       Concerned input name
 * @return orxTRUE if active status is new, orxFALSE otherwise
 '/
Declare Function orxInput_HasNewStatus(ByVal _zInputName As Const orxSTRING) As orxBOOL

/'* Gets input value
 * @param[in] _zInputName       Concerned input name
 * @return orxFLOAT
 '/
Declare Function orxInput_GetValue(ByVal _zInputName As Const orxSTRING) As orxFLOAT

/'* Sets input value (will prevail on peripheral inputs only once)
 * @param[in] _zInputName       Concerned input name
 * @param[in] _fValue           Value to set, orxFLOAT_0 to deactivate
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxInput_SetValue(ByVal _zInputName As Const orxSTRING,ByVal _fValue As orxFLOAT) As orxSTATUS

/'* Sets permanent input value (will prevail on peripheral inputs till reset)
 * @param[in] _zInputName       Concerned input name
 * @param[in] _fValue           Value to set, orxFLOAT_0 to deactivate
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxInput_SetPermanentValue(ByVal _zInputName As Const orxSTRING,ByVal _fValue As orxFLOAT) As orxSTATUS

/'* Resets input value (peripheral inputs will then be used instead of code ones)
 * @param[in] _zInputName       Concerned input name
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxInput_ResetValue(ByVal _zInputName As Const orxSTRING) As orxSTATUS


/'* Sets an input combine mode
 * @param[in] _zName            Concerned input name
 * @param[in] _bCombine         If orxTRUE, all assigned bindings need to be active in order to activate input, otherwise input will be considered active if any of its binding is
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxInput_SetCombineMode(ByVal _zName As Const orxSTRING,ByVal _bCombine As orxBOOL) As orxSTATUS

/'* Is an input in combine mode?
 * @param[in] _zName            Concerned input name
 * @return orxTRUE if the input is in combine mode, orxFALSE otherwise
 '/
Declare Function orxInput_IsInCombineMode(ByVal _zName As Const orxSTRING) As orxBOOL


/'* Binds an input to a mouse/joystick button, keyboard key or joystick axis
 * @param[in] _zName            Concerned input name
 * @param[in] _eType            Type of peripheral to bind
 * @param[in] _eID              ID of button/key/axis to bind
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxInput_Bind(ByVal _zName As Const orxSTRING,ByVal _eType As orxINPUT_TYPE,ByVal _eID As orxENUM) As orxSTATUS

/'* Unbinds a mouse/joystick button, keyboard key or joystick axis
 * @param[in] _eType            Type of peripheral to unbind
 * @param[in] _eID              ID of button/key/axis to unbind
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxInput_Unbind(ByVal _eType As orxINPUT_TYPE,ByVal _eID As orxENUM) As orxSTATUS

/'* Gets the input counter to which a mouse/joystick button, keyboard key or joystick axis is bound
 * @param[in] _eType            Type of peripheral to test
 * @param[in] _eID              ID of button/key/axis to test
 * @return Number of bound inputs
 '/
Declare Function orxInput_GetBoundInputCounter(ByVal _eType As orxINPUT_TYPE,ByVal _eID As orxENUM) As orxU32

/'* Gets the input name to which a mouse/joystick button, keyboard key or joystick axis is bound (at given index)
 * @param[in] _eType            Type of peripheral to test
 * @param[in] _eID              ID of button/key/axis to test
 * @param[in] _u32InputIndex    Index of the desired input
 * @return orxSTRING input name if bound / orxSTRING_EMPY otherwise
 '/
Declare Function orxInput_GetBoundInput(ByVal _eType As orxINPUT_TYPE,ByVal _eID As orxENUM,ByVal _u32InputIndex As orxU32) As Const orxSTRING

/'* Gets an input binding (mouse/joystick button, keyboard key or joystick axis) at a given index
 * @param[in]   _zName            Concerned input name
 * @param[in]   _u32BindingIndex  Index of the desired binding
 * @param[out]  _peType           List of binding types (if a slot is not bound, its value is orxINPUT_TYPE_NONE)
 * @param[out]  _peID             List of binding IDs (button/key/axis)
 * @return orxSTATUS_SUCCESS if input exists, orxSTATUS_FAILURE otherwise
 '/
Declare Function orxInput_GetBinding(ByVal _zName As Const orxSTRING,ByVal _u32BindingIndex As orxU32,ByVal _peType As orxINPUT_TYPE Ptr,ByVal _peID As orxENUM Ptr) As orxSTATUS

/'* Gets an input binding (mouse/joystick button, keyboard key or joystick axis) list
 * @param[in]   _zName          Concerned input name
 * @param[out]  _aeTypeList     List of binding types (if a slot is not bound, its value is orxINPUT_TYPE_NONE)
 * @param[out]  _aeIDList       List of binding IDs (button/key/axis)
 * @return orxSTATUS_SUCCESS if input exists, orxSTATUS_FAILURE otherwise
 '/
Declare Function orxInput_GetBindingList(ByVal _zName As Const orxSTRING,_aeTypeList() As orxINPUT_TYPE,_aeIDList() As orxENUM) As orxSTATUS


/'* Gets a binding name
 * @param[in]   _eType          Binding type (mouse/joystick button, keyboard key or joystick axis)
 * @param[in]   _eID            Binding ID (ID of button/key/axis to bind)
 * @return orxSTRING (binding's name) if success, orxSTRING_EMPTY otherwise
 '/
Declare Function orxInput_GetBindingName(ByVal _eType As orxINPUT_TYPE,ByVal _eID As orxENUM) As Const orxSTRING

/'* Gets active binding (current pressed key/button/...) so as to allow on-the-fly user rebinding
 * @param[out]  _peType         Active binding type (mouse/joystick button, keyboard key or joystick axis)
 * @param[out]  _peID           Active binding ID (ID of button/key/axis to bind)
 * @return orxSTATUS_SUCCESS if one active binding is found, orxSTATUS_FAILURE otherwise
 '/
Declare Function orxInput_GetActiveBinding(ByVal _peType As orxINPUT_TYPE Ptr,ByVal _peID As orxENUM Ptr) As orxSTATUS

End Extern

#endif /'_orxINPUT_BI'/

/'* @} '/
