/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxMouse.h
 * @date 22/11/2003
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxMouse
 * 
 * Mouse plugin module
 * Plugin module that handles mouse-like peripherals
 *
 * @{
 '/


#ifndef _orxMOUSE_BI
#define _orxMOUSE_BI

#include "orxInclude.bi"
''#include "orx/plugin/orxPluginCore.bi"
#include "orx/math/orxVector.bi"


/'* Button enum
 '/
Enum orxMOUSE_BUTTON

  orxMOUSE_BUTTON_LEFT = 0,
  orxMOUSE_BUTTON_RIGHT,
  orxMOUSE_BUTTON_MIDDLE,
  orxMOUSE_BUTTON_EXTRA_1,
  orxMOUSE_BUTTON_EXTRA_2,
  orxMOUSE_BUTTON_WHEEL_UP,
  orxMOUSE_BUTTON_WHEEL_DOWN,
  orxMOUSE_BUTTON_NUMBER,

  orxMOUSE_BUTTON_NONE = orxENUM_NONE

End Enum


/'* Event enum
 '/
Enum orxMOUSE_EVENT

  orxMOUSE_EVENT_BUTTON_PRESSED = 0,
  orxMOUSE_EVENT_BUTTON_RELEASED,

  orxMOUSE_EVENT_NUMBER,

  orxMOUSE_EVENT_NONE = orxENUM_NONE

End Enum


/'* Mouse event payload
 '/
Type orxMOUSE_EVENT_PAYLOAD

  As orxMOUSE_BUTTON eButton        /'*< Mouse button : 4 '/

End Type


#define orxMOUSE_KZ_CONFIG_SECTION      "Mouse"
#define orxMOUSE_KZ_CONFIG_SHOW_CURSOR  "ShowCursor"


/'**************************************************************************
 * Functions directly implemented by orx core
 **************************************************************************'/

Extern "Windows"

/'* Mouse module setup
 '/
Declare Sub orxMouse_Setup()


/'**************************************************************************
 * Functions extended by plugins
 **************************************************************************'/

/'* Inits the mouse module
 * @return Returns the status of the operation
 '/
Declare Function orxMouse_Init() As orxSTATUS

/'* Exits from the mouse module
 '/
Declare Sub orxMouse_Exit()

/'* Sets mouse position
* @param[in] _pvPosition  Mouse position
* @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
'/
Declare Function orxMouse_SetPosition(ByVal _pvPosition As Const orxVECTOR Ptr) As orxSTATUS

/'* Gets mouse position
 * @param[out] _pvPosition  Mouse position
 * @return orxVECTOR / orxNULL
 '/
Declare Function orxMouse_GetPosition(ByVal _pvPosition As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Is mouse button pressed?
 * @param[in] _eButton          Mouse button to check
 * @return orxTRUE if presse / orxFALSE otherwise
 '/
Declare Function orxMouse_IsButtonPressed(ByVal _eButton As orxMOUSE_BUTTON) As orxBOOL

/'* Gets mouse move delta (since last call)
 * @param[out] _pvMoveDelta Mouse move delta
 * @return orxVECTOR / orxNULL
 '/
Declare Function orxMouse_GetMoveDelta(ByVal _pvMoveDelta As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets mouse wheel delta (since last call)
 * @return Mouse wheel delta
 '/
Declare Function orxMouse_GetWheelDelta() As orxFLOAT

/'* Shows mouse cursor
* @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
'/
Declare Function orxMouse_ShowCursor(ByVal _bShow As orxBOOL) As orxSTATUS

/'* Gets button literal name
 * @param[in] _eButton          Concerned button
 * @return Button's name
 '/
Declare Function orxMouse_GetButtonName(ByVal _eButton As orxMOUSE_BUTTON) As Const orxSTRING

End Extern

#endif /' _orxMOUSE_BI '/

/'* @} '/
