/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxParam.h
 * @date 09/09/2005
 * @author bestel@arcallians.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxParam
 * 
 * Param module
 * Module that handles command line parameters
 *
 * @{
 '/


#ifndef _orxPARAM_BI
#define _orxPARAM_BI


#include "orxInclude.bi"


/' Macro definition '/

/' Flags definitions '/
#define orxPARAM_KU32_FLAG_NONE             &H00000000 /'*< No flags (default behaviour) '/
#define orxPARAM_KU32_FLAG_MULTIPLE_ALLOWED &H00000001 /'*< It's possible to have multiple instance of the same param '/
#define orxPARAM_KU32_FLAG_STOP_ON_ERROR    &H00000002 /'*< If the param extra flag parsing fails, stop the engine '/

/'* Callback definition (called when a parameter has been found
 * @param[in] _u32NbParam Number of extra parameters read for this option
 * @param[in] _azParams   Array of extra parameters (the first one is always the option name)
 * @return Returns orxSTATUS_SUCCESS if informations read are correct, orxSTATUS_FAILURE if a problem has occured
 '/
Type orxPARAM_INIT_FUNCTION As Function(ByVal _u32NbParam As orxU32,_azParams As Const orxSTRING Ptr) As orxSTATUS

/' Parameter's info '/
Type __orxPARAM_t

  /' Flags associated to the parameters' parser '/
  As orxU32                  u32Flags
  
  /' Short parameter name (ex: "h" for help (will be -h)) '/
  As orxSTRING               zShortName

  /' Long parameter name (ex: "help" for help (will be --help)) '/
  As orxSTRING               zLongName

  /' Short description (ex: "shows params' list") '/
  As orxSTRING               zShortDesc

  /' Long description (ex: "Display the list of parameters.
   * -help <param> displays the <param> long description") '/
  As orxSTRING               zLongDesc
  
  /' Function called when the parameter is detected in the command line '/
  As orxPARAM_INIT_FUNCTION  pfnParser
  
End Type

Type orxPARAM As __orxPARAM_t

Extern "Windows"

/'* Setups param module
 '/
Declare Sub orxParam_Setup()

/'* Inits the param Module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxParam_Init() As orxSTATUS

/'* Exits from the param module
 '/
Declare Sub orxParam_Exit()


/'* Registers a new parameter
 * @param[in] _pstParam Informations about the option to register
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxParam_Register(ByVal _pstParam As Const orxPARAM Ptr) As orxSTATUS

/'* Displays help if requested
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxParam_DisplayHelp() As orxSTATUS

/'* Sets the command line arguments
 * @param[in] _u32NbParams  Number of read parameters
 * @param[in] _azParams     List of parameters
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxParam_SetArgs(ByVal _u32NbParams As orxU32,_azParams As orxSTRING Ptr) As orxSTATUS

End Extern

#endif /' _orxPARAM_BI '/

/'* @} '/
