/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxAABox.h
 * @date 03/10/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxAABox
 * 
 * Axis-aligned box module
 * Module that handles axis-aligned boxes
 *
 * @{
 '/


#ifndef _orxAABOX_BI
#define _orxAABOX_BI

#include "orxInclude.bi"
#include "orx/math/orxVector.bi"


/'* Public structure definition
 '/
Type __orxAABOX_t

  As orxVECTOR vTL /'*< Top left corner vector : 12 '/
  As orxVECTOR vBR /'*< Bottom right corner vector : 24 '/

End Type

Type orxAABOX As __orxAABOX_t


/' *** AABox inlined functions *** '/


/'* Reorders AABox corners
 * @param[in]   _pstBox  Box to reorder
 * @return      Reordered AABox
 '/
#macro orxAABox_Reorder(_pstBox,_result)

  /' Checks '/
  orxASSERT(_pstBox <> orxNULL)

  /' Reorders coordinates so as to have upper left & bottom right box corners '/

  /' Z coord '/
  If(_pstBox->vTL.fZ > _pstBox->vBR.fZ) Then
  
    /' Swaps '/
    orxSWAP32(_pstBox->vTL.fZ, _pstBox->vBR.fZ)
  
	End If
  
	/' Y coord '/
  If(_pstBox->vTL.fY > _pstBox->vBR.fY) Then
  
    /' Swaps '/
    orxSWAP32(_pstBox->vTL.fY, _pstBox->vBR.fY)
  
	End If

  /' X coord '/
  If(_pstBox->vTL.fX > _pstBox->vBR.fX) Then
  
    /' Swaps '/
    orxSWAP32(_pstBox->vTL.fX, _pstBox->vBR.fX)
  
	End If

  /' Done! '/
  _result = _pstBox
#endmacro

/'* Sets axis aligned box values
 * @param[out]  _pstRes                       AABox to set
 * @param[in]   _pvTL                         Top left corner
 * @param[in]   _pvBR                         Bottom right corner
 * @return      orxAABOX / orxNULL
 '/
#macro orxAABox_Set(_pstRes,_pvTL,_pvBR,_retVal)
Scope
  
	Dim As orxVECTOR vTEMP
	Dim As AABox aTEMP
  /' Checks '/
  orxASSERT(_pstRes <> orxNULL)
  orxASSERT(_pvTL <> orxNULL)
  orxASSERT(_pvBR <> orxNULL)

  /' Sets values '/
  orxVector_Copy(@(_pstRes->vTL), _pvTL,vTEMP)
  orxVector_Copy(@(_pstRes->vBR), _pvBR,vTEMP)

  /' Reorders corners '/
  orxAABox_Reorder(_pstRes,aTEMP)

  /' Done! '/
  _retval = _pstRes

#endmacro

/'* Is position inside axis aligned box test
 * @param[in]   _pstBox                       Box to test against position
 * @param[in]   _pvPosition                   Position to test against the box
 * @return      orxTRUE if position is inside the box, orxFALSE otherwise
 '/
#macro orxAABox_IsInside(_pstBox,_pvPosition,_result)

  
	_result = orxFALSE
     
  orxASSERT(_pstBox <> orxNULL)
  orxASSERT(_pvPosition <> orxNULL)

  /' Z intersected? '/
  If((_pvPosition->fZ >= _pstBox->vTL.fZ) AndAlso _
  (_pvPosition->fZ <= _pstBox->vBR.fZ)) Then  
    /' X intersected? '/
    If((_pvPosition->fX >= _pstBox->vTL.fX) AndAlso _
    (_pvPosition->fX <= _pstBox->vBR.fX)) Then    
      /' Y intersected? '/
      If((_pvPosition->fY >= _pstBox->vTL.fY) AndAlso _
       (_pvPosition->fY <= _pstBox->vBR.fY)) Then      
        /' Intersects '/
        _result = orxTRUE
      End If
    End If
  End If

  /' Done! '/
  
#endmacro

/'* Tests axis aligned box intersection
 * @param[in]   _pstBox1                      First box operand
 * @param[in]   _pstBox2                      Second box operand
 * @return      orxTRUE if boxes intersect, orxFALSE otherwise
 '/
#macro orxAABox_TestIntersection(_pstBox1,_pstBox2,_result)

  _result = orxFALSE

  /' Checks '/
  orxASSERT(_pstBox1 <> orxNULL)
  orxASSERT(_pstBox2 <> orxNULL)

  /' Z intersected? '/
  if((_pstBox2->vBR.fZ >= _pstBox1->vTL.fZ) AndAlso _
  (_pstBox2->vTL.fZ <= _pstBox1->vBR.fZ)) Then
  
    /' X intersected? '/
    If((_pstBox2->vBR.fX >= _pstBox1->vTL.fX) AndAlso _
    (_pstBox2->vTL.fX <= _pstBox1->vBR.fX)) Then
    
      /' Y intersected? '/
      If((_pstBox2->vBR.fY >= _pstBox1->vTL.fY) AndAlso
      (_pstBox2->vTL.fY <= _pstBox1->vBR.fY)) Then
      
        /' Intersects '/
        _result orxTRUE
      End If
    End If
  End If

  /' Done! '/
  
#endmacro

/'* Tests axis aligned box 2D intersection (no Z-axis test)
 * @param[in]   _pstBox1                      First box operand
 * @param[in]   _pstBox2                      Second box operand
 * @return      orxTRUE if boxes intersect in 2D, orxFALSE otherwise
 '/
#macro orxAABox_Test2DIntersection(_pstBox1,_pstBox2,_result)

  _result = orxFALSE

  /' Checks '/
  orxASSERT(_pstBox1 <> orxNULL)
  orxASSERT(_pstBox2 <> orxNULL)

  /' X intersected? '/
  If((_pstBox2->vBR.fX >= _pstBox1->vTL.fX) _
  AndAlso (_pstBox2->vTL.fX <= _pstBox1->vBR.fX)) Then  
    /' Y intersected? '/
    If((_pstBox2->vBR.fY >= _pstBox1->vTL.fY) AndAlso _
    (_pstBox2->vTL.fY <= _pstBox1->vBR.fY)) Then
    
      /' Intersects '/
      _result = orxTRUE
    End If
  End If

  /' Done! '/	

#endmacro

/'* Copies an AABox onto another one
 * @param[out]   _pstDst                      AABox to copy to (destination)
 * @param[in]   _pstSrc                       AABox to copy from (destination)
 * @return      Destination AABox
 '/
#macro orxAABox_Copy(_pstDst,_result)

  /' Checks '/
  orxASSERT(_pstDst <> orxNULL)
  orxASSERT(_pstSrc <> orxNULL)

  /' Copies it '/
  orxMemory_Copy(_pstDst, _pstSrc, sizeof(orxAABOX))

  /' Done! '/
  _result = _pstDst

#endmacro


/'* Moves an AABox
 * @param[out]  _pstRes                       AABox where to store result
 * @param[in]   _pstOp                        AABox to move
 * @param[in]   _pvMove                       Move vector
 * @return      Moved AABox
 '/
#macro orxAABox_Move(_pstRes,_pstOp,_pvMove)
Scope
  Dim As orxVECTOR vTEMP
  /' Checks '/
  orxASSERT(_pstRes <> orxNULL)
  orxASSERT(_pstOp <> orxNULL)
  orxASSERT(_pvMove <> orxNULL)

  /' Updates result '/
  orxVector_Add(@(_pstRes->vTL), @(_pstOp->vTL), _pvMove,vTEMP)
  orxVector_Add(@(_pstRes->vBR), @(_pstOp->vBR), _pvMove,vTEMP)

  /' Done! '/
	_result = _pstRes
End Scope 
#endmacro


/'* Gets AABox center position
 * @param[in]   _pstOp                        Concerned AABox
 * @param[out]  _pvRes                        Center position
 * @return      Center position vector
 '/
#macro orxAABox_GetCenter(_pstOp,_pvRes,_result)
Scope
  /' Checks '/
  orxASSERT(_pstOp <> orxNULL)
  orxASSERT(_pvRes <> orxNULL)
  Dim As orxVECTOR vTEMP
	
  /' Gets box center '/
  orxVector_Add(_pvRes, @(_pstOp->vTL), @(_pstOp->vBR),vTEMP)
  orxVector_Mulf(_pvRes, _pvRes, orx2F(0.5f),vTEMP)

  /' Done! '/
  _result = _pvRes
End Scope
#endmacro


#endif /' _orxAABOX_BI '/

/'* @} '/
