/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxObject.h
 * @date 01/12/2003
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxObject
 *
 * Object module
 * Allows to creates and handle objects
 * Objects are structures containers that can refer to many other structures such as frames, graphics, etc...
 *
 * @{
 '/


#ifndef _orxOBJECT_BI
#define _orxOBJECT_BI


#include "orxInclude.bi"
#include "orx/object/orxStructure.bi"
#include "orx/core/orxClock.bi"
#include "orx/memory/orxBank.bi"
#include "orx/anim/orxAnimSet.bi"
#include "orx/display/orxDisplay.bi"
#include "orx/math/orxOBox.bi"
#include "orx/sound/orxSound.bi"


/'* Event enum
 '/
Enum orxOBJECT_EVENT

  orxOBJECT_EVENT_CREATE = 0,
  orxOBJECT_EVENT_DELETE,

  orxOBJECT_EVENT_NUMBER,

  orxOBJECT_EVENT_NONE = orxENUM_NONE

End Enum


/'* Internal object structure '/
Type __orx_OBJECT_t As Any
Type orxOBJECT As __orxOBJECT_t

Extern "Windows"

/'* Object module setup
 '/
Declare Sub orxObject_Setup()

/'* Inits the object module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_Init() As orxSTATUS

/'* Exits from the object module
 '/
Declare Sub orxObject_Exit()

/'* Creates an empty object
 * @return orxOBJECT / orxNULL
 '/
Declare Function orxObject_Create() As orxOBJECT Ptr

/'* Creates an object from config
 * @param[in]   _zConfigID    Config ID
 * @ return orxOBJECT / orxNULL
 '/
Declare Function orxObject_CreateFromConfig(ByVal _zConfigID As Const orxSTRING) As orxOBJECT Ptr

/'* Deletes an object
 * @param[in] _pstObject        Concerned object
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_Delete(ByVal _pstObject As orxOBJECT Ptr) As orxSTATUS

/'* Enables/disables an object
 * @param[in]   _pstObject    Concerned object
 * @param[in]   _bEnable      Enable / disable
 '/
Declare Sub orxObject_Enable(ByVal _pstObject As orxOBJECT Ptr,ByVal _bEnable As orxBOOL)

/'* Is object enabled?
 * @param[in]   _pstObject    Concerned object
 * @return      orxTRUE if enabled, orxFALSE otherwise
 '/
Declare Function orxObject_IsEnabled(ByVal _pstObject As Const orxOBJECT Ptr) As orxBOOL

/'* Pauses/unpauses an object
 * @param[in]   _pstObject    Concerned object
 * @param[in]   _bPause       Pause / unpause
 '/
Declare Sub orxObject_Pause(ByVal _pstObject As orxOBJECT Ptr,ByVal _bPause As orxBOOL)

/'* Is object paused?
 * @param[in]   _pstObject    Concerned object
 * @return      orxTRUE if paused, orxFALSE otherwise
 '/
Declare Function orxObject_IsPaused(ByVal _pstObject As Const orxOBJECT Ptr) As orxBOOL


/'* Sets render status of an object
 * @param[in]   _pstObject    Concerned object
 * @param[in]   _bRendered    Rendered or not this frame
 '/
Declare Sub orxObject_SetRendered(ByVal _pstObject As orxOBJECT Ptr,ByVal _bRendered As orxBOOL)

/'* Is object rendered this frame?
 * @param[in]   _pstObject    Concerned object
 * @return      orxTRUE if rendered, orxFALSE otherwise
 '/
Declare Function orxObject_IsRendered(ByVal _pstObject As Const orxOBJECT Ptr) As orxBOOL


/'* Sets user data for an object
 * @param[in]   _pstObject    Concerned object
 * @param[in]   _pUserData    User data to store / orxNULL
 '/
Declare Sub orxObject_SetUserData(ByVal _pstObject As orxOBJECT Ptr,ByVal _pUserData As Any Ptr)

/'* Gets object's user data
 * @param[in]   _pstObject    Concerned object
 * @return      Stored user data / orxNULL
 '/
Declare Function orxObject_GetUserData(ByVal _pstObject As Const orxOBJECT Ptr) As Any Ptr

/'* Sets owner for an object
 * @param[in]   _pstObject    Concerned object
 * @param[in]   _pOwner       Owner to set / orxNULL
 '/
Declare Sub orxObject_SetOwner(ByVal _pstObject As orxOBJECT Ptr,ByVal _pOwner As Any Ptr)

/'* Gets object's owner
 * @param[in]   _pstObject    Concerned object
 * @return      Owner / orxNULL
 '/
Declare Function orxObject_GetOwner(ByVal _pstObject As Const orxOBJECT Ptr) As orxSTRUCTURE Ptr


/'* Gets object's first child (only if created with a config ChildList)
 * @param[in]   _pstObject    Concerned object
 * @return      First child object / orxNULL
 '/
Declare Function orxObject_GetChild(ByVal _pstObject As Const orxOBJECT Ptr) As orxOBJECT Ptr

/'* Gets object's next sibling (only if created with a config ChildList)
 * @param[in]   _pstObject    Concerned object
 * @return      Next sibling object / orxNULL
 '/
Declare Function orxObject_GetSibling(ByVal _pstObject As Const orxOBJECT Ptr) As orxOBJECT Ptr


/'* Sets associated clock for an object
 * @param[in]   _pstObject    Concerned object
 * @param[in]   _pstClock     Clock to associate / orxNULL
 '/
Declare Function orxObject_SetClock(ByVal _pstObject As orxOBJECT Ptr,ByVal _pstClock As orxCLOCK Ptr) As orxSTATUS

/'* Gets object's clock
 * @param[in]   _pstObject    Concerned object
 * @return      Associated clock / orxNULL
 '/
Declare Function orxObject_GetClock(ByVal _pstObject As Const orxOBJECT Ptr) As orxCLOCK Ptr


/'* Links a structure to an object
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pstStructure   Structure to link
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_LinkStructure(ByVal _pstObject As orxOBJECT Ptr,ByVal _pstStructure As orxSTRUCTURE Ptr) As orxSTATUS

/'* Unlinks structure from an object, given its structure ID
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _eStructureID   ID of structure to unlink
 '/
Declare Sub orxObject_UnlinkStructure(ByVal _pstObject As orxOBJECT Ptr,ByVal _eStructureID As orxSTRUCTURE_ID)


/' *** Object accessors *** '/


/'* Structure used by an object get accessor, given its structure ID. Structure must then be cast correctly (see helper macro)
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _eStructureID   ID of the structure to get
 * @return orxSTRUCTURE / orxNULL
 '/
Declare Function _orxObject_GetStructure(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _eStructureID As orxSTRUCTURE_ID) As orxSTRUCTURE Ptr


/'* Sets object flipping
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _bFlipX         Flip it on X axis
 * @param[in]   _bFlipY         Flip it on Y axis
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetFlip(ByVal _pstObject As orxOBJECT Ptr,ByVal _bFlipX As orxBOOL,ByVal _bFlipY As orxBOOL) As orxSTATUS

/'* Gets object flipping
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pbFlipX        X axis flipping
 * @param[in]   _pbFlipY        Y axis flipping
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_GetFlip(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pbFlipX As orxBOOL Ptr,ByVal _pbFlipY As orxBOOL Ptr) As orxSTATUS


/'* Sets object pivot
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pvPivot        Object pivot
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetPivot(ByVal _pstObject As orxOBJECT Ptr,ByVal _pvPivot As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets object position
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pvPosition     Object position
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetPosition(ByVal _pstObject As orxOBJECT Ptr,ByVal _pvPosition As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets object rotation
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _fRotation      Object rotation (radians)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetRotation(ByVal _pstObject As orxOBJECT Ptr,ByVal _fRotation As orxFLOAT) As orxSTATUS

/'* Sets object scale
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pvScale        Object scale vector
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetScale(ByVal _pstObject As orxOBJECT Ptr,ByVal _pvScale As Const orxVECTOR Ptr) As orxSTATUS

/'* Get object pivot
 * @param[in]   _pstObject      Concerned object
 * @param[out]  _pvPivot        Object pivot
 * @return      orxVECTOR / orxNULL
 '/
Declare Function orxObject_GetPivot(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pvPivot As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Get object origin
 * @param[in]   _pstObject      Concerned object
 * @param[out]  _pvOrigin       Object origin
 * @return      orxVECTOR / orxNULL
 '/
Declare Function orxObject_GetOrigin(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pvOrigin As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Get object position
 * @param[in]   _pstObject      Concerned object
 * @param[out]  _pvPosition     Object position
 * @return      orxVECTOR / orxNULL
 '/
Declare Function orxObject_GetPosition(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pvPosition As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Get object world position
 * @param[in]   _pstObject      Concerned object
 * @param[out]  _pvPosition     Object world position
 * @return      orxVECTOR / orxNULL
 '/
Declare Function orxObject_GetWorldPosition(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pvPosition As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Get object rotation
 * @param[in]   _pstObject      Concerned object
 * @return      orxFLOAT (radians)
 '/
Declare Function orxObject_GetRotation(ByVal _pstObject As Const orxOBJECT Ptr) As orxFLOAT

/'* Get object world rotation
 * @param[in]   _pstObject      Concerned object
 * @return      orxFLOAT (radians)
 '/
Declare Function orxObject_GetWorldRotation(ByVal _pstObject As Const orxOBJECT Ptr) As orxFLOAT

/'* Get object scale
 * @param[in]   _pstObject      Concerned object
 * @param[out]  _pvScale        Object scale vector
 * @return      Scale vector
 '/
Declare Function orxObject_GetScale(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pvScale As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets object world scale
 * @param[in]   _pstObject      Concerned object
 * @param[out]  _pvScale        Object world scale
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_GetWorldScale(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pvScale As orxVECTOR Ptr) As orxVECTOR Ptr


/'* Sets an object parent
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pParent        Parent structure to set (object, spawner, camera or frame) / orxNULL
 * @return      orsSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetParent(ByVal _pstObject As orxOBJECT Ptr,ByVal _pParent As Any Ptr) As orxSTATUS


/'* Gets object size
 * @param[in]   _pstObject      Concerned object
 * @param[out]  _pvSize         Object's size
 * @return      orxVECTOR / orxNULL
 '/
Declare Function orxObject_GetSize(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pvSize As orxVECTOR Ptr) As orxVECTOR Ptr


/'* Sets an object animset
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pstAnimSet     Animation set to set / orxNULL
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetAnimSet(ByVal _pstObject As orxOBJECT Ptr,ByVal _pstAnimSet As orxANIMSET Ptr) As orxSTATUS


/'* Sets an object animation frequency
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _fFrequency     Frequency to set
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetAnimFrequency(ByVal _pstObject As orxOBJECT Ptr,ByVal _fFrequency As orxFLOAT) As orxSTATUS

/'* Sets current animation for object
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zAnimName      Animation name (config's one) to set / orxNULL
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetCurrentAnim(ByVal _pstObject As orxOBJECT Ptr,ByVal _zAnimName As Const orxSTRING) As orxSTATUS

/'* Sets target animation for object
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zAnimName      Animation name (config's one) to set / orxNULL
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetTargetAnim(ByVal _pstObject As orxOBJECT Ptr,ByVal _zAnimName As Const orxSTRING) As orxSTATUS

/'* Is current animation test
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zAnimName      Animation name (config's one) to test
 * @return      orxTRUE / orxFALSE
 '/
Declare Function orxObject_IsCurrentAnim(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _zAnimName As Const orxSTRING) As orxBOOL

/'* Is target animation test
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zAnimName      Animation name (config's one) to test
 * @return      orxTRUE / orxFALSE
 '/
Declare Function orxObject_IsTargetAnim(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _zAnimName As Const orxSTRING) As orxBOOL


/'* Gets object's bounding box (OBB)
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pstBoundingBox Bounding box result
 * @return      Bounding box / orxNULL
 '/
Declare Function orxObject_GetBoundingBox(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pstBoundingBox As orxOBOX Ptr) As orxOBOX Ptr


/'* Sets an object speed
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pvSpeed        Speed to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetSpeed(ByVal _pstObject As orxOBJECT Ptr,ByVal _pvSpeed As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets an object speed relative to its rotation/scale
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pvRelativeSpeed Relative speed to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetRelativeSpeed(ByVal _pstObject As orxOBJECT Ptr,ByVal _pvRelativeSpeed As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets an object angular velocity
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _fVelocity      Angular velocity to set (radians/seconds)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetAngularVelocity(ByVal _pstObject As orxOBJECT Ptr,ByVal _fVelocity As orxFLOAT) As orxSTATUS

/'* Sets an object custom gravity
 * @param[in]   _pstObject        Concerned object
 * @param[in]   _pvCustomGravity  Custom gravity to set / orxNULL to remove it
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetCustomGravity(ByVal _pstObject As orxOBJECT Ptr,ByVal _pvCustomGravity As Const orxVECTOR Ptr) As orxSTATUS

/'* Gets an object speed
 * @param[in]   _pstObject      Concerned object
 * @param[out]   _pvSpeed       Speed to get
 * @return      Object speed / orxNULL
 '/
Declare Function orxObject_GetSpeed(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pvSpeed As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets an object relative speed
 * @param[in]   _pstObject      Concerned object
 * @param[out]  _pvRelativeSpeed Relative speed to get
 * @return      Object relative speed / orxNULL
 '/
Declare Function orxObject_GetRelativeSpeed(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pvRelativeSpeed As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets an object angular velocity
 * @param[in]   _pstObject      Concerned object
 * @return      Object angular velocity (radians/seconds)
 '/
Declare Function orxObject_GetAngularVelocity(ByVal _pstObject As Const orxOBJECT Ptr) As orxFLOAT

/'* Gets an object custom gravity
 * @param[in]   _pstObject        Concerned object
 * @param[out]  _pvCustomGravity  Custom gravity to get
 * @return      Object custom gravity / orxNULL is object doesn't have any
 '/
Declare Function orxObject_GetCustomGravity(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pvCustomGravity As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets an object center of mass
 * @param[in]   _pstObject      Concerned object
 * @param[out]  _pvMassCenter   Mass center to get
 * @return      Mass center / orxNULL
 '/
Declare Function orxObject_GetMassCenter(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pvMassCenter As orxVECTOR Ptr) As orxVECTOR Ptr


/'* Sets object text string, if object is associated to a text
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zString        String to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetTextString(ByVal _pstObject As orxOBJECT Ptr,ByVal _zString As Const orxSTRING) As orxSTATUS


/'* Applies a torque
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _fTorque        Torque to apply
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_ApplyTorque(ByVal _pstObject As orxOBJECT Ptr,ByVal _fTorque As orxFLOAT) As orxSTATUS

/'* Applies a force
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pvForce        Force to apply
 * @param[in]   _pvPoint        Point (world coordinates) where the force will be applied, if orxNULL, center of mass will be used
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_ApplyForce(ByVal _pstObject As orxOBJECT Ptr,ByVal _pvForce As Const orxVECTOR Ptr,ByVal _pvPoint As Const orxVECTOR Ptr) As orxSTATUS

/'* Applies an impulse
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pvImpulse      Impulse to apply
 * @param[in]   _pvPoint        Point (world coordinates) where the impulse will be applied, if orxNULL, center of mass will be used
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_ApplyImpulse(ByVal _pstObject As orxOBJECT Ptr,ByVal _pvImpulse As Const orxVECTOR Ptr,ByVal _pvPoint As Const orxVECTOR Ptr) As orxSTATUS


/'* Issues a raycast to test for potential objects in the way
 * @param[in]   _pvStart        Start of raycast
 * @param[in]   _pvEnd          End of raycast
 * @param[in]   _u16SelfFlags   Selfs flags used for filtering (0xFFFF for no filtering)
 * @param[in]   _u16CheckMask   Check mask used for filtering (0xFFFF for no filtering)
 * @param[in]   _pvContact      If non-null and a contact is found it will be stored here
 * @param[in]   _pvNormal       If non-null and a contact is found, its normal will be stored here
 * @return Colliding orxOBJECT / orxNULL
 '/
Declare Function orxObject_Raycast(ByVal _pvStart As Const orxVECTOR Ptr,ByVal _pvEnd As Const orxVECTOR Ptr,ByVal _u16SelfFlags As orxU16,ByVal _u16CheckMask As orxU16,ByVal _pvContact As orxVECTOR Ptr,ByVal _pvNormal As orxVECTOR Ptr) As orxOBJECT Ptr


/'* Sets object color
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _pstColor       Color to set, orxNULL to remove any specifig color
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetColor(ByVal _pstObject As orxOBJECT Ptr,ByVal _pstColor As Const orxCOLOR Ptr) As orxSTATUS

/'* Clears object color
 * @param[in]   _pstObject      Concerned object
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_ClearColor(ByVal _pstObject As orxOBJECT Ptr) As orxSTATUS

/'* Object has color accessor
 * @param[in]   _pstObject      Concerned object
 * @return      orxTRUE / orxFALSE
 '/
Declare Function orxObject_HasColor(ByVal _pstObject As Const orxOBJECT Ptr) As orxBOOL

/'* Gets object color
 * @param[in]   _pstObject      Concerned object
 * @param[out]  _pstColor       Object's color
 * @return      orxCOLOR / orxNULL
 '/
Declare Function orxObject_GetColor(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pstColor As orxCOLOR Ptr) As orxCOLOR Ptr


/'* Adds an FX using its config ID
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zFXConfigID    Config ID of the FX to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_AddFX(ByVal _pstObject As orxOBJECT Ptr,ByVal _zFXConfigID As Const orxSTRING) As orxSTATUS

/'* Adds a unique FX using its config ID
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zFXConfigID    Config ID of the FX to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_AddUniqueFX(ByVal _pstObject As orxOBJECT Ptr,ByVal _zFXConfigID As Const orxSTRING) As orxSTATUS

/'* Adds a delayed FX using its config ID
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zFXConfigID    Config ID of the FX to add
 * @param[in]   _fDelay         Delay time
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_AddDelayedFX(ByVal _pstObject As orxOBJECT Ptr,ByVal _zFXConfigID As Const orxSTRING,ByVal _fDelay As orxFLOAT) As orxSTATUS

/'* Adds a unique delayed FX using its config ID
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zFXConfigID    Config ID of the FX to add
 * @param[in]   _fDelay         Delay time
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_AddUniqueDelayedFX(ByVal _pstObject As orxOBJECT Ptr,ByVal _zFXConfigID As Const orxSTRING,ByVal _fDelay As orxFLOAT) As orxSTATUS

/'* Removes an FX using using its config ID
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zFXConfigID    Config ID of the FX to remove
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_RemoveFX(ByVal _pstObject As orxOBJECT Ptr,ByVal _zFXConfigID As Const orxSTRING) As orxSTATUS


/'* Adds a sound using its config ID
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zSoundConfigID Config ID of the sound to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_AddSound(ByVal _pstObject As orxOBJECT Ptr,ByVal _zSoundConfigID As Const orxSTRING) As orxSTATUS

/'* Removes a sound using using its config ID
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zSoundConfigID Config ID of the sound to remove
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_RemoveSound(ByVal _pstObject As orxOBJECT Ptr,ByVal _zSoundConfigID As Const orxSTRING) As orxSTATUS

/'* Gets last added sound (Do *NOT* destroy it directly before removing it!!!)
 * @param[in]   _pstObject      Concerned object
 * @return      orxSOUND / orxNULL
 '/
Declare Function orxObject_GetLastAddedSound(ByVal _pstObject As Const orxOBJECT Ptr) As orxSOUND Ptr


/'* Adds a shader to an object using its config ID
 * @param[in]   _pstObject        Concerned object
 * @param[in]   _zShaderConfigID  Config ID of the shader to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_AddShader(ByVal _pstObject As orxOBJECT Ptr,ByVal _zShaderConfigID As Const orxSTRING) As orxSTATUS

/'* Removes a shader using using its config ID
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _zShaderConfigID Config ID of the shader to remove
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_RemoveShader(ByVal _pstObject As orxOBJECT Ptr,ByVal _zShaderConfigID As Const orxSTRING) As orxSTATUS

/'* Enables an object's shader
 * @param[in]   _pstObject        Concerned object
 * @param[in]   _bEnable          Enable / disable
 '/
Declare Sub orxObject_EnableShader(ByVal _pstObject As orxOBJECT Ptr,ByVal _bEnable As orxBOOL)

/'* Is an object's shader enabled?
 * @param[in]   _pstObject        Concerned object
 * @return      orxTRUE if enabled, orxFALSE otherwise
 '/
Declare Function orxObject_IsShaderEnabled(ByVal _pstObject As Const orxOBJECT Ptr) As orxBOOL


/'* Gets object config name
 * @param[in]   _pstObject      Concerned object
 * @return      orxSTRING / orxSTRING_EMPTY
 '/
Declare Function orxObject_GetName(ByVal _pstObject As Const orxOBJECT Ptr) As Const orxSTRING


/'* Creates a list of object at neighboring of the given box (ie. whose bounding volume intersects this box)
 * @param[in]   _pstCheckBox    Box to check intersection with
 * @return      orxBANK / orxNULL
 '/
Declare Function orxObject_CreateNeighborList(ByVal _pstCheckBox As Const orxOBOX Ptr) As orxBANK Ptr

/'* Deletes an object list created with orxObject_CreateNeighborList
 * @param[in]   _pstObjectList  Concerned object list
 '/
Declare Sub orxObject_DeleteNeighborList(ByVal _pstObjectList As orxBANK Ptr)


/'* Sets object smoothing
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _eSmoothing     Smoothing type (enabled, default or none)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetSmoothing(ByVal _pstObject As orxOBJECT Ptr,ByVal _eSmoothing As orxDISPLAY_SMOOTHING) As orxSTATUS

/'* Gets object smoothing
 * @param[in]   _pstObject     Concerned object
 * @return Smoothing type (enabled, default or none)
 '/
Declare Function orxObject_GetSmoothing(ByVal _pstObject As Const orxOBJECT Ptr) As orxDISPLAY_SMOOTHING


/'* Sets object repeat (wrap) values
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _fRepeatX       X-axis repeat value
 * @param[in]   _fRepeatY       Y-axis repeat value
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetRepeat(ByVal _pstObject As orxOBJECT Ptr,ByVal _fRepeatX As orxFLOAT,ByVal _fRepeatY As orxFLOAT) As orxSTATUS

/'* Gets object repeat (wrap) values
 * @param[in]   _pstObject     Concerned object
 * @param[out]  _pfRepeatX      X-axis repeat value
 * @param[out]  _pfRepeatY      Y-axis repeat value
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_GetRepeat(ByVal _pstObject As Const orxOBJECT Ptr,ByVal _pfRepeatX As orxFLOAT Ptr,ByVal _pfRepeatY As orxFLOAT Ptr) As orxSTATUS


/'* Sets object blend mode
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _eBlendMode     Blend mode (alpha, multiply, add or none)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetBlendMode(ByVal _pstObject As orxOBJECT Ptr,ByVal _eBlendMode As orxDISPLAY_BLEND_MODE) As orxSTATUS

/'* Gets object blend mode
 * @param[in]   _pstObject     Concerned object
 * @return Blend mode (alpha, multiply, add or none)
 '/
Declare Function orxObject_GetBlendMode(ByVal _pstObject As Const orxOBJECT Ptr) As orxDISPLAY_BLEND_MODE


/'* Sets object lifetime
 * @param[in]   _pstObject      Concerned object
 * @param[in]   _fLifeTime      Lifetime to set, negative value to disable it
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxObject_SetLifeTime(ByVal _pstObject As orxOBJECT Ptr,ByVal _fLifeTime As orxFLOAT) As orxSTATUS

/'* Gets object lifetime
 * @param[in]   _pstObject      Concerned object
 * @return      Lifetime / negative value if none
 '/
Declare Function orxObject_GetLifeTime(ByVal _pstObject As Const orxOBJECT Ptr) As orxFLOAT


/'* Picks the first active object with graphic "under" the given position
 * @param[in]   _pvPosition     Position to pick from
 * @return      orxOBJECT / orxNULL
 '/
Declare Function orxObject_Pick(ByVal _pvPosition As Const orxVECTOR Ptr) As orxOBJECT Ptr

End Extern

#endif /' _orxOBJECT_BI '/

/'* @} '/
