/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxSpawner.h
 * @date 06/09/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxSpawner
 *
 * Spawner module
 * Allows to spawn orxSPAWNERS
 * Spawners derived from structures
 *
 * @{
 '/


#ifndef _orxSPAWNER_BI
#define _orxSPAWNER_BI


#include "orxInclude.bi"
#include "orx/object/orxStructure.bi"
#include "orx/object/orxFrame.bi"


/'* Spawner flags
 '/
#define orxSPAWNER_KU32_FLAG_NONE                 &H00000000  /'*< No flags '/

#define orxSPAWNER_KU32_FLAG_AUTO_DELETE          &H00000001  /'*< Auto delete flag '/
#define orxSPAWNER_KU32_FLAG_AUTO_RESET           &H00000002  /'*< Auto delete flag '/
#define orxSPAWNER_KU32_FLAG_USE_ALPHA            &H00000004  /'*< Use alpha flag '/
#define orxSPAWNER_KU32_FLAG_USE_COLOR            &H00000008  /'*< Use color flag '/
#define orxSPAWNER_KU32_FLAG_USE_ROTATION         &H00000010  /'*< Use rotation flag '/
#define orxSPAWNER_KU32_FLAG_USE_SCALE            &H00000020  /'*< Use scale flag '/
#define orxSPAWNER_KU32_FLAG_USE_RELATIVE_SPEED   &H00000040  /'*< Use relative speed flag '/
#define orxSPAWNER_KU32_FLAG_USE_SELF_AS_PARENT   &H00000080  /'*< Use self as parent flag '/

#define orxSPAWNER_KU32_MASK_USER_ALL             &H000000FF  /'*< All mask '/


/'* Event enum
 '/
Enum orxSPAWNER_EVENT

  orxSPAWNER_EVENT_SPAWN = 0,
  orxSPAWNER_EVENT_DELETE,
  orxSPAWNER_EVENT_RESET,
  orxSPAWNER_EVENT_EMPTY,
  orxSPAWNER_EVENT_WAVE_START,
  orxSPAWNER_EVENT_WAVE_STOP,

  orxSPAWNER_EVENT_NUMBER,

  orxSPAWNER_EVENT_NONE = orxENUM_NONE

End Enum


/'* Internal spawner structure '/
Type __orxSPAWNER_t As Any
Type orxSPAWNER As __orxSPAWNER_t               

Extern "Windows"

/'* Spawner module setup
 '/
Declare Sub orxSpawner_Setup()

/'* Inits the spawner module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_Init() As orxSTATUS

/'* Exits from the spawner module
 '/
Declare Sub orxSpawner_Exit()

/'* Creates an empty spawner
 * @return orxSPAWNER / orxNULL
 '/
Declare Function orxSpawner_Create() As orxSPAWNER Ptr

/'* Creates a spawner from config
 * @param[in]   _zConfigID    Config ID
 * @ return orxSPAWNER / orxNULL
 '/
Declare Function orxSpawner_CreateFromConfig(ByVal _zConfigID As Const orxSTRING) As orxSPAWNER Ptr

/'* Deletes a spawner
 * @param[in] _pstSpawner       Concerned spawner
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_Delete(ByVal _pstSpawner As orxSPAWNER Ptr) As orxSTATUS

/'* Enables/disables a spawner
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _bEnable      Enable / disable
 '/
Declare Sub orxSpawner_Enable(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _bEnable As orxBOOL)

/'* Is spawner enabled?
 * @param[in]   _pstSpawner     Concerned spawner
 * @return      orxTRUE if enabled, orxFALSE otherwise
 '/
Declare Function orxSpawner_IsEnabled(ByVal _pstSpawner As Const orxSPAWNER Ptr) As orxBOOL


/'* Resets (and disables) a spawner
 * @param[in]   _pstSpawner     Concerned spawner
 '/
Declare Sub orxSpawner_Reset(ByVal _pstSpawner As orxSPAWNER Ptr)

/'* Sets spawner total object limit
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _u32TotalObjectLimit Total object limit, 0 for unlimited
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_SetTotalObjectLimit(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _u32TotalObjectLimit As orxU32) As orxSTATUS

/'* Sets spawner active object limit
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _u32ActiveObjectLimit Active object limit, 0 for unlimited
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_SetActiveObjectLimit(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _u32ActiveObjectLimit As orxU32) As orxSTATUS

/'* Gets spawner total object limit
 * @param[in]   _pstSpawner     Concerned spawner
 * @return      Total object limit, 0 for unlimited
 '/
Declare Function orxSpawner_GetTotalObjectLimit(ByVal _pstSpawner As Const orxSPAWNER Ptr) As orxU32

/'* Gets spawner active object limit
 * @param[in]   _pstSpawner     Concerned spawner
 * @return      Active object limit, 0 for unlimited
 '/
Declare Function orxSpawner_GetActiveObjectLimit(ByVal _pstSpawner As Const orxSPAWNER Ptr) As orxU32


/'* Sets spawner wave size
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _u32WaveSize    Number of objects to spawn in a wave / 0 for deactivating wave mode
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_SetWaveSize(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _u32WaveSize As orxU32) As orxSTATUS

/'* Sets spawner wave delay
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _fWaveDelay     Delay between two waves / -1 for deactivating wave mode
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_SetWaveDelay(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _fWaveDelay As orxFLOAT) As orxSTATUS

/'* Gets spawner wave size
 * @param[in]   _pstSpawner     Concerned spawner
 * @return      Number of objects spawned in a wave / 0 if not in wave mode
 '/
Declare Function orxSpawner_GetWaveSize(ByVal _pstSpawner As Const orxSPAWNER Ptr) As orxU32

/'* Gets spawner wave delay
 * @param[in]   _pstSpawner     Concerned spawner
 * @return      Delay between two waves / -1 if not in wave mode
 '/
Declare Function orxSpawner_GetWaveDelay(ByVal _pstSpawner As Const orxSPAWNER Ptr) As orxFLOAT


/'* Sets spawner object speed
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _pvObjectSpeed  Speed to apply to every spawned object / orxNULL to not apply any speed
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_SetObjectSpeed(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _pvObjectSpeed As Const orxVECTOR Ptr) As orxSTATUS

/'* Gets spawner object speed
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _pvObjectSpeed  Speed applied to every spawned object
 * @return      Speed applied to every spawned object / orxNULL if none is applied
 '/
Declare Function orxSpawner_GetObjectSpeed(ByVal _pstSpawner As Const orxSPAWNER Ptr,ByVal _pvObjectSpeed As orxVECTOR Ptr) As orxVECTOR Ptr


/'* Spawns items
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _u32Number      Number of items to spawn
* @return      Number of spawned items
  '/
Declare Function orxSpawner_Spawn(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _u32Number As orxU32) As orxU32


/'* Gets spawner frame
 * @param[in]   _pstSpawner     Concerned spawner
 * @return      orxFRAME
 '/
Declare Function orxSpawner_GetFrame(ByVal _pstSpawner As Const orxSPAWNER Ptr) As orxFRAME Ptr


/'* Sets spawner position
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _pvPosition     Spawner position
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_SetPosition(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _pvPosition As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets spawner rotation
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _fRotation      Spawner rotation (radians)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_SetRotation(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _fRotation As orxFLOAT) As orxSTATUS

/'* Sets spawner scale
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _pvScale        Spawner scale vector
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_SetScale(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _pvScale As Const orxVECTOR Ptr) As orxSTATUS

/'* Get spawner position
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[out]  _pvPosition     Spawner position
 * @return      orxVECTOR / orxNULL
 '/
Declare Function orxSpawner_GetPosition(ByVal _pstSpawner As Const orxSPAWNER Ptr,ByVal _pvPosition As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Get spawner world position
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[out]  _pvPosition     Spawner world position
 * @return      orxVECTOR / orxNULL
 '/
Declare Function orxSpawner_GetWorldPosition(ByVal _pstSpawner As Const orxSPAWNER Ptr,ByVal _pvPosition As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Get spawner rotation
 * @param[in]   _pstSpawner     Concerned spawner
 * @return      orxFLOAT (radians)
 '/
Declare Function orxSpawner_GetRotation(ByVal _pstSpawner As Const orxSPAWNER Ptr) As orxFLOAT

/'* Get spawner world rotation
 * @param[in]   _pstSpawner     Concerned spawner
 * @return      orxFLOAT (radians)
 '/
Declare Function orxSpawner_GetWorldRotation(ByVal _pstSpawner As Const orxSPAWNER Ptr) As orxFLOAT

/'* Get spawner scale
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[out]  _pvScale        Spawner scale vector
 * @return      Scale vector
 '/
Declare Function orxSpawner_GetScale(ByVal _pstSpawner As Const orxSPAWNER Ptr,ByVal _pvScale As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets spawner world scale
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[out]  _pvScale        Spawner world scale
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_GetWorldScale(ByVal _pstSpawner As Const orxSPAWNER Ptr,ByVal _pvScale As orxVECTOR Ptr) As orxVECTOR Ptr


/'* Sets an spawner parent
 * @param[in]   _pstSpawner     Concerned spawner
 * @param[in]   _pParent        Parent structure to set (spawner, spawner, camera or frame) / orxNULL
 * @return      orsSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSpawner_SetParent(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _pParent As Any Ptr) As orxSTATUS


/'* Sets owner for a spawner
 * @param[in]   _pstSpawner   Concerned spawner
 * @param[in]   _pOwner       Owner to set / orxNULL
 '/
Declare Sub orxSpawner_SetOwner(ByVal _pstSpawner As orxSPAWNER Ptr,ByVal _pOwner As Any Ptr)

/'* Gets spawner's owner
 * @param[in]   _pstSpawner   Concerned object
 * @return      Owner / orxNULL
 '/
Declare Function orxSpawner_GetOwner(ByVal _pstSpawner As Const orxSPAWNER Ptr) As orxSTRUCTURE Ptr


/'* Gets spawner name
 * @param[in]   _pstSpawner     Concerned spawner
 * @return      orxSTRING / orxSTRING_EMPTY
 '/
Declare Function orxSpawner_GetName(ByVal _pstSpawner As Const orxSPAWNER Ptr) As Const orxSTRING

End Extern

#endif /' _orxSPAWNER_BI '/

/'* @} '/
