/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxStructure.h
 * @date 08/12/2003
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxStructure
 *
 * Structure module
 * Allows to creates and handle structures.
 * Structures can be referenced by other structures (or objects).
 *
 * @{
 '/


#ifndef _orxSTRUCTURE_BI
#define _orxSTRUCTURE_BI

#include "orxInclude.bi"
#include "orx/core/orxClock.bi"
#include "orx/debug/orxDebug.bi"
#include "orx/memory/orxMemory.bi"

/'* Structure magic number
 '/
#ifdef __orxDEBUG__
  #define orxSTRUCTURE_MAGIC_TAG_ACTIVE   &HDEFACED0
#else
  #define orxSTRUCTURE_MAGIC_TAG_ACTIVE   &H00000000
#endif

#define orxSTRUCTURE_MAGIC_TAG_DELETED    &HDEADC0DE


/'* Structure IDs
 '/
Enum orxSTRUCTURE_ID

  /' *** Following structures can be linked to objects *** '/

  orxSTRUCTURE_ID_ANIMPOINTER = 0,
  orxSTRUCTURE_ID_BODY,
  orxSTRUCTURE_ID_CLOCK,
  orxSTRUCTURE_ID_FRAME,
  orxSTRUCTURE_ID_FXPOINTER,
  orxSTRUCTURE_ID_GRAPHIC,
  orxSTRUCTURE_ID_SHADERPOINTER,
  orxSTRUCTURE_ID_SOUNDPOINTER,
  orxSTRUCTURE_ID_SPAWNER,

  orxSTRUCTURE_ID_LINKABLE_NUMBER,

  /' *** Below this point, structures can not be linked to objects *** '/

  orxSTRUCTURE_ID_ANIM = orxSTRUCTURE_ID_LINKABLE_NUMBER,
  orxSTRUCTURE_ID_ANIMSET,
  orxSTRUCTURE_ID_CAMERA,
  orxSTRUCTURE_ID_FX,
  orxSTRUCTURE_ID_OBJECT,
  orxSTRUCTURE_ID_SHADER,
  orxSTRUCTURE_ID_SOUND,
  orxSTRUCTURE_ID_TEXT,
  orxSTRUCTURE_ID_TEXTURE,
  orxSTRUCTURE_ID_VIEWPORT,

  orxSTRUCTURE_ID_NUMBER,

  orxSTRUCTURE_ID_NONE = orxENUM_NONE

End Enum

/'* Structure storage types
 '/
Enum orxSTRUCTURE_STORAGE_TYPE

  orxSTRUCTURE_STORAGE_TYPE_LINKLIST = 0,
  orxSTRUCTURE_STORAGE_TYPE_TREE,

  orxSTRUCTURE_STORAGE_TYPE_NUMBER,

  orxSTRUCTURE_STORAGE_TYPE_NONE = orxENUM_NONE,

End Enum

/'* Public structure (Must be first derived structure member!)
 '/
Type  __orxSTRUCTURE_t

  As orxSTRUCTURE_ID eID            /'*< Structure ID : 4 '/
  As orxU32          u32RefCounter  /'*< Reference counter : 8 '/
  As orxU32          u32Flags       /'*< Flags : 12 '/
  As orxHANDLE       hStorageNode   /'*< Internal storage node handle : 16 '/

End Type

Type orxSTRUCTURE As __orxSTRUCTURE_t

Extern "Windows"

/'* Structure update callback function type
 '/
Type orxSTRUCTURE_UPDATE_FUNCTION As Function (ByVal _pstStructure As orxSTRUCTURE Ptr, _
																							 ByVal _pstCaller As Const orxSTRUCTURE Ptr, _
																							 ByVal _pstClockInfo As Const orxCLOCK_INFO Ptr _
																							) As orxSTATUS


/'* Gets structure pointer / debug mode
 * @param[in]   _pStructure    Concerned structure
 * @param[in]   _eStructureID   ID to test the structure against
 * @return      Valid orxSTRUCTURE, orxNULL otherwise
 '/
Declare Function _orxStructure_GetPointer(ByVal _pStructure As Const Any Ptr,ByVal _eStructureID As orxSTRUCTURE_ID) As orxSTRUCTURE Ptr


/'* Structure module setup
 '/
Declare Sub orxStructure_Setup()

/'* Initializess the structure module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxStructure_Init() As orxSTATUS

/'* Exits from the structure module
 '/
Declare Sub orxStructure_Exit()


/'* Registers a given ID
 * @param[in]   _eStructureID   Concerned structure ID
 * @param[in]   _eStorageType   Storage type to use for this structure type
 * @param[in]   _eMemoryType    Memory type to store this structure type
 * @param[in]   _u32Size        Structure size
 * @param[in]   _pfnUpdate      Structure update function
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxStructure_Register_(ByVal _eStructureID As orxSTRUCTURE_ID, _
																				ByVal _eStorageType As orxSTRUCTURE_STORAGE_TYPE, _
																				ByVal _eMemoryType As orxMEMORY_TYPE, _
																				ByVal _u32Size As orxU32, _
																				ByVal _pfnUpdate As Const orxSTRUCTURE_UPDATE_FUNCTION _
																			 ) As orxSTATUS

/'* Unregisters a given ID
 * @param[in]   _eStructureID   Concerned structure ID
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxStructure_Unregister(ByVal _eStructureID As orxSTRUCTURE_ID) As orxSTATUS


/'* Creates a clean structure for given type
 * @param[in]   _eStructureID   Concerned structure ID
 * @return      orxSTRUCTURE / orxNULL
 '/
Declare Function orxStructure_Create(ByVal _eStructureID As orxSTRUCTURE_ID) As orxSTRUCTURE Ptr

/'* Deletes a structure (needs to be cleaned beforehand)
 * @param[in]   _pStructure    Concerned structure
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxStructure_Delete(ByVal _pStructure As Any Ptr) As orxSTATUS


/'* Gets structure storage type
 * @param[in]   _eStructureID   Concerned structure ID
 * @return      orxSTRUCTURE_STORAGE_TYPE
 '/
Declare Function orxStructure_GetStorageType(ByVal _eStructureID As orxSTRUCTURE_ID) As orxSTRUCTURE_STORAGE_TYPE

/'* Gets given type structure counter
 * @param[in]   _eStructureID   Concerned structure ID
 * @return      orxU32 / orxU32_UNDEFINED
 '/
Declare Function orxStructure_GetCounter(ByVal _eStructureID As orxSTRUCTURE_ID) As orxU32

/'* Updates structure if update function was registered for the structure type
 * @param[in]   _pStructure    Concerned structure
 * @param[in]   _phCaller       Caller structure
 * @param[in]   _pstClockInfo   Update associated clock info
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxStructure_Update(ByVal _pStructure As Any Ptr, _
																		 ByVal _phCaller As Const Any Ptr, _
																		 ByVal _pstClockInfo As Const orxCLOCK_INFO Ptr _
																		) As orxSTATUS


/'* *** Structure storage accessors *** '/


/'* Gets first stored structure (first list cell or tree root depending on storage type)
 * @param[in]   _eStructureID   Concerned structure ID
 * @return      orxSTRUCTURE
 '/
Declare Function orxStructure_GetFirst(ByVal _eStructureID As orxSTRUCTURE_ID) As orxSTRUCTURE Ptr

/'* Gets last stored structure (last list cell or tree root depending on storage type)
 * @param[in]   _eStructureID   Concerned structure ID
 * @return      orxSTRUCTURE
 '/
Declare Function orxStructure_GetLast(ByVal _eStructureID As orxSTRUCTURE_ID) As orxSTRUCTURE Ptr

/'* Gets structure tree parent
 * @param[in]   _pStructure    Concerned structure
 * @return      orxSTRUCTURE
 '/
Declare Function orxStructure_GetParent(ByVal _pStructure As Const Any Ptr) As orxSTRUCTURE Ptr

/'* Gets structure tree child
 * @param[in]   _pStructure    Concerned structure
 * @return      orxSTRUCTURE
 '/
Declare Function orxStructure_GetChild(ByVal _pStructure As Const Any Ptr) As orxSTRUCTURE Ptr

/'* Gets structure tree sibling
 * @param[in]   _pStructure    Concerned structure
 * @return      orxSTRUCTURE
 '/
Declare Function orxStructure_GetSibling(ByVal _pStructure As Const Any Ptr) As orxSTRUCTURE Ptr

/'* Gets structure list previous
 * @param[in]   _pStructure    Concerned structure
 * @return      orxSTRUCTURE
 '/
Declare Function orxStructure_GetPrevious(ByVal _pStructure As Const Any Ptr) As orxSTRUCTURE Ptr

/'* Gets structure list next
 * @param[in]   _pStructure    Concerned structure
 * @return      orxSTRUCTURE
 '/
Declare Function orxStructure_GetNext(ByVal _pStructure As Const Any Ptr) As orxSTRUCTURE Ptr

/'* Sets structure tree parent
 * @param[in]   _pStructure    Concerned structure
 * @param[in]   _phParent       Structure to set as parent
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxStructure_SetParent(ByVal _pStructure As Any Ptr,ByVal _phParent As Any Ptr) As orxSTATUS

End Extern

#endif /' _orxSTRUCTURE_BI '/

/'* @} '/
