/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxBody.h
 * @date 10/03/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxBody
 * 
 * Body Module
 * Allows to creates and handle physical bodies
 * They are used as container with associated properties
 * Bodies are used by objects
 * They thus can be referenced by objects as structures
 *
 * @{
 '/


#ifndef _orxBODY_BI
#define _orxBODY_BI

#include "orxInclude.bi"
#include "orx/object/orxStructure.bi"
#include "orx/physics/orxPhysics.bi"

/'* Misc defines
 '/
#define orxBODY_KU32_PART_MAX_NUMBER          8


/'* Internal Body structure
 '/
Type __orxBODY_t As Any
Type orxBODY As __orxBODY_t

/'* Internal Body part structure
 '/
Type __orxBODY_PART_t As Any
Type orxBODY_PART As __orxBODY_PART_t


/'* Body module setup
 '/
Declare Sub orxBody_Setup()

/'* Inits the Body module
 '/
Declare Function orxBody_Init() As orxSTATUS

/'* Exits from the Body module
 '/
Declare Sub orxBody_Exit()


/'* Creates an empty body
 * @param[in]   _pstOwner                     Body's owner used for collision callbacks (usually an orxOBJECT)
 * @param[in]   _pstBodyDef                   Body definition
 * @return      Created orxGRAPHIC / orxNULL
 '/
Declare Function orxBody_Create(ByVal _pstOwner As Const orxSTRUCTURE Ptr,ByVal _pstBodyDef As Const orxBODY_DEF Ptr) As orxBODY Ptr

/'* Creates a body from config
 * @param[in]   _pstOwner                     Body's owner used for collision callbacks (usually an orxOBJECT)
 * @param[in]   _zConfigID                    Body config ID
 * @return      Created orxGRAPHIC / orxNULL
 '/
Declare Function orxBody_CreateFromConfig(ByVal _pstOwner As Const orxSTRUCTURE Ptr,ByVal _zConfigID As Const orxSTRING) As orxBODY Ptr

/'* Deletes a body
 * @param[in]   _pstBody        Concerned body
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_Delete(ByVal _pstBody As orxBODY Ptr) As orxSTATUS

/'* Gets a body owner
 * @param[in]   _pstBody        Concerned body
 * @return      orxSTRUCTURE / orxNULL
 '/
Declare Function orxBody_GetOwner(ByVal _pstBody As Const orxBODY Ptr) As orxSTRUCTURE Ptr

/'* Adds a part to body
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _u32Index       Part index (should be less than orxBODY_KU32_PART_MAX_NUMBER)
 * @param[in]   _pstBodyPartDef Body part definition
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_AddPart(ByVal _pstBody As orxBODY Ptr,ByVal _u32Index As orxU32,ByVal _pstBodyPartDef As Const orxBODY_PART_DEF Ptr) As orxSTATUS

/'* Adds a part to body from config
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _u32Index       Part index (should be less than orxBODY_KU32_PART_MAX_NUMBER)
 * @param[in]   _zConfigID      Body part config ID
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_AddPartFromConfig(ByVal _pstBody As orxBODY Ptr,ByVal _u32Index As orxU32,ByVal _zConfigID As Const orxSTRING) As orxSTATUS

/'* Gets a body part
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _u32Index       Body part index (should be less than orxBODY_KU32_DATA_MAX_NUMBER)
 * @return      orxPHYSICS_BODY_PART / orxNULL
 '/
Declare Function orxBody_GetPart(ByVal _pstBody As Const orxBODY Ptr,ByVal _u32Index As orxU32) As orxPHYSICS_BODY_PART Ptr

/'* Gets a body part name
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _u32Index       Part index (should be less than orxBODY_KU32_DATA_MAX_NUMBER)
 * @return      orxSTRING / orxNULL
 '/
Declare Function orxBody_GetPartName(ByVal _pstBody As Const orxBODY Ptr,ByVal _u32Index As orxU32) As Const orxSTRING

/'* Removes a body part
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _u32Index       Part index (should be less than orxBODY_KU32_DATA_MAX_NUMBER)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_RemovePart(ByVal _pstBody As orxBODY Ptr,ByVal _u32Index As orxU32) As orxSTATUS


/'* Sets a body template
 * @param[in]   _pstBodyTemplate  Body template to set / orxNULL to remove it
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_SetTemplate(ByVal _pstBodyTemplate As Const orxBODY_DEF Ptr) As orxSTATUS

/'* Sets a body part template
 * @param[in]   _pstBodyPartTemplate  Body part template to set / orxNULL to remove it
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_SetPartTemplate(ByVal _pstBodyPartTemplate As Const orxBODY_PART_DEF Ptr) As orxSTATUS

/'* Gets the body template
 * @param[out]  _pstBodyTemplate  Body template to get
 * @return      orxBODY_DEF / orxNULL
 '/
Declare Function orxBody_GetTemplate(ByVal _pstBodyTemplate As orxBODY_DEF Ptr) As orxBODY_DEF Ptr

/'* Gets the body part template
 * @param[out]  _pstBodyPartTemplate  Body part template to get
 * @return      orxBODY_PART_DEF / orxNULL
 '/
Declare Function orxBody_GetPartTemplate(ByVal _pstBodyPartTemplate As orxBODY_PART_DEF Ptr) As orxBODY_PART_DEF Ptr

/'* Sets a body position
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _pvPosition     Position to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_SetPosition(ByVal _pstBody As orxBODY Ptr,ByVal _pvPosition As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets a body rotation
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _fRotation      Rotation to set (radians)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_SetRotation(ByVal _pstBody As orxBODY Ptr,ByVal _fRotation As orxFLOAT) As orxSTATUS

/'* Sets a body scale
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _pvScale        Scale to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_SetScale(ByVal _pstBody As orxBODY Ptr,ByVal _pvScale As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets a body speed
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _pvSpeed        Speed to set
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_SetSpeed(ByVal _pstBody As orxBODY Ptr,ByVal _pvSpeed As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets a body angular velocity
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _fVelocity      Angular velocity to set (radians/seconds)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_SetAngularVelocity(ByVal _pstBody As orxBODY Ptr,ByVal _fVelocity As orxFLOAT) As orxSTATUS

/'* Sets a body custom gravity
 * @param[in]   _pstBody          Concerned body
 * @param[in]   _pvCustomGravity  Custom gravity to set / orxNULL to remove it
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_SetCustomGravity(ByVal _pstBody As orxBODY Ptr,ByVal _pvCustomGravity As Const orxVECTOR Ptr) As orxSTATUS

/'* Gets a body position
 * @param[in]   _pstBody        Concerned body
 * @param[out]  _pvPosition     Position to get
 * @return      Body position / orxNULL
 '/
Declare Function orxBody_GetPosition(ByVal _pstBody As Const orxBODY Ptr,ByVal _pvPosition As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets a body rotation
 * @param[in]   _pstBody        Concerned body
 * @return      Body rotation (radians)
 '/
Declare Function orxBody_GetRotation(ByVal _pstBody As Const orxBODY Ptr) As orxFLOAT

/'* Gets a body speed
 * @param[in]   _pstBody        Concerned body
 * @param[out]   _pvSpeed       Speed to get
 * @return      Body speed / orxNULL
 '/
Declare Function orxBody_GetSpeed(ByVal _pstBody As Const orxBODY Ptr,ByVal _pvSpeed As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets a body angular velocity
 * @param[in]   _pstBody        Concerned body
 * @return      Body angular velocity (radians/seconds)
 '/
Declare Function orxBody_GetAngularVelocity(ByVal _pstBody As Const orxBODY Ptr) As orxFLOAT

/'* Gets a body custom gravity
 * @param[in]   _pstBody          Concerned body
 * @param[out]  _pvCustomGravity  Custom gravity to get
 * @return      Body custom gravity / orxNULL is object doesn't have any
 '/
Declare Function orxBody_GetCustomGravity(ByVal _pstBody As Const orxBODY Ptr,ByVal _pvCustomGravity As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets a body center of mass
 * @param[in]   _pstBody        Concerned body
 * @param[out]  _pvMassCenter   Mass center to get
 * @return      Mass center / orxNULL
 '/
Declare Function orxBody_GetMassCenter(ByVal _pstBody As Const orxBODY Ptr,ByVal _pvMassCenter As orxVECTOR Ptr) As orxVECTOR Ptr


/'* Applies a torque
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _fTorque        Torque to apply
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_ApplyTorque(ByVal _pstBody As orxBODY Ptr,ByVal _fTorque As orxFLOAT) As orxSTATUS

/'* Applies a force
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _pvForce        Force to apply
 * @param[in]   _pvPoint        Point (world coordinates) where the force will be applied, if orxNULL, center of mass will be used
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_ApplyForce(ByVal _pstBody As orxBODY Ptr,ByVal _pvForce As Const orxVECTOR Ptr,ByVal _pvPoint As Const orxVECTOR Ptr) As orxSTATUS

/'* Applies an impulse
 * @param[in]   _pstBody        Concerned body
 * @param[in]   _pvImpulse      Impulse to apply
 * @param[in]   _pvPoint        Point (world coordinates) where the impulse will be applied, if orxNULL, center of mass will be used
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxBody_ApplyImpulse(ByVal _pstBody As orxBODY Ptr,ByVal _pvImpulse As Const orxVECTOR Ptr,ByVal _pvPoint As Const orxVECTOR Ptr) As orxSTATUS


/'* Issues a raycast to test for potential bodies in the way
 * @param[in]   _pvStart        Start of raycast
 * @param[in]   _pvEnd          End of raycast
 * @param[in]   _u16SelfFlags   Selfs flags used for filtering (0xFFFF for no filtering)
 * @param[in]   _u16CheckMask   Check mask used for filtering (0xFFFF for no filtering)
 * @param[in]   _pvContact      If non-null and a contact is found it will be stored here
 * @param[in]   _pvNormal       If non-null and a contact is found, its normal will be stored here
 * @return Colliding orxBODY / orxNULL
 '/
Declare Function orxBody_Raycast(ByVal _pvStart As Const orxVECTOR Ptr,ByVal _pvEnd As Const orxVECTOR Ptr,ByVal _u16SelfFlags As orxU16,ByVal _u16CheckMask As orxU16,ByVal _pvContact As orxVECTOR Ptr,ByVal _pvNormal As orxVECTOR Ptr) As orxBODY Ptr


/'* Applies physics simulation result to the Body
 * @param[in]   _pstBody                      Concerned body
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Sub orxBody_ApplySimulationResult(ByVal _pstBody As orxBODY Ptr)

#endif /' _orxBODY_BI '/

/'* @} '/
