/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxPhysics.h
 * @date 24/03/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxPhysics
 * 
 * Physics plugin module
 * Module that handles physics interactions
 *
 * @{
 '/


#ifndef _orxPHYSICS_BI
#define _orxPHYSICS_BI


#include "orxInclude.bi"
''#include "orx/plugin/orxPluginCore.bi"
#include "orx/math/orxAABox.bi"


/'* Body definition flags
 '/
#define orxBODY_DEF_KU32_FLAG_NONE                &H00000000  /'*< No flags '/

#define orxBODY_DEF_KU32_FLAG_2D                  &H00000001  /'*< 2D type body def flag '/
#define orxBODY_DEF_KU32_FLAG_DYNAMIC             &H00000002  /'*< Dynamic type body def flag '/
#define orxBODY_DEF_KU32_FLAG_HIGH_SPEED          &H00000004  /'*< High speed type body def flag '/
#define orxBODY_DEF_KU32_FLAG_FIXED_ROTATION      &H00000008  /'*< Body can't be rotated by physics '/
#define orxBODY_DEF_KU32_FLAG_CAN_SLIDE           &H00000010  /'*< Body is allowed to slide '/

#define orxBODY_DEF_KU32_MASK_ALL                 &HFFFFFFFF  /'*< Body def all mask '/

/'* Body part definition flags
 '/
#define orxBODY_PART_DEF_KU32_FLAG_NONE           &H00000000  /'*< No flags '/

#define orxBODY_PART_DEF_KU32_FLAG_SPHERE         &H00000001  /'*< Sphere body part def flag '/
#define orxBODY_PART_DEF_KU32_FLAG_BOX            &H00000002  /'*< Box body part def flag '/
#define orxBODY_PART_DEF_KU32_FLAG_MESH           &H00000004  /'*< Mesh body part def flag '/
#define orxBODY_PART_DEF_KU32_FLAG_SOLID          &H00000010  /'*< Solid body part def flag '/

#define orxBODY_PART_DEF_KU32_MASK_ALL            &HFFFFFFFF  /'*< Body part def all mask '/


/'* Misc defines
 '/
#define orxBODY_PART_DEF_KU32_MESH_VERTEX_NUMBER  8


/'* Body definition
 '/
Type __orxBODY_DEF_t

  As orxVECTOR vPosition                      /'*< Position : 12 '/
  As orxFLOAT  fRotation                      /'*< Rotation : 16 '/
  As orxFLOAT  fInertia                       /'*< Inertia : 20 '/
  As orxFLOAT  fMass                          /'*< Mass : 24 '/
  As orxFLOAT  fLinearDamping                 /'*< Linear damping : 28 '/
  As orxFLOAT  fAngularDamping                /'*< Angular damping : 32 '/
  As orxU32    u32Flags                       /'*< Control flags : 36 '/

End Type

Type orxBODY_DEF As __orxBODY_DEF_t

Type orxBODY_PART_DEF As __orxBODY_DEF_t

/'* Shape definition
 '/
Type __orxBODY_PART_DEF_t

  As orxVECTOR vScale                         /'*< Scale : 12 '/
  As orxFLOAT  fFriction                      /'*< Friction : 16 '/
  As orxFLOAT  fRestitution                   /'*< Restitution : 20 '/
  As orxFLOAT  fDensity                       /'*< Density : 24 '/
  As orxU16    u16SelfFlags                   /'*< Self defining flags : 26 '/
  As orxU16    u16CheckMask                   /'*< Check mask : 28 '/
  As orxU32    u32Flags                       /'*< Control flags : 32 '/

  Union
  
    Type    
      As orxVECTOR vCenter                    /'*< Sphere center : 44 '/
      As orxFLOAT  fRadius                    /'*< Sphere radius : 48 '/
    End Type '' stSphere                             /'*< Sphere : 48 '/

    Type    
      As orxAABOX  stBox                      /'*< Axis aligned Box : 56 '/
    End Type  ''stAABox                              /'*< Box : 56 '/

    Type    
      As orxU32    u32VertexCounter           /'*< Mesh vertex counter : 36 '/
      As orxVECTOR avVertices(orxBODY_PART_DEF_KU32_MESH_VERTEX_NUMBER)  /'*< Mesh vertices : 132 '/
    End Type ''stMesh
		
  End Union                                        /'*< Shape : 132 '/
End Type


/'* Event enum
 '/
Enum orxPHYSICS_EVENT

  orxPHYSICS_EVENT_CONTACT_ADD = 0,
  orxPHYSICS_EVENT_CONTACT_REMOVE,
  orxPHYSICS_EVENT_OUT_OF_WORLD,

  orxPHYSICS_EVENT_NUMBER,

  orxPHYSICS_EVENT_NONE = orxENUM_NONE

End Enum 


/'* Contact event payload
 '/
Type __orxPHYSICS_EVENT_PAYLOAD_t

  As orxVECTOR vPosition                    /'*< Contact position : 12 '/
  As orxVECTOR vNormal                      /'*< Contact normal : 24 '/
  As orxSTRING zSenderPartName              /'*< Sender part name : 28 '/
  As orxSTRING zRecipientPartName           /'*< Recipient part name : 32 '/

End Type

Type orxPHYSICS_EVENT_PAYLOAD As __orxPHYSICS_EVENT_PAYLOAD_t

/'* Internal physics body structure
 '/
Type orxPHYSICS_BODY As Any

/'* Internal physics shape structure
 '/
Type orxPHYSICS_BODY_PART As Any


/'* Config defines
 '/
#define orxPHYSICS_KZ_CONFIG_SECTION      "Physics"
#define orxPHYSICS_KZ_CONFIG_GRAVITY      "Gravity"
#define orxPHYSICS_KZ_CONFIG_ALLOW_SLEEP  "AllowSleep"
#define orxPHYSICS_KZ_CONFIG_WORLD_LOWER  "WorldLowerBound"
#define orxPHYSICS_KZ_CONFIG_WORLD_UPPER  "WorldUpperBound"
#define orxPHYSICS_KZ_CONFIG_ITERATIONS   "IterationsPerStep"
#define orxPHYSICS_KZ_CONFIG_FREQUENCY    "SimulationFrequency"
#define orxPHYSICS_KZ_CONFIG_RATIO        "DimensionRatio"
#define orxPHYSICS_KZ_CONFIG_FIXED_DT     "FixedDT"


/'**************************************************************************
 * Functions directly implemented by orx core
 **************************************************************************'/

/'* Physics module setup
 '/
Declare Sub orxPhysics_Setup()


/'**************************************************************************
 * Functions extended by plugins
 **************************************************************************'/

/'* Inits the physics module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxPhysics_Init() As orxSTATUS

/'* Exits from the physics module
 '/
Declare Sub orxPhysics_Exit()


/'* Sets physics gravity
 * @param[in]   _pvGravity                            Gravity to set
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxPhysics_SetGravity(ByVal _pvGravity As Const orxVECTOR Ptr) As orxSTATUS

/'* Gets physics gravity
 * @param[in]   _pvGravity                            Gravity to get
 * @return orxVECTOR / orxNULL
 '/
Declare Function orxPhysics_GetGravity(ByVal _pvGravity As orxVECTOR Ptr) As orxVECTOR Ptr


/'* Creates a physical body
 * @param[in]   _hUserData                            User data to associate with this physical body
 * @param[in]   _pstBodyDef                           Physical body definition
 * @return orxPHYSICS_BODY / orxNULL
 '/
Declare Function orxPhysics_CreateBody(ByVal _hUserData As Const orxHANDLE,ByVal _pstBodyDef As Const orxBODY_DEF Ptr) As orxPHYSICS_BODY Ptr

/'* Deletes a physical body
 * @param[in]   _pstBody                              Concerned physical body
 '/
Declare Sub orxPhysics_DeleteBody(ByVal _pstBody As orxPHYSICS_BODY Ptr)

/'* Creates a part for a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[in]   _pstBodyPartDef                       Physical body part definition
 * @return orxPHYSICS_BODY_PART / orxNULL
 '/
Declare Function orxPhysics_CreateBodyPart(ByVal _pstBody As orxPHYSICS_BODY Ptr,ByVal _pstBodyPartDef As Const orxBODY_PART_DEF Ptr) As orxPHYSICS_BODY_PART Ptr

/'* Deletes a physical body part
 * @param[in]   _pstBodyPart                          Concerned physical body part
 '/
Declare Sub orxPhysics_DeleteBodyPart(ByVal _pstBodyPart As orxPHYSICS_BODY_PART Ptr)


/'* Sets the position of a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[in]   _pvPosition                           Position to set
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxPhysics_SetPosition(ByVal _pstBody As orxPHYSICS_BODY Ptr,ByVal _pvPosition As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets the rotation of a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[in]   _fRotation                            Rotation (radians) to set
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxPhysics_SetRotation(ByVal _pstBody As orxPHYSICS_BODY Ptr,ByVal _fRotation As orxFLOAT) As orxSTATUS

/'* Sets the speed of a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[in]   _pvSpeed                              Speed to set
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxPhysics_SetSpeed(ByVal _pstBody As orxPHYSICS_BODY Ptr,ByVal _pvSpeed As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets the angular velocity of a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[in]   _fVelocity                            Angular velocity (radians/seconds) to set
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxPhysics_SetAngularVelocity(ByVal _pstBody As orxPHYSICS_BODY Ptr,ByVal _fVelocity As orxFLOAT) As orxSTATUS

/'* Sets the custom gravity of a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[in]   _pvCustomGravity                      Custom gravity multiplier to set / orxNULL to remove it
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxPhysics_SetCustomGravity(ByVal _pstBody As orxPHYSICS_BODY Ptr,ByVal _pvCustomGravity As Const orxVECTOR Ptr) As orxSTATUS

/'* Gets the position of a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[out]  _pvPosition                           Position to get
 * @return Position of the physical body
 '/
Declare Function orxPhysics_GetPosition(ByVal _pstBody As Const orxPHYSICS_BODY Ptr,ByVal _pvPosition As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets the rotation of a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @return Rotation (radians) of the physical body
 '/
Declare Function orxPhysics_GetRotation(ByVal _pstBody As Const orxPHYSICS_BODY Ptr) As orxFLOAT

/'* Gets the speed of a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[out]  _pvSpeed                              Speed to get
 * @return Speed of the physical body
 '/
Declare Function orxPhysics_GetSpeed(ByVal _pstBody As Const orxPHYSICS_BODY Ptr,ByVal _pvSpeed As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets the angular velocity of a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @return Angular velocity (radians/seconds) of the physical body
 '/
Declare Function orxPhysics_GetAngularVelocity(ByVal _pstBody As Const orxPHYSICS_BODY Ptr) As orxFLOAT

/'* Gets the custom gravity of a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[out]  _pvCustomGravity                      Custom gravity to get
 * @return      Physical body custom gravity / orxNULL is object doesn't have any
 '/
Declare Function orxPhysics_GetCustomGravity(ByVal _pstBody As Const orxPHYSICS_BODY Ptr,ByVal _pvCustomGravity As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets the center of mass of a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[out]  _pvMassCenter                         Center of mass to get
 * @return Center of mass of the physical body
 '/
Declare Function orxPhysics_GetMassCenter(ByVal _pstBody As Const orxPHYSICS_BODY Ptr,ByVal _pvMassCenter As orxVECTOR Ptr) As orxVECTOR Ptr


/'* Applies a torque to a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[in]   _fTorque                              Torque to apply
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxPhysics_ApplyTorque(ByVal _pstBody As orxPHYSICS_BODY Ptr,ByVal _fTorque As orxFLOAT) As orxSTATUS

/'* Applies a force to a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[in]   _pvForce                              Force to apply
 * @param[in]   _pvPoint                              Point of application (if null, center of mass will be used)
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxPhysics_ApplyForce(ByVal _pstBody As orxPHYSICS_BODY Ptr,ByVal _pvForce As Const orxVECTOR Ptr,ByVal _pvPoint As Const orxVECTOR Ptr) As orxSTATUS

/'* Applies an impulse to a physical body
 * @param[in]   _pstBody                              Concerned physical body
 * @param[in]   _pvImpulse                            Impulse to apply
 * @param[in]   _pvPoint                              Point of application (if null, center of mass will be used)
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxPhysics_ApplyImpulse(ByVal _pstBody As orxPHYSICS_BODY Ptr,ByVal _pvImpulse As Const orxVECTOR Ptr,ByVal _pvPoint As Const orxVECTOR Ptr) As orxSTATUS


/'* Issues a raycast to test for potential physics bodies in the way
 * @param[in]   _pvStart                              Start of raycast
 * @param[in]   _pvEnd                                End of raycast
 * @param[in]   _u16SelfFlags                         Selfs flags used for filtering (&HFFFF for no filtering)
 * @param[in]   _u16CheckMask                         Check mask used for filtering (&HFFFF for no filtering)
 * @param[in]   _pvContact                            If non-null and a contact is found it will be stored here
 * @param[in]   _pvNormal                             If non-null and a contact is found, its normal will be stored here
 * @return Colliding body's user data / orxHANDLE_UNDEFINED
 '/
Declare Function orxPhysics_Raycast(ByVal _pvStart As Const orxVECTOR Ptr,ByVal _pvEnd As Const orxVECTOR Ptr,ByVal _u16SelfFlags As orxU16,ByVal _u16CheckMask As orxU16,ByVal _pvContact As orxVECTOR Ptr,ByVal _pvNormal As orxVECTOR Ptr) As orxHANDLE


/'* Enables/disables physics simulation
 * @param[in]   _bEnable                              Enable / disable
 '/
Declare Sub orxPhysics_EnableSimulation(ByVal _bEnable As orxBOOL)

#endif /' _orxPHYSICS_BI '/

/'* @} '/
