/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxPluginCore.h
 * @date 24/04/2003
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxPlugin
 * 
 * Plugin core header
 * This header is used to define structures & functions for core plugins info.
 *
 * @{
 '/


#ifndef _orxPLUGIN_CORE_BI
#define _orxPLUGIN_CORE_BI


/'********************************************
 Constants
 ********************************************'/

#include "orxInclude.bi"

#include "orx/plugin/orxPluginType.bi"
#include "orx/plugin/define/orxPlugin_CoreDefine.bi"


/'********************************************
 Structures
 ********************************************'/
 
/' Structure '/
Type __orxPLUGIN_CORE_FUNCTION_t

  As orxPLUGIN_FUNCTION_ID eFunctionID        /'*< Function ID : 4 '/
  As orxPLUGIN_FUNCTION   Ptr pfnFunction        /'*< Function Address : 8 '/
  As orxPLUGIN_FUNCTION    pfnDefaultFunction /'*< Default Function : 12 '/

End Type

Type orxPLUGIN_CORE_FUNCTION As __orxPLUGIN_CORE_FUNCTION_t

/' Structure '/
Type __orxPLUGIN_USER_FUNCTION_INFO_t

  As orxPLUGIN_FUNCTION_ID eFunctionID                        /'*< Function ID '/
  As orxPLUGIN_FUNCTION pfnFunction                           /'*< Function Address '/
  As orxCHAR zFunctionArgs(orxPLUGIN_KU32_FUNCTION_ARG_SIZE)  /'*< Function Argument Types '/
  As orxSTRING zFunctionName                                  /'*< Function Name '/

End Type

Type orxPLUGIN_USER_FUNCTION_INFO As __orxPLUGIN_USER_FUNCTION_INFO_t


/'* Plugin init function prototype
 '/
Type orxPLUGIN_INIT_FUNCTION As Function (ByVal _peUserFunctionNumber As orxU32 Ptr, _
                                          ByVal _pastUserFunctionInfo As orxPLUGIN_USER_FUNCTION_INFO Ptr Ptr _
																				 ) As orxSTATUS


/'********************************************
 Function prototypes
 ********************************************'/

/'* Adds an info structure for the given core module
 * Has to be called during a core module init
 * @param[in] _ePluginCoreID          The numeric id of the core plugin
 * @param[in] _eModuleID              Corresponding module ID
 * @param[in] _astCoreFunction        The pointer on the core functions info array
 * @param[in] _u32CoreFunctionNumber  Number of functions in the array
 * @return nothing.
 '/
Declare Sub orxPlugin_AddCoreInfo(ByVal _ePluginCoreID As orxPLUGIN_CORE_ID,_
                                  ByVal _eModuleID As orxMODULE_ID, _
																	ByVal _astCoreFunction As Const orxPLUGIN_CORE_FUNCTION Ptr, _
																	ByVal _u32CoreFunctionNumber As orxU32)

#ifdef __orxEMBEDDED__

/'* Binds a core plugin to its embedded implementation
 * Has to be called during a core module init
 * @param[in] _ePluginCoreID          The numeric id of the core plugin
 * @param[in] _pfnPluginInit          Embedded plug-in init function
 * @return nothing
 '/
Declare Sub orxPlugin_BindCoreInfo(ByVal _ePluginCoreID As orxPLUGIN_CORE_ID, _
                                   ByVal _pfnPluginInit As orxPLUGIN_INIT_FUNCTION)

#endif /' __orxEMBEDDED__ '/

/'* Default core plugin function
 * Needs to be referenced by all core functions at module init.
 '/
Declare Function orxPlugin_DefaultCoreFunction(ByVal _zFunctionName As Const orxSTRING, _
                                               ByVal _zFileName As Const orxSTRING, _
																							 ByVal _u32Line As orxU32) As Any Ptr


/'**************************************************************************
 ***************************************************************************
 ******                 CORE FUNCTION NAME HANDLING                   ******
 ***************************************************************************
 **************************************************************************'/

/' *** Core function pointer name *** '/
#define orxPLUGIN_CORE_FUNCTION_POINTER_NAME(FUNCTION_NAME) _orxCoreFunctionPointer_##FUNCTION_NAME

/' *** Core function default name *** '/
#define orxPLUGIN_DEFAULT_CORE_FUNCTION_NAME(FUNCTION_NAME) _orxDefaultCoreFunction_##FUNCTION_NAME

/' *** Default core function declaration *** '/

#define orxPLUGIN_DEFAULT_CORE_FUNCTION_DEFINE(FUNCTION_NAME)  _
  Extern orxFASTCALL orxPLUGIN_DEFAULT_CORE_FUNCTION_NAME(FUNCTION_NAME)() As Any Ptr :_
                                                                                      :_
    Return(orxPlugin_DefaultCoreFunction(#FUNCTION_NAME, __FILE__, __LINE__))         :_              
  End Function


/'**************************************************************************
 ***************************************************************************
 ******             CORE FUNCTION DEFINITION HANDLING                 ******
 ***************************************************************************
 **************************************************************************'/

/' *** Definition macro *** '/
'#ifdef __orxMSVC__
'#pragma warning(disable : 4113)
'#endif /' __orxMSVC__ '/

'#define orxPLUGIN_DEFINE_CORE_FUNCTION(FUNCTION_NAME, RETURN, ...)          \
  'orxPLUGIN_DEFAULT_CORE_FUNCTION_DEFINE(FUNCTION_NAME)                     \
                                                                            '\
  'RETURN (orxFASTCALL *orxPLUGIN_CORE_FUNCTION_POINTER_NAME(FUNCTION_NAME))(__VA_ARGS__) = (RETURN(orxFASTCALL *)(__VA_ARGS__)) (&orxPLUGIN_DEFAULT_CORE_FUNCTION_NAME(FUNCTION_NAME))


/'**************************************************************************
 ***************************************************************************
 ******                   CORE INFO ARRAY HANDLING                    ******
 ***************************************************************************
 **************************************************************************'/
 
'/' *** Core info array begin macro *** '/
'#define orxPLUGIN_BEGIN_CORE_FUNCTION_ARRAY(PLUGIN_SUFFIX)                                                                            \
  'static const orxPLUGIN_CORE_FUNCTION sastPluginFunctionInfo_##PLUGIN_SUFFIX[orxPLUGIN_FUNCTION_BASE_ID_##PLUGIN_SUFFIX##_NUMBER] =  \
  '{

'/' *** Core info array add macro *** '/
'#define orxPLUGIN_ADD_CORE_FUNCTION_ARRAY(PLUGIN_SUFFIX, FUNCTION_SUFFIX, FUNCTION_NAME) \
  '{(orxPLUGIN_FUNCTION_ID)orxPLUGIN_FUNCTION_BASE_ID_##PLUGIN_SUFFIX##_##FUNCTION_SUFFIX, (orxPLUGIN_FUNCTION *)&orxPLUGIN_CORE_FUNCTION_POINTER_NAME(FUNCTION_NAME), (orxPLUGIN_FUNCTION)&orxPLUGIN_DEFAULT_CORE_FUNCTION_NAME(FUNCTION_NAME)},

'/' *** Core info array end macro *** '/
'#define orxPLUGIN_END_CORE_FUNCTION_ARRAY(PLUGIN_SUFFIX)                        \
  '};                                                                            \
  'extern orxIMPORT orxSTATUS orxPLUGIN_K_CORE_INIT_FUNCTION_NAME(PLUGIN_SUFFIX)(orxU32 *, orxPLUGIN_USER_FUNCTION_INFO **); \
  'extern orxDLLAPI void orxPLUGIN_CORE_REGISTER_FUNCTION_NAME(PLUGIN_SUFFIX)()  \
  '{                                                                             \
    'orxPLUGIN_REGISTER_CORE_INFO(PLUGIN_SUFFIX);                                \
    'return;                                                                     \
  '}


/'**************************************************************************
 ***************************************************************************
 ******               CORE INFO REGISTRATION HANDLING                 ******
 ***************************************************************************
 **************************************************************************'/

  '#ifdef __orxEMBEDDED__

  '/' *** Core info register macro *** '/
  '#define orxPLUGIN_REGISTER_CORE_INFO(PLUGIN_SUFFIX)  _
    'orxPlugin_AddCoreInfo(orxPLUGIN_CORE_ID_##PLUGIN_SUFFIX, orxMODULE_ID_##PLUGIN_SUFFIX, sastPluginFunctionInfo_##PLUGIN_SUFFIX, sizeof(sastPluginFunctionInfo_##PLUGIN_SUFFIX) / SizeOf(orxPLUGIN_CORE_FUNCTION)):_
    'orxPlugin_BindCoreInfo(orxPLUGIN_CORE_ID_##PLUGIN_SUFFIX, orxPLUGIN_K_CORE_INIT_FUNCTION_NAME(PLUGIN_SUFFIX))

  '#else /' __orxEMBEDDED__ '/

  '/' *** Core info register macro *** '/
  '#define orxPLUGIN_REGISTER_CORE_INFO(PLUGIN_SUFFIX) _
	'orxPlugin_AddCoreInfo(orxPLUGIN_CORE_ID_##PLUGIN_SUFFIX, orxMODULE_ID_##PLUGIN_SUFFIX, sastPluginFunctionInfo_##PLUGIN_SUFFIX, sizeof(sastPluginFunctionInfo_##PLUGIN_SUFFIX) / SizeOf(orxPLUGIN_CORE_FUNCTION))

  '#endif /' __orxEMBEDDED__ '/

#endif /' _orxPLUGIN_CORE_BI '/

/'* @} '/
