/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxCamera.h
 * @date 10/12/2003
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxCamera
 * 
 * Camera Module
 * Allows to creates and handle cameras
 * Camera are structures used to render graphic (2D/3D) objects
 * They thus can be referenced by other structures
 *
 * @{
 '/


#ifndef _orxCAMERA_BI
#define _orxCAMERA_BI


#include "orxInclude.bi"
#include "orx/object/orxFrame.bi"
#include "orx/math/orxAABox.bi"


/'* Anim flags
 '/
#define orxCAMERA_KU32_FLAG_NONE              &H00000000 /'*< No flags '/

#define orxCAMERA_KU32_FLAG_2D                &H00000001 /'*< 2D flag '/

#define orxCAMERA_KU32_MASK_USER_ALL          &H000000FF /'*< User all mask '/


/'* Internal camera structure
 '/
Type __orxCAMERA_t As Any 
Type orxCAMERA As __orxCAMERA_t

Extern "Windows"

/'* Camera module setup
 '/
Declare Sub orxCamera_Setup()

/'* Inits the Camera module
 '/
Declare Function orxCamera_Init() As orxSTATUS

/'* Exits from the Camera module
 '/
Declare Sub orxCamera_Exit()


/'* Creates a camera
 * @param[in]   _u32Flags               Camera flags (2D / ...)
 * @return      Created orxCAMERA / orxNULL
 '/
Declare Function orxCamera_Create(ByVal _u32Flags As orxU32) As orxCAMERA Ptr

/'* Creates a camera from config
 * @param[in]   _zConfigID    Config ID
 * @ return orxCAMERA / orxNULL
 '/
Declare Function orxCamera_CreateFromConfig(ByVal _zConfigID As Const orxSTRING) As orxCAMERA Ptr

/'* Deletes a camera
 * @param[in]   _pstCamera      Camera to delete
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxCamera_Delete(ByVal _pstCamera As orxCAMERA Ptr) As orxSTATUS


/'* Sets camera frustum
 * @param[in]   _pstCamera      Concerned camera
 * @param[in]   _fWidth         Width of frustum
 * @param[in]   _fHeight        Height of frustum
 * @param[in]   _fNear          Near distance of frustum
 * @param[in]   _fFar           Far distance of frustum
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxCamera_SetFrustum(ByVal _pstCamera As orxCAMERA Ptr,ByVal _fWidth As orxFLOAT,ByVal _fHeight As orxFLOAT,ByVal _fNear As orxFLOAT,ByVal _fFar As orxFLOAT) As orxSTATUS

/'* Sets camera position
 * @param[in]   _pstCamera      Concerned camera
 * @param[in]   _pvPosition     Camera position
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxCamera_SetPosition(ByVal _pstCamera As orxCAMERA Ptr,ByVal _pvPosition As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets camera rotation
 * @param[in]   _pstCamera      Concerned camera
 * @param[in]   _fRotation      Camera rotation (radians)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxCamera_SetRotation(ByVal _pstCamera As orxCAMERA Ptr,ByVal _fRotation As orxFLOAT) As orxSTATUS

/'* Sets camera zoom
 * @param[in]   _pstCamera      Concerned camera
 * @param[in]   _fZoom          Camera zoom
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxCamera_SetZoom(ByVal _pstCamera As orxCAMERA Ptr,ByVal _fZoom As orxFLOAT) As orxSTATUS

/'* Gets camera frustum (3D box for 2D camera)
 * @param[in]   _pstCamera      Concerned camera
 * @param[out]  _pstFrustum    Frustum box
 * @return      Frustum orxAABOX
 '/
Declare Function orxCamera_GetFrustum(ByVal _pstCamera As Const orxCAMERA Ptr,ByVal _pstFrustum As orxAABOX Ptr) As orxAABOX Ptr

/'* Get camera position
 * @param[in]   _pstCamera      Concerned camera
 * @param[out]  _pvPosition     Camera position
 * @return      orxVECTOR
 '/
Declare Function orxCamera_GetPosition(ByVal _pstCamera As Const orxCAMERA Ptr,ByVal _pvPosition As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Get camera rotation
 * @param[in]   _pstCamera      Concerned camera
 * @return      Rotation value (radians)
 '/
Declare Function orxCamera_GetRotation(ByVal _pstCamera As Const orxCAMERA Ptr) As orxFLOAT

/'* Get camera zoom
 * @param[in]   _pstCamera      Concerned camera
 * @return      Zoom value
 '/
Declare Function orxCamera_GetZoom(ByVal _pstCamera As Const orxCAMERA Ptr) As orxFLOAT

/'* Gets camera config name
 * @param[in]   _pstCamera      Concerned camera
 * @return      orxSTRING / orxSTRING_EMPTY
 '/
Declare Function orxCamera_GetName(ByVal _pstCamera As Const orxCAMERA Ptr) As Const orxSTRING

/'* Gets camera given its name
 * @param[in]   _zName          Camera name
 * @return      orxCAMERA / orxNULL
 '/
Declare Function orxCamera_Get(ByVal _zName As Const orxSTRING) As orxCAMERA Ptr


/'* Gets camera frame
 * @param[in]   _pstCamera      Concerned camera
 * @return      orxFRAME
 '/
Declare Function orxCamera_GetFrame(ByVal _pstCamera As Const orxCAMERA Ptr) As orxFRAME Ptr

/'* Sets camera parent
 * @param[in]   _pstCamera      Concerned camera
 * @param[in]   _pParent        Parent structure to set (object, spawner, camera or frame) / orxNULL
 * @return      orsSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxCamera_SetParent(ByVal _pstCamera As orxCAMERA Ptr,ByVal _pParent As Any Ptr) As orxSTATUS

End Extern

#endif /' _orxCAMERA_BI '/

/'* @} '/
