/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxFX.h
 * @date 30/06/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxFX
 *
 * FX module
 * Allows to creates rendering special effects affecting color, scale, rotation and position of visuals.
 *
 * @{
 '/


#ifndef _orxFX_BI
#define _orxFX_BI


#include "orxInclude.bi"
#include "orx/math/orxVector.bi"
#include "orx/object/orxObject.bi"


/'* Slot flags
 '/
#define orxFX_SLOT_KU32_FLAG_ABSOLUTE           &H00000100  /'*< Absolute flag '/
#define orxFX_SLOT_KU32_FLAG_USE_ROTATION       &H00000200  /'*< Use rotation flag '/
#define orxFX_SLOT_KU32_FLAG_USE_SCALE          &H00000400  /'*< Use scale flag '/


/'* FX curve enum
 '/
Enum orxFX_CURVE
 
   orxFX_CURVE_LINEAR = 0,
   orxFX_CURVE_TRIANGLE,
   orxFX_CURVE_SINE,
   orxFX_CURVE_SQUARE,

   orxFX_CURVE_NUMBER,

   orxFX_CURVE_NONE = orxENUM_NONE

End Enum 


/'* Internal FX structure
 '/
Type orxFX As __orxFX_t


/'* Event enum
 '/
Enum orxFX_EVENT

  orxFX_EVENT_START = 0,                        /'*< Event sent when a FX starts '/
  orxFX_EVENT_STOP,                             /'*< Event sent when a FX stops '/
  orxFX_EVENT_ADD,                              /'*< Event sent when a FX is added '/
  orxFX_EVENT_REMOVE,                           /'*< Event sent when a FX is removed '/
  orxFX_EVENT_LOOP,                             /'*< Event sent when a FX is looping '/

  orxFX_EVENT_NUMBER,

  orxFX_EVENT_NONE = orxENUM_NONE

End Enum

/'* FX event payload
 '/
Type __orxFX_EVENT_PAYLOAD_t

  As orxFX Ptr pstFX                              /'*< FX reference : 4 '/
  As orxSTRING zFXName                            /'*< FX name : 8 '/

End Type

Type orxFX_EVENT_PAYLOAD As __orxFX_EVENT_PAYLOAD_t


Extern "Windows"

/'* FX module setup
 '/
Declare Sub orxFX_Setup()

/'* Inits the FX module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFX_Init() As orxSTATUS

/'* Exits from the FX module
 '/
Declare Sub orxFX_Exit()

/'* Creates an empty FX
 * @return orxFX / orxNULL
 '/
Declare Function orxFX_Create() As orxFX Ptr

/'* Creates an FX from config
 * @param[in]   _zConfigID    Config ID
 * @ return orxFX / orxNULL
 '/
Declare Function orxFX_CreateFromConfig(ByVal _zConfigID As Const orxSTRING) As orxFX Ptr

/'* Deletes an FX
 * @param[in] _pstFX            Concerned FX
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFX_Delete(ByVal _pstFX As orxFX Ptr) As orxSTATUS

/'* Applies FX on object
 * @param[in] _pstFX            FX to apply
 * @param[in] _pstObject        Object on which to apply the FX
 * @param[in] _fStartTime       FX local application start time
 * @param[in] _fEndTime         FX local application end time
 * @return    orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFX_Apply(ByVal _pstFX As Const orxFX Ptr,ByVal _pstObject As orxOBJECT Ptr,ByVal _fStartTime As orxFLOAT,ByVal _fEndTime As orxFLOAT) As orxSTATUS

/'* Enables/disables an FX
 * @param[in]   _pstFX          Concerned FX
 * @param[in]   _bEnable        Enable / disable
 '/
Declare Sub orxFX_Enable(ByVal _pstFX As orxFX Ptr,ByVal _bEnable As orxBOOL)

/'* Is FX enabled?
 * @param[in]   _pstFX          Concerned FX
 * @return      orxTRUE if enabled, orxFALSE otherwise
 '/
Declare Function orxFX_IsEnabled(ByVal _pstFX As Const orxFX Ptr) As orxBOOL

/'* Adds alpha to an FX
 * @param[in]   _pstFX          Concerned FX
 * @param[in]   _fStartTime     Time start
 * @param[in]   _fEndTime       Time end
 * @param[in]   _fCyclePeriod   Cycle period
 * @param[in]   _fCyclePhase    Cycle phase (at start)
 * @param[in]   _fAmplification Curve linear amplification over time (1.0 for none)
 * @param[in]   _fAcceleration  Curve linear acceleration over time (1.0 for none)
 * @param[in]   _fStartAlpha    Starting alpha value
 * @param[in]   _fEndAlpha      Ending alpha value
 * @param[in]   _eCurve         Blending curve type
 * @param[in]   _fPow           Blending curve exponent
 * @param[in]   _u32Flags       Param flags
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFX_AddAlpha(ByVal _pstFX As orxFX Ptr,ByVal _fStartTime As orxFLOAT,ByVal _fEndTime As orxFLOAT,ByVal _fCyclePeriod As orxFLOAT,ByVal _fCyclePhase As orxFLOAT,ByVal _fAmplification As orxFLOAT,ByVal _fAcceleration As orxFLOAT,ByVal _fStartAlpha As orxFLOAT,ByVal _fEndAlpha As orxFLOAT,ByVal _eCurve As orxFX_CURVE,ByVal _fPow As orxFLOAT,ByVal _u32Flags As orxU32) As orxSTATUS

/'* Adds color to an FX
 * @param[in]   _pstFX          Concerned FX
 * @param[in]   _fStartTime     Time start
 * @param[in]   _fEndTime       Time end
 * @param[in]   _fCyclePeriod   Cycle period
 * @param[in]   _fCyclePhase    Cycle phase (at start)
 * @param[in]   _fAmplification Curve linear amplification over time (1.0 for none)
 * @param[in]   _fAcceleration  Curve linear acceleration over time (1.0 for none)
 * @param[in]   _pvStartColor   Starting color value
 * @param[in]   _pvEndColor     Ending color value
 * @param[in]   _eCurve         Blending curve type
 * @param[in]   _fPow           Blending curve exponent
 * @param[in]   _u32Flags       Param flags
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFX_AddColor(ByVal _pstFX As orxFX Ptr,ByVal _fStartTime As orxFLOAT,ByVal _fEndTime As orxFLOAT,ByVal _fCyclePeriod As orxFLOAT,ByVal _fCyclePhase As orxFLOAT,ByVal _fAmplification As orxFLOAT,ByVal _fAcceleration As orxFLOAT,ByVal _pvStartColor As orxVECTOR Ptr,ByVal _pvEndColor As orxVECTOR Ptr,ByVal _eCurve As orxFX_CURVE,ByVal _fPow As orxFLOAT,ByVal _u32Flags As orxU32) As orxSTATUS

/'* Adds rotation to an FX
 * @param[in]   _pstFX          Concerned FX
 * @param[in]   _fStartTime     Time start
 * @param[in]   _fEndTime       Time end
 * @param[in]   _fCyclePeriod   Cycle period
 * @param[in]   _fCyclePhase    Cycle phase (at start)
 * @param[in]   _fAmplification Curve linear amplification over time (1.0 for none)
 * @param[in]   _fAcceleration  Curve linear acceleration over time (1.0 for none)
 * @param[in]   _fStartRotation Starting rotation value (radians)
 * @param[in]   _fEndRotation   Ending rotation value (radians)
 * @param[in]   _eCurve         Blending curve type
 * @param[in]   _fPow           Blending curve exponent
 * @param[in]   _u32Flags       Param flags
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFX_AddRotation(ByVal _pstFX As orxFX Ptr,ByVal _fStartTime As orxFLOAT,ByVal _fEndTime As orxFLOAT,ByVal _fCyclePeriod As orxFLOAT,ByVal _fCyclePhase As orxFLOAT,ByVal _fAmplification As orxFLOAT,ByVal _fAcceleration As orxFLOAT,ByVal _fStartRotation As orxFLOAT,ByVal _fEndRotation As orxFLOAT,ByVal _eCurve As orxFX_CURVE,ByVal _fPow As orxFLOAT,ByVal _u32Flags As orxU32) As orxSTATUS

/'* Adds scale to an FX
 * @param[in]   _pstFX          Concerned FX
 * @param[in]   _fStartTime     Time start
 * @param[in]   _fEndTime       Time end
 * @param[in]   _fCyclePeriod   Cycle period
 * @param[in]   _fCyclePhase    Cycle phase (at start)
 * @param[in]   _fAmplification Curve linear amplification over time (1.0 for none)
 * @param[in]   _fAcceleration  Curve linear acceleration over time (1.0 for none)
 * @param[in]   _pvStartScale   Starting scale value
 * @param[in]   _pvEndScale     Ending scale value
 * @param[in]   _eCurve         Blending curve type
 * @param[in]   _fPow           Blending curve exponent
 * @param[in]   _u32Flags       Param flags
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFX_AddScale(ByVal _pstFX As orxFX Ptr,ByVal _fStartTime As orxFLOAT,ByVal _fEndTime As orxFLOAT,ByVal _fCyclePeriod As orxFLOAT,ByVal _fCyclePhase As orxFLOAT,ByVal _fAmplification As orxFLOAT,ByVal _fAcceleration As orxFLOAT,ByVal _pvStartScale As Const orxVECTOR Ptr,ByVal _pvEndScale As Const orxVECTOR Ptr,ByVal _eCurve As orxFX_CURVE,ByVal _fPow As orxFLOAT,ByVal _u32Flags As orxU32) As orxSTATUS

/'* Adds position to an FX
 * @param[in]   _pstFX          Concerned FX
 * @param[in]   _fStartTime     Time start
 * @param[in]   _fEndTime       Time end
 * @param[in]   _fCyclePeriod   Cycle period
 * @param[in]   _fCyclePhase    Cycle phase (at start)
 * @param[in]   _fAmplification Curve linear amplification over time (1.0 for none)
 * @param[in]   _fAcceleration  Curve linear acceleration over time (1.0 for none)
 * @param[in]   _pvStartTranslation Starting position value
 * @param[in]   _pvEndTranslation Ending position value
 * @param[in]   _eCurve         Blending curve type
 * @param[in]   _fPow           Blending curve exponent
 * @param[in]   _u32Flags       Param flags
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFX_AddPosition(ByVal _pstFX As orxFX Ptr,ByVal _fStartTime As orxFLOAT,ByVal _fEndTime As orxFLOAT,ByVal _fCyclePeriod As orxFLOAT,ByVal _fCyclePhase As orxFLOAT,ByVal _fAmplification As orxFLOAT,ByVal _fAcceleration As orxFLOAT,ByVal _pvStartTranslation As Const orxVECTOR Ptr,ByVal _pvEndTranslation As Const orxVECTOR Ptr,ByVal _eCurve As orxFX_CURVE,ByVal _fPow As orxFLOAT,ByVal _u32Flags As orxU32) As orxSTATUS

/'* Adds speed to an FX
 * @param[in]   _pstFX          Concerned FX
 * @param[in]   _fStartTime     Time start
 * @param[in]   _fEndTime       Time end
 * @param[in]   _fCyclePeriod   Cycle period
 * @param[in]   _fCyclePhase    Cycle phase (at start)
 * @param[in]   _fAmplification Curve linear amplification over time (1.0 for none)
 * @param[in]   _fAcceleration  Curve linear acceleration over time (1.0 for none)
 * @param[in]   _pvStartSpeed   Starting speed value
 * @param[in]   _pvEndSpeed     Ending speed value
 * @param[in]   _eCurve         Blending curve type
 * @param[in]   _fPow           Blending curve exponent
 * @param[in]   _u32Flags       Param flags
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFX_AddSpeed(ByVal _pstFX As orxFX Ptr,ByVal _fStartTime As orxFLOAT,ByVal _fEndTime As orxFLOAT,ByVal _fCyclePeriod As orxFLOAT,ByVal _fCyclePhase As orxFLOAT,ByVal _fAmplification As orxFLOAT,ByVal _fAcceleration As orxFLOAT,ByVal _pvStartSpeed As Const orxVECTOR Ptr,ByVal _pvEndSpeed As Const orxVECTOR Ptr,ByVal _eCurve As orxFX_CURVE,ByVal _fPow As orxFLOAT,ByVal _u32Flags As orxU32) As orxSTATUS

/'* Gets FX duration
 * @param[in]   _pstFX          Concerned FX
 * @return      orxFLOAT
 '/
Declare Function orxFX_GetDuration(ByVal _pstFX As Const orxFX Ptr) As orxFLOAT

/'* Gets FX name
 * @param[in]   _pstFX          Concerned FX
 * @return      orxSTRING / orxSTRING_EMPTY
 '/
Declare Function orxFX_GetName(ByVal _pstFX As Const orxFX Ptr) As Const orxSTRING

/'* Set FX loop property
 * @param[in]   _pstFX          Concerned FX
 * @param[in]   _bLoop          Loop / don't loop
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFX_Loop(ByVal _pstFX As orxFX Ptr,ByVal _bLoop As orxBOOL) As orxSTATUS

/'* Is FX looping
 * @param[in]   _pstFX          Concerned FX
 * @return      orxTRUE if looping, orxFALSE otherwise
 '/
Declare Function orxFX_IsLooping(ByVal _pstFX As Const orxFX Ptr) As orxBOOL

End Extern

#endif /' _orxFX_BI '/

/'* @} '/
