/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxFXPointer.h
 * @date 30/06/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxFXPointer
 *
 * FXPointer module
 * Allows to creates rendering special effects containers for objects.
 *
 * @{
 '/


#ifndef _orxFXPOINTER_BI
#define _orxFXPOINTER_BI


#include "orxInclude.bi"
#include "orx/render/orxFX.bi"
#include "orx/object/orxStructure.bi"


/'* Misc defines
 '/
#define orxFXPOINTER_KU32_FX_NUMBER             4


/'* Internal FXPointer structure '/
Type orxFXPOINTER As __orxFXPOINTER_t


Extern "Windows"

/'* FXPointer module setup
 '/
Declare Sub orxFXPointer_Setup()

/'* Inits the FXPointer module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFXPointer_Init() As orxSTATUS

/'* Exits from the FXPointer module
 '/
Declare Sub orxFXPointer_Exit()


/'* Creates an empty FXPointer
 * @param[in]   _pstOwner       FXPointer's owner used for event callbacks (usually an orxOBJECT)
 * @return orxFXPOINTER / orxNULL
 '/
Declare Function orxFXPointer_Create(ByVal _pstOwner As Const orxSTRUCTURE Ptr) As orxFXPOINTER Ptr

/'* Deletes an FXPointer
 * @param[in] _pstFXPointer     Concerned FXPointer
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFXPointer_Delete(ByVal _pstFXPointer As orxFXPOINTER Ptr) As orxSTATUS

/'* Gets an FXPointer owner
 * @param[in]   _pstFXPointer   Concerned FXPointer
 * @return      orxSTRUCTURE / orxNULL
 '/
Declare Function orxFXPointer_GetOwner(ByVal _pstFXPointer As Const orxFXPOINTER Ptr) As orxSTRUCTURE Ptr

/'* Enables/disables an FXPointer
 * @param[in]   _pstFXPointer   Concerned FXPointer
 * @param[in]   _bEnable        Enable / disable
 '/
Declare Sub orxFXPointer_Enable(ByVal _pstFXPointer As orxFXPOINTER Ptr,ByVal _bEnable As orxBOOL)

/'* Is FXPointer enabled?
 * @param[in]   _pstFXPointer   Concerned FXPointer
 * @return      orxTRUE if enabled, orxFALSE otherwise
 '/
Declare Function orxFXPointer_IsEnabled(ByVal _pstFXPointer As Const orxFXPOINTER Ptr) As orxBOOL


/'* Adds an FX
 * @param[in]   _pstFXPointer Concerned FXPointer
 * @param[in]   _pstFX        FX to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFXPointer_AddFX(ByVal _pstFXPointer As orxFXPOINTER Ptr,ByVal _pstFX As orxFX Ptr) As orxSTATUS

/'* Adds a delayed FX
 * @param[in]   _pstFXPointer Concerned FXPointer
 * @param[in]   _pstFX        FX to add
 * @param[in]   _fDelay       Delay time
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFXPointer_AddDelayedFX(ByVal _pstFXPointer As orxFXPOINTER Ptr,ByVal _pstFX As orxFX Ptr,ByVal _fDelay As orxFLOAT) As orxSTATUS

/'* Removes an FX
 * @param[in]   _pstFXPointer Concerned FXPointer
 * @param[in]   _pstFX        FX to remove
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFXPointer_RemoveFX(ByVal _pstFXPointer As orxFXPOINTER Ptr,ByVal _pstFX As orxFX Ptr) As orxSTATUS

/'* Adds an FX using its config ID
 * @param[in]   _pstFXPointer Concerned FXPointer
 * @param[in]   _zFXConfigID  Config ID of the FX to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFXPointer_AddFXFromConfig(ByVal _pstFXPointer As orxFXPOINTER Ptr,ByVal _zFXConfigID As Const orxSTRING) As orxSTATUS

/'* Adds a unique FX using its config ID
 * @param[in]   _pstFXPointer Concerned FXPointer
 * @param[in]   _zFXConfigID  Config ID of the FX to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFXPointer_AddUniqueFXFromConfig(ByVal _pstFXPointer As orxFXPOINTER Ptr,ByVal _zFXConfigID As Const orxSTRING) As orxSTATUS

/'* Adds a delayed FX using its config ID
 * @param[in]   _pstFXPointer Concerned FXPointer
 * @param[in]   _zFXConfigID  Config ID of the FX to add
 * @param[in]   _fDelay       Delay time
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFXPointer_AddDelayedFXFromConfig(ByVal _pstFXPointer As orxFXPOINTER Ptr,ByVal _zFXConfigID As Const orxSTRING,ByVal _fDelay As orxFLOAT) As orxSTATUS

/'* Adds a unique delayed FX using its config ID
 * @param[in]   _pstFXPointer Concerned FXPointer
 * @param[in]   _zFXConfigID  Config ID of the FX to add
 * @param[in]   _fDelay       Delay time
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFXPointer_AddUniqueDelayedFXFromConfig(ByVal _pstFXPointer As orxFXPOINTER Ptr,ByVal _zFXConfigID As Const orxSTRING,ByVal _fDelay As orxFLOAT) As orxSTATUS

/'* Removes an FX using using its config ID
 * @param[in]   _pstFXPointer Concerned FXPointer
 * @param[in]   _zFXConfigID  Config ID of the FX to remove
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxFXPointer_RemoveFXFromConfig(ByVal _pstFXPointer As orxFXPOINTER Ptr,ByVal _zFXConfigID As Const orxSTRING) As orxSTATUS

End Extern

#endif /' _orxFXPointer_BI '/

/'* @} '/
