/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxShaderPointer.h
 * @date 08/04/2009
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxShaderPointer
 *
 * ShaderPointer module
 * Allows to creates shaders (rendering post effects) containers for objects.
 *
 * @{
 '/


#ifndef _orxSHADERPOINTER_BI
#define _orxSHADERPOINTER_BI


#include "orxInclude.bi"
#include "orx/render/orxShader.bi"
#include "orx/object/orxStructure.bi"


/'* Misc defines
 '/
#define orxSHADERPOINTER_KU32_SHADER_NUMBER     4


/'* Internal ShaderPointer structure '/

Type __orxSHADERPOINTER_t As Any Ptr

Type orxSHADERPOINTER As __orxSHADERPOINTER_t

Extern "Windows"

/'* ShaderPointer module setup
 '/
Declare Sub orxShaderPointer_Setup()

/'* Inits the ShaderPointer module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShaderPointer_Init() As orxSTATUS

/'* Exits from the ShaderPointer module
 '/
Declare Sub orxShaderPointer_Exit()


/'* Creates an empty ShaderPointer
 * @param[in]   _pstOwner       ShaderPointer's owner used for rendering (usually an orxOBJECT)
 * @return orxSHADERPOINTER / orxNULL
 '/
Declare Function orxShaderPointer_Create(ByVal _pstOwner As Const orxSTRUCTURE Ptr) As orxSHADERPOINTER Ptr

/'* Deletes an ShaderPointer
 * @param[in] _pstShaderPointer     Concerned ShaderPointer
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShaderPointer_Delete(ByVal _pstShaderPointer As orxSHADERPOINTER Ptr) As orxSTATUS

/'* Renders a ShaderPointer
 * @param[in] _pstShaderPointer     Concerned ShaderPointer
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShaderPointer_Render(ByVal _pstShaderPointer As Const orxSHADERPOINTER Ptr) As orxSTATUS

/'* Gets an ShaderPointer owner
 * @param[in]   _pstShaderPointer   Concerned ShaderPointer
 * @return      orxSTRUCTURE / orxNULL
 '/
Declare Function orxShaderPointer_GetOwner(ByVal _pstShaderPointer As Const orxSHADERPOINTER Ptr) As orxSTRUCTURE Ptr

/'* Enables/disables an ShaderPointer
 * @param[in]   _pstShaderPointer   Concerned ShaderPointer
 * @param[in]   _bEnable        Enable / disable
 '/
Declare Sub orxShaderPointer_Enable(ByVal _pstShaderPointer As orxSHADERPOINTER Ptr,ByVal _bEnable As orxBOOL)

/'* Is ShaderPointer enabled?
 * @param[in]   _pstShaderPointer   Concerned ShaderPointer
 * @return      orxTRUE if enabled, orxFALSE otherwise
 '/
Declare Function orxShaderPointer_IsEnabled(ByVal _pstShaderPointer As Const orxSHADERPOINTER Ptr) As orxBOOL


/'* Adds a shader
 * @param[in]   _pstShaderPointer Concerned ShaderPointer
 * @param[in]   _pstShader        Shader to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShaderPointer_AddShader(ByVal _pstShaderPointer As orxSHADERPOINTER Ptr,ByVal _pstShader As orxSHADER Ptr) As orxSTATUS

/'* Removes a shader
 * @param[in]   _pstShaderPointer Concerned ShaderPointer
 * @param[in]   _pstShader        Shader to remove
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShaderPointer_RemoveShader(ByVal _pstShaderPointer As orxSHADERPOINTER Ptr,ByVal _pstShader As orxSHADER Ptr) As orxSTATUS

/'* Adds a shader using its config ID
 * @param[in]   _pstShaderPointer Concerned ShaderPointer
 * @param[in]   _zShaderConfigID  Config ID of the shader to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShaderPointer_AddShaderFromConfig(ByVal _pstShaderPointer As orxSHADERPOINTER Ptr,ByVal _zShaderConfigID As Const orxSTRING) As orxSTATUS

/'* Removes a shader using using its config ID
 * @param[in]   _pstShaderPointer Concerned ShaderPointer
 * @param[in]   _zShaderConfigID  Config ID of the shader to remove
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxShaderPointer_RemoveShaderFromConfig(ByVal _pstShaderPointer As orxSHADERPOINTER Ptr,ByVal _zShaderConfigID As Const orxSTRING) As orxSTATUS

End Extern

#endif /' _orxSHADERPOINTER_BI '/

/'* @} '/
