/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxViewport.h
 * @date 14/12/2003
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxViewport
 * 
 * Viewport module
 * Allows to creates and handle viewports.
 * Viewports are structures associated to cameras and used for rendering.
 *
 * @{
 '/


#ifndef _orxVIEWPORT_BI
#define _orxVIEWPORT_BI

#include "orxInclude.bi"
#include "orx/core/orxClock.bi"
#include "orx/render/orxCamera.bi"
#include "orx/render/orxShaderPointer.bi"
#include "orx/display/orxTexture.bi"


/'* Viewport flags
 '/
#define orxVIEWPORT_KU32_FLAG_ALIGN_CENTER    &H00000000  /'*< Center vertical / horizontal alignment '/
#define orxVIEWPORT_KU32_FLAG_ALIGN_LEFT      &H10000000  /'*< Left horizontal alignment '/
#define orxVIEWPORT_KU32_FLAG_ALIGN_RIGHT     &H20000000  /'*< Right horizontal alignment '/
#define orxVIEWPORT_KU32_FLAG_ALIGN_TOP       &H40000000  /'*< Top vertical alignment '/
#define orxVIEWPORT_KU32_FLAG_ALIGN_BOTTOM    &H80000000  /'*< Bottom vertical alignment '/


/'* Internal Viewport structure '/
Type orxVIEWPORT As __orxVIEWPORT_t

Extern "Windows"

/'* Viewport module setup
 '/
Declare Sub orxViewport_Setup()

/'* Inits the viewport module
 '/
Declare Function orxViewport_Init() As orxSTATUS

/'* Exits from the viewport module
 '/
Declare Sub orxViewport_Exit()


/'* Creates a viewport
 * @return      Created orxVIEWPORT / orxNULL
 '/
Declare Function orxViewport_Create() As orxVIEWPORT Ptr

/'* Creates a viewport from config
 * @param[in]   _zConfigID    Config ID
 * @ return orxVIEWPORT / orxNULL
 '/
Declare Function orxViewport_CreateFromConfig(ByVal _zConfigID As Const orxSTRING) As orxVIEWPORT Ptr

/'* Deletes a viewport
 * @param[in]   _pstViewport    Viewport to delete
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxViewport_Delete(ByVal _pstViewport As orxVIEWPORT Ptr) As orxSTATUS


/'* Sets a viewport alignment
 * @param[in]   _pstViewport    Concerned viewport
 * @param[in]   _u32AlignFlags  Alignment flags (must be OR'ed)
 '/
Declare Sub orxViewport_SetAlignment(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _u32AlignFlags As orxU32)


/'* Sets a viewport texture
 * @param[in]   _pstViewport    Concerned viewport
 * @param[in]   _pstTexture     Texture to associate with the viewport
 '/
Declare Sub orxViewport_SetTexture(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _pstTexture As orxTEXTURE Ptr)

/'* Gets a viewport texture
 * @param[in]   _pstViewport    Concerned viewport
 * @return      Associated orxTEXTURE / orxNULL
 '/
Declare Function orxViewport_GetTexture(ByVal _pstViewport As Const orxVIEWPORT Ptr) As orxTEXTURE Ptr


/'* Sets a viewport background color
 * @param[in]   _pstViewport    Concerned viewport
 * @param[in]   _pstColor        Color to use for background
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxViewport_SetBackgroundColor(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _pstColor As Const orxCOLOR Ptr) As orxSTATUS

/'* Gets a viewport texture
 * @param[in]   _pstViewport    Concerned viewport
 * @param[out]  _pstColor       Viewport's color
 * @return      Current background color
 '/
Declare Function orxViewport_GetBackgroundColor(ByVal _pstViewport As Const orxVIEWPORT Ptr,ByVal _pstColor As orxCOLOR Ptr) As orxCOLOR Ptr


/'* Enables / disables a viewport
 * @param[in]   _pstViewport    Concerned viewport
 * @param[in]   _bEnable        Enable / disable
 '/
Declare Sub orxViewport_Enable(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _bEnable As orxBOOL)

/'* Is a viewport enabled?
 * @param[in]   _pstViewport    Concerned viewport
 * @return      orxTRUE / orxFALSE
 '/
Declare Function orxViewport_IsEnabled(ByVal _pstViewport As Const orxVIEWPORT Ptr) As orxBOOL


/'* Enables / disables background clearing for a viewport
 * @param[in]   _pstViewport    Concerned viewport
 * @param[in]   _bEnable        Enable / disable
 '/
Declare Sub orxViewport_EnableBackgroundClearing(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _bEnable As orxBOOL)

/'* Has a viewport background clearing enabled?
 * @param[in]   _pstViewport    Concerned viewport
 * @return      orxTRUE / orxFALSE
 '/
Declare Function orxViewport_IsBackgroundClearingEnabled(ByVal _pstViewport As Const orxVIEWPORT Ptr) As orxBOOL


/'* Sets a viewport camera
 * @param[in]   _pstViewport    Concerned viewport
 * @param[in]   _pstCamera      Associated camera
 '/
Declare Sub orxViewport_SetCamera(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _pstCamera As orxCAMERA Ptr)

/'* Gets a viewport camera
 * @param[in]   _pstViewport    Concerned viewport
 * @return      Associated camera / orxNULL
 '/
Declare Function orxViewport_GetCamera(ByVal _pstViewport As Const orxVIEWPORT Ptr) As orxCAMERA Ptr


/'* Adds a shader to a viewport using its config ID
 * @param[in]   _pstViewport      Concerned viewport
 * @param[in]   _zShaderConfigID  Config ID of the shader to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxViewport_AddShader(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _zShaderConfigID As Const orxSTRING) As orxSTATUS

/'* Removes a shader using using its config ID
 * @param[in]   _pstViewport      Concerned viewport
 * @param[in]   _zShaderConfigID Config ID of the shader to remove
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxViewport_RemoveShader(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _zShaderConfigID As Const orxSTRING) As orxSTATUS

/'* Enables a viewport's shader
 * @param[in]   _pstViewport      Concerned viewport
 * @param[in]   _bEnable          Enable / disable
 '/
Declare Sub orxViewport_EnableShader(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _bEnable As orxBOOL)

/'* Is a viewport's shader enabled?
 * @param[in]   _pstViewport      Concerned viewport
 * @return      orxTRUE if enabled, orxFALSE otherwise
 '/
Declare Function orxViewport_IsShaderEnabled(ByVal _pstViewport As Const orxVIEWPORT Ptr) As orxBOOL

/'* Gets a viewport's shader pointer
 * @param[in]   _pstViewport      Concerned viewport
 * @return      orxSHADERPOINTER / orxNULL
 '/
Declare Function orxViewport_GetShaderPointer(ByVal _pstViewport As Const orxVIEWPORT Ptr) As Const orxSHADERPOINTER Ptr


/'* Sets a viewport position
 * @param[in]   _pstViewport    Concerned viewport
 * @param[in]   _fX             X axis position (top left corner)
 * @param[in]   _fY             Y axis position (top left corner)
 '/
Declare Sub orxViewport_SetPosition(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _fX As orxFLOAT,ByVal _fY As orxFLOAT)

/'* Sets a viewport relative position
 * @param[in]   _pstViewport    Concerned viewport
 * @param[in]   _u32AlignFlags  Alignment flags
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxViewport_SetRelativePosition(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _u32AlignFlags As orxU32) As orxSTATUS

/'* Gets a viewport position
 * @param[in]   _pstViewport    Concerned viewport
 * @param[out]  _pfX            X axis position (top left corner)
 * @param[out]  _pfY            Y axis position (top left corner)
 '/
Declare Sub orxViewport_GetPosition(ByVal _pstViewport As Const orxVIEWPORT Ptr,ByVal _pfX As orxFLOAT Ptr,ByVal _pfY As orxFLOAT Ptr)


/'* Sets a viewport size
 * @param[in]   _pstViewport    Concerned viewport
 * @param[in]   _fWidth         Width
 * @param[in]   _fHeight        Height
 '/
Declare Sub orxViewport_SetSize(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _fWidth As orxFLOAT,ByVal _fHeight As orxFLOAT)

/'* Sets a viewport relative size
 * @param[in]   _pstViewport    Concerned viewport
 * @param[in]   _fWidth         Relative width (0.0f - 1.0f)
 * @param[in]   _fHeight        Relative height (0.0f - 1.0f)
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxViewport_SetRelativeSize(ByVal _pstViewport As orxVIEWPORT Ptr,ByVal _fWidth As orxFLOAT,ByVal _fHeight As orxFLOAT) As orxSTATUS

/'* Gets a viewport size
 * @param[in]   _pstViewport    Concerned viewport
 * @param[out]  _pfWidth        Width
 * @param[out]  _pfHeight       Height
 '/
Declare Sub orxViewport_GetSize(ByVal _pstViewport As Const orxVIEWPORT Ptr,ByVal _pfWidth As orxFLOAT Ptr,ByVal _pfHeight As orxFLOAT Ptr)

/'* Gets a viewport relative size
 * @param[in]   _pstViewport    Concerned viewport
 * @param[out]  _pfWidth        Relative width
 * @param[out]  _pfHeight       Relative height
 '/
Declare Sub orxViewport_GetRelativeSize(ByVal _pstViewport As Const orxVIEWPORT Ptr,ByVal _pfWidth As orxFLOAT Ptr,ByVal _pfHeight As orxFLOAT Ptr)

/'* Gets a viewport clipping
 * @param[in]   _pstViewport    Concerned viewport
 * @param[out]  _pu32TLX        X coordinate of top left corner
 * @param[out]  _pu32TLY        Y coordinate of top left corner
 * @param[out]  _pu32BRX        X coordinate of bottom right corner
 * @param[out]  _pu32BRY        Y coordinate of bottom right corner
 '/
Declare Sub orxViewport_GetClipping(ByVal _pstViewport As Const orxVIEWPORT Ptr,ByVal _pu32TLX As orxU32 Ptr,ByVal _pu32TLY As orxU32 Ptr,ByVal _pu32BRX As orxU32 Ptr,ByVal _pu32BRY As orxU32 Ptr)

/'* Gets an axis aligned box of viewport
 * @param[in]   _pstViewport    Concerned viewport
 * @param[out]  _pstBox         Output box
 * @return orxAABOX / orxNULL
 '/
Declare Function orxViewport_GetBox(ByVal _pstViewport As Const orxVIEWPORT Ptr,ByVal _pstBox As orxAABOX Ptr) As orxAABOX Ptr

/'* Get viewport correction ratio
 * @param[in]   _pstViewport  Concerned viewport
 * @return      Correction ratio value
 '/
Declare Function orxViewport_GetCorrectionRatio(ByVal _pstViewport As Const orxVIEWPORT Ptr) As orxFLOAT

End Extern

#endif /' _orxVIEWPORT_BI '/

/'* @} '/
