/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxSound.h
 * @date 13/07/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxSound
 * 
 * Sound module
 * Module that handles sound
 *
 * @{
 '/


#ifndef _orxSOUND_BI
#define _orxSOUND_BI


#include "orxInclude.bi"
#include "orx/math/orxVector.bi"


/'* Sound status enum
 '/
Enum orxSOUND_STATUS

  orxSOUND_STATUS_PLAY = 0,
  orxSOUND_STATUS_PAUSE,
  orxSOUND_STATUS_STOP,

  orxSOUND_STATUS_NUMBER,

  orxSOUND_STATUS_NONE = orxENUM_NONE

End Enum 


/'* Internal Sound structure
 '/
Type __orxSOUND_t As Any
Type orxSOUND As __orxSOUND_t


/'* Event enum
 '/
Enum orxSOUND_EVENT

  orxSOUND_EVENT_START = 0,                   /'*< Event sent when a sound starts '/
  orxSOUND_EVENT_STOP,                        /'*< Event sent when a sound stops '/

  orxSOUND_EVENT_NUMBER,

  orxSOUND_EVENT_NONE = orxENUM_NONE

End Enum

/'* Sound event payload
 '/
Type __orxSOUND_EVENT_PAYLOAD_t

  As orxSOUND Ptr pstSound                         /'*< Sound reference : 4 '/
  As orxSTRING zSoundName                       /'*< Sound name : 8 '/

End Type

Type orxSOUND_EVENT_PAYLOAD As __orxSOUND_EVENT_PAYLOAD_t


Extern "Windows"

/'* Sound module setup
 '/
Declare Sub orxSound_Setup()

/'* Initializes the sound module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSound_Init() As orxSTATUS

/'* Exits from the sound module
 '/
Declare Sub orxSound_Exit()


/'* Creates sound from config
 * @param[in]   _zConfigID    Config ID
 * @ return orxSOUND / orxNULL
 '/
Declare Function orxSound_CreateFromConfig(ByVal _zConfigID As Const orxSTRING) As orxSOUND Ptr

/'* Deletes sound
 * @param[in] _pstSound       Concerned Sound
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSound_Delete(ByVal _pstSound As orxSOUND Ptr) As orxSTATUS


/'* Plays sound
 * @param[in] _pstSound       Concerned Sound
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSound_Play(ByVal _pstSound As orxSOUND Ptr) As orxSTATUS

/'* Pauses sound
 * @param[in] _pstSound       Concerned Sound
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSound_Pause(ByVal _pstSound As orxSOUND Ptr) As orxSTATUS

/'* Stops sound
 * @param[in] _pstSound       Concerned Sound
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSound_Stop(ByVal _pstSound As orxSOUND Ptr) As orxSTATUS


/'* Sets sound volume
 * @param[in] _pstSound       Concerned Sound
 * @param[in] _fVolume        Desired volume (0.0 - 1.0)
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSound_SetVolume(ByVal _pstSound As orxSOUND Ptr,ByVal _fVolume As orxFLOAT) As orxSTATUS

/'* Sets sound pitch
 * @param[in] _pstSound       Concerned Sound
 * @param[in] _fPitch         Desired pitch
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSound_SetPitch(ByVal _pstSound As orxSOUND Ptr,ByVal _fPitch As orxFLOAT) As orxSTATUS

/'* Sets sound position
 * @param[in] _pstSound       Concerned Sound
 * @param[in] _pvPosition     Desired position
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSound_SetPosition(ByVal _pstSound As orxSOUND Ptr,ByVal _pvPosition As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets sound attenuation
 * @param[in] _pstSound       Concerned Sound
 * @param[in] _fAttenuation   Desired attenuation
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSound_SetAttenuation(ByVal _pstSound As orxSOUND Ptr,ByVal _fAttenuation As orxFLOAT) As orxSTATUS

/'* Sets sound reference distance
 * @param[in] _pstSound       Concerned Sound
 * @param[in] _fDistance      Within this distance, sound is perceived at its maximum volume
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSound_SetReferenceDistance(ByVal _pstSound As orxSOUND Ptr,ByVal _fDistance As orxFLOAT) As orxSTATUS

/'* Loops sound
 * @param[in] _pstSound       Concerned Sound
 * @param[in] _bLoop          orxTRUE / orxFALSE
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSound_Loop(ByVal _pstSound As orxSOUND Ptr,ByVal _bLoop As orxBOOL) As orxSTATUS


/'* Gets sound volume
 * @param[in] _pstSound       Concerned Sound
 * @return orxFLOAT
 '/
Declare Function orxSound_GetVolume(ByVal _pstSound As Const orxSOUND Ptr) As orxFLOAT

/'* Gets sound pitch
 * @param[in] _pstSound       Concerned Sound
 * @return orxFLOAT
 '/
Declare Function orxSound_GetPitch(ByVal _pstSound As Const orxSOUND Ptr) As orxFLOAT

/'* Gets sound position
 * @param[in]  _pstSound      Concerned Sound
 * @param[out] _pvPosition    Sound's position
 * @return orxVECTOR / orxNULL
 '/
Declare Function orxSound_GetPosition(ByVal _pstSound As Const orxSOUND Ptr,ByVal _pvPosition As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets sound attenuation
 * @param[in] _pstSound       Concerned Sound
 * @return orxFLOAT
 '/
Declare Function orxSound_GetAttenuation(ByVal _pstSound As Const orxSOUND Ptr) As orxFLOAT

/'* Gets sound reference distance
 * @param[in] _pstSound       Concerned Sound
 * @return orxFLOAT
 '/
Declare Function orxSound_GetReferenceDistance(ByVal _pstSound As Const orxSOUND Ptr) As orxFLOAT

/'* Is sound looping?
 * @param[in] _pstSound       Concerned Sound
 * @return orxTRUE / orxFALSE
 '/
Declare Function orxSound_IsLooping(ByVal _pstSound As Const orxSOUND Ptr) As orxBOOL


/'* Gets sound duration
 * @param[in] _pstSound       Concerned Sound
 * @return orxFLOAT
 '/
Declare Function orxSound_GetDuration(ByVal _pstSound As Const orxSOUND Ptr) As orxFLOAT

/'* Gets sound status
 * @param[in] _pstSound       Concerned Sound
 * @return orxSOUND_STATUS
 '/
Declare Function orxSound_GetStatus(ByVal _pstSound As Const orxSOUND Ptr) As orxSOUND_STATUS

/'* Gets sound config name
 * @param[in]   _pstSound     Concerned sound
 * @return      orxSTRING / orxSTRING_EMPTY
 '/
Declare Function orxSound_GetName(ByVal _pstSound As Const orxSOUND Ptr) As Const orxSTRING

End Extern
#endif /'_orxSOUND_BI'/

/'* @} '/
