/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxSoundPointer.h
 * @date 20/07/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxSoundPointer
 *
 * Sound pointer module
 * Allows to creates sound containers for objects.
 *
 * @{
 '/


#ifndef _orxSOUNDPOINTER_BI
#define _orxSOUNDPOINTER_BI


#include "orxInclude.bi"
#include "orx/sound/orxSound.bi"
#include "orx/object/orxStructure.bi"


/'* Misc defines
 '/
#define orxSOUNDPOINTER_KU32_SOUND_NUMBER       4


/'* Internal SoundPointer structure '/
Type orxSOUNDPOINTER As __orxSOUNDPOINTER_t


Extern "Windows"

/'* SoundPointer module setup
 '/
Declare Sub orxSoundPointer_Setup()

/'* Inits the SoundPointer module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundPointer_Init() As orxSTATUS

/'* Exits from the SoundPointer module
 '/
Declare Sub orxSoundPointer_Exit()


/'* Creates an empty SoundPointer
 * @param[in]   _pstOwner           Sound's owner used for event callbacks (usually an orxOBJECT)
 * @return orxSOUNDPOINTER / orxNULL
 '/
Declare Function orxSoundPointer_Create(ByVal _pstOwner As Const orxSTRUCTURE Ptr) As orxSOUNDPOINTER Ptr

/'* Deletes a SoundPointer
 * @param[in] _pstSoundPointer      Concerned SoundPointer
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundPointer_Delete(ByVal _pstSoundPointer As orxSOUNDPOINTER Ptr) As orxSTATUS

/'* Gets a SoundPointer owner
 * @param[in]   _pstSoundPointer    Concerned SoundPointer
 * @return      orxSTRUCTURE / orxNULL
 '/
Declare Function orxSoundPointer_GetOwner(ByVal _pstSoundPointer As Const orxSOUNDPOINTER Ptr) As orxSTRUCTURE Ptr

/'* Enables/disables a SoundPointer
 * @param[in]   _pstSoundPointer    Concerned SoundPointer
 * @param[in]   _bEnable        Enable / disable
 '/
Declare Sub orxSoundPointer_Enable(ByVal _pstSoundPointer As orxSOUNDPOINTER Ptr,ByVal _bEnable As orxBOOL)

/'* Is SoundPointer enabled?
 * @param[in]   _pstSoundPointer    Concerned SoundPointer
 * @return      orxTRUE if enabled, orxFALSE otherwise
 '/
Declare Function orxSoundPointer_IsEnabled(ByVal _pstSoundPointer As Const orxSOUNDPOINTER Ptr) As orxBOOL


/'* Plays all related sounds
 * @param[in] _pstSoundPointer      Concerned SoundPointer
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundPointer_Play(ByVal _pstSoundPointer As orxSOUNDPOINTER Ptr) As orxSTATUS

/'* Pauses all related sounds
 * @param[in] _pstSoundPointer      Concerned SoundPointer
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundPointer_Pause(ByVal _pstSoundPointer As orxSOUNDPOINTER Ptr) As orxSTATUS

/'* Stops all related sounds
 * @param[in] _pstSoundPointer      Concerned SoundPointer
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundPointer_Stop(ByVal _pstSoundPointer As orxSOUNDPOINTER Ptr) As orxSTATUS


/'* Adds a sound
 * @param[in]   _pstSoundPointer    Concerned SoundPointer
 * @param[in]   _pstSound           Sound to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundPointer_AddSound(ByVal _pstSoundPointer As orxSOUNDPOINTER Ptr,ByVal _pstSound As orxSOUND Ptr) As orxSTATUS

/'* Removes a sound
 * @param[in]   _pstSoundPointer    Concerned SoundPointer
 * @param[in]   _pstSound           Sound to remove
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundPointer_RemoveSound(ByVal _pstSoundPointer As orxSOUNDPOINTER Ptr,ByVal _pstSound As orxSOUND Ptr) As orxSTATUS

/'* Adds a sound using its config ID
 * @param[in]   _pstSoundPointer    Concerned SoundPointer
 * @param[in]   _zSoundConfigID     Config ID of the sound to add
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundPointer_AddSoundFromConfig(ByVal _pstSoundPointer As orxSOUNDPOINTER Ptr,ByVal _zSoundConfigID As Const orxSTRING) As orxSTATUS

/'* Removes a sound using using its config ID
 * @param[in]   _pstSoundPointer    Concerned SoundPointer
 * @param[in]   _zSoundConfigID     Config ID of the sound to remove
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundPointer_RemoveSoundFromConfig(ByVal _pstSoundPointer As orxSOUNDPOINTER Ptr,ByVal _zSoundConfigID As Const orxSTRING) As orxSTATUS

/'* Gets last added sound (Do *NOT* destroy it directly before removing it!!!)
 * @param[in]   _pstSoundPointer    Concerned SoundPointer
 * @return      orxSOUND / orxNULL
 '/
Declare Function orxSoundPointer_GetLastAddedSound(ByVal _pstSoundPointer As Const orxSOUNDPOINTER Ptr) As orxSOUND Ptr

End Extern

#endif /' _orxSOUNDPOINTER_BI '/

/'* @} '/
