/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxSoundSystem.h
 * @date 13/07/2008
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxSoundSystem
 * 
 * Sound system plugin module
 * Plugin module that handles sound system
 *
 * @{
 '/


#ifndef _orxSOUNDSYSTEM_BI
#define _orxSOUNDSYSTEM_BI

#include "orxInclude.bi"
#include "orx/math/orxVector.bi"
''#include "orx/plugin/orxPluginCore.bi"


/'**************************************************************************
 * Structure declaration                                                   *
 **************************************************************************'/

/'* Abstract sound structures
 '/
Type orxSOUNDSYSTEM_SOUND As Any
Type orxSOUNDSYSTEM_SAMPLE As Any

/'* Sound system status enum
 '/
Enum orxSOUNDSYSTEM_STATUS

  orxSOUNDSYSTEM_STATUS_PLAY = 0,
  orxSOUNDSYSTEM_STATUS_PAUSE,
  orxSOUNDSYSTEM_STATUS_STOP,

  orxSOUNDSYSTEM_STATUS_NUMBER,

  orxSOUNDSYSTEM_STATUS_NONE = orxENUM_NONE

End Enum 


/'* Config defines
 '/
#define orxSOUNDSYSTEM_KZ_CONFIG_SECTION  "SoundSystem"
#define orxSOUNDSYSTEM_KZ_CONFIG_RATIO    "DimensionRatio"


/'**************************************************************************
 * Functions directly implemented by orx core
 **************************************************************************'/

Extern "Windows"

/'* Sound system module setup
 '/
Declare Sub orxSoundSystem_Setup()


/'**************************************************************************
 * Functions extended by plugins
 **************************************************************************'/

/'* Inits the sound system module
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundSystem_Init() As orxSTATUS

/'* Exits from the sound system module
 '/
Declare Sub orxSoundSystem_Exit()

/'* Loads a sound sample from file (cannot be played directly)
 * @param[in]   _zFilename                            Name of the file to load as a sample (completely loaded in memory, useful for sound effects)
 * @return orxSOUNDSYSTEM_SAMPLE / orxNULL
 '/
Declare Function orxSoundSystem_LoadSample(ByVal _zFilename As Const orxSTRING) As orxSOUNDSYSTEM_SAMPLE Ptr

/'* Unloads a sound sample
 * @param[in]   _pstSample                            Concerned sample
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundSystem_UnloadSample(ByVal _pstSample As orxSOUNDSYSTEM_SAMPLE Ptr) As orxSTATUS

/'* Creates a sound from preloaded sample (can be played directly)
 * @param[in]   _pstSample                            Concerned sample
 * @return orxSOUNDSYSTEM_SOUND / orxNULL
 '/
Declare Function orxSoundSystem_CreateFromSample(ByVal _pstSample As Const orxSOUNDSYSTEM_SAMPLE Ptr) As orxSOUNDSYSTEM_SOUND Ptr

/'* Creates a streamed sound from file (can be played directly)
 * @param[in]   _zFilename                            Name of the file to load as a stream (won't be completely loaded in memory, useful for musics)
 * @return orxSOUNDSYSTEM_SOUND / orxNULL
 '/
Declare Function orxSoundSystem_CreateStreamFromFile(ByVal _zFilename As Const orxSTRING) As orxSOUNDSYSTEM_SOUND Ptr

/'* Deletes a sound
 * @param[in]   _pstSound                             Concerned sound
 '/
Declare Function orxSoundSystem_Delete(ByVal _pstSound As orxSOUNDSYSTEM_SOUND Ptr) As orxSTATUS

/'* Plays a sound
 * @param[in]   _pstSound                             Concerned sound
 * @return orxSTATUS_SUCCESS / orxSTATSUS_FAILURE
 '/
Declare Function orxSoundSystem_Play(ByVal _pstSound As orxSOUNDSYSTEM_SOUND Ptr) As orxSTATUS

/'* Pauses a sound
 * @param[in]   _pstSound                             Concerned sound
 * @return orxSTATUS_SUCCESS / orxSTATSUS_FAILURE
 '/
Declare Function orxSoundSystem_Pause(ByVal _pstSound As orxSOUNDSYSTEM_SOUND Ptr) As orxSTATUS

/'* Stops a sound
 * @param[in]   _pstSound                             Concerned sound
 * @return orxSTATUS_SUCCESS / orxSTATSUS_FAILURE
 '/
Declare Function orxSoundSystem_Stop(ByVal _pstSound As orxSOUNDSYSTEM_SOUND Ptr) As orxSTATUS

/'* Sets a sound volume
 * @param[in]   _pstSound                             Concerned sound
 * @param[in]   _fVolume                              Volume to set [0, 1]
 * @return orxSTATUS_SUCCESS / orxSTATSUS_FAILURE
 '/
Declare Function orxSoundSystem_SetVolume(ByVal _pstSound As orxSOUNDSYSTEM_SOUND Ptr,ByVal _fVolume As orxFLOAT) As orxSTATUS

/'* Sets a sound pitch
 * @param[in]   _pstSound                             Concerned sound
 * @param[in]   _fPitch                               Pitch to set
 * @return orxSTATUS_SUCCESS / orxSTATSUS_FAILURE
 '/
Declare Function orxSoundSystem_SetPitch(ByVal _pstSound As orxSOUNDSYSTEM_SOUND Ptr,ByVal _fPitch As orxFLOAT) As orxSTATUS

/'* Sets a sound position
 * @param[in]   _pstSound                             Concerned sound
 * @param[in]   _pvPosition                           Position to set
 * @return orxSTATUS_SUCCESS / orxSTATSUS_FAILURE
 '/
Declare Function orxSoundSystem_SetPosition(ByVal _pstSound As orxSOUNDSYSTEM_SOUND Ptr,ByVal _pvPosition As Const orxVECTOR Ptr) As orxSTATUS

/'* Sets a sound attenuation
 * @param[in] _pstSound                               Concerned Sound
 * @param[in] _fAttenuation                           Desired attenuation
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundSystem_SetAttenuation(ByVal _pstSound As orxSOUNDSYSTEM_SOUND Ptr,ByVal _fAttenuation As orxFLOAT) As orxSTATUS

/'* Sets a sound reference distance
 * @param[in] _pstSound                               Concerned Sound
 * @param[in] _fDistance                              Within this distance, sound is perceived at its maximum volume
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundSystem_SetReferenceDistance(ByVal _pstSound As orxSOUNDSYSTEM_SOUND Ptr,ByVal _fDistance As orxFLOAT) As orxSTATUS

/'* Loops a sound
 * @param[in]   _pstSound                             Concerned sound
 * @param[in]   _bLoop                                Loop / no loop
 * @return orxSTATUS_SUCCESS / orxSTATSUS_FAILURE
 '/
Declare Function orxSoundSystem_Loop(ByVal _pstSound As orxSOUNDSYSTEM_SOUND Ptr,ByVal _bLoop As orxBOOL) As orxSTATUS

/'* Gets a sound volume
 * @param[in]   _pstSound                             Concerned sound
 * @return Sound's volume
 '/
Declare Function orxSoundSystem_GetVolume(ByVal _pstSound As Const orxSOUNDSYSTEM_SOUND Ptr) As orxFLOAT

/'* Gets a sound pitch
 * @param[in]   _pstSound                             Concerned sound
 * @return Sound's pitch
 '/
Declare Function orxSoundSystem_GetPitch(ByVal _pstSound As Const orxSOUNDSYSTEM_SOUND Ptr) As orxFLOAT

/'* Gets a sound position
 * @param[in]   _pstSound                             Concerned sound
 * @param[out]  _pvPosition                           Position to get
 * @return Sound's position
 '/
Declare Function orxSoundSystem_GetPosition(ByVal _pstSound As Const orxSOUNDSYSTEM_SOUND Ptr,ByVal _pvPosition As orxVECTOR Ptr) As orxVECTOR Ptr

/'* Gets a sound attenuation
 * @param[in] _pstSound                               Concerned Sound
 * @return Sound's attenuation
 '/
Declare Function orxSoundSystem_GetAttenuation(ByVal _pstSound As Const orxSOUNDSYSTEM_SOUND Ptr) As orxFLOAT

/'* Gets a sound reference distance
 * @param[in] _pstSound                               Concerned Sound
 * @return Sound's reference distance
 '/
Declare Function orxSoundSystem_GetReferenceDistance(ByVal _pstSound As Const orxSOUNDSYSTEM_SOUND Ptr) As orxFLOAT

/'* Is sound looping?
 * @param[in]   _pstSound                             Concerned sound
 * @return orxTRUE if looping, orxFALSE otherwise
 '/
Declare Function orxSoundSystem_IsLooping(ByVal _pstSound As Const orxSOUNDSYSTEM_SOUND Ptr) As orxBOOL

/'* Gets a sound duration
 * @param[in]   _pstSound                             Concerned sound
 * @return Sound's duration (seconds)
 '/
Declare Function orxSoundSystem_GetDuration(ByVal _pstSound As Const orxSOUNDSYSTEM_SOUND Ptr) As orxFLOAT

/'* Gets a sound status (play/pause/stop)
 * @param[in]   _pstSound                             Concerned sound
 * @return orxSOUNDSYSTEM_STATUS
 '/
Declare Function orxSoundSystem_GetStatus(ByVal _pstSound As Const orxSOUNDSYSTEM_SOUND Ptr) As orxSOUNDSYSTEM_STATUS

/'* Sets global volume
 * @param[in] _fGlobalVolume                          Global volume to set
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundSystem_SetGlobalVolume(ByVal _fGlobalVolume As orxFLOAT) As orxSTATUS

/'* Gets global volume
 * @return Gobal volume
 '/
Declare Function orxSoundSystem_GetGlobalVolume() As orxFLOAT

/'* Sets listener position
 * @param[in] _pvPosition                             Desired position
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxSoundSystem_SetListenerPosition(ByVal _pvPosition As Const orxVECTOR Ptr) As orxSTATUS

/'* Gets listener position
 * @param[out] _pvPosition                            Listener's position
 * @return orxVECTOR / orxNULL
 '/
Declare Function orxSoundSystem_GetListenerPosition(ByVal _pvPosition As orxVECTOR Ptr) As orxVECTOR Ptr

End Extern

#endif /' _orxSOUNDSYSTEM_BI '/

/'* @} '/
