/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxHashTable.h
 * @date 05/05/2005
 * @author cursor@arcallians.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxHashTable
 * 
 * Hashtable module
 * Module that handles Hashtables
 *
 * @{
 '/

 
#ifndef _orxHASHTABLE_BI
#define _orxHASHTABLE_BI
 
#include "orxInclude.bi"
#include "orx/memory/orxMemory.bi"

/' Internal hash table structure '/
Type __orxHASHTABLE_t As Any
Type orxHASHTABLE As __orxHASHTABLE_t

/' Define flags '/
#define orxHASHTABLE_KU32_FLAG_NONE             &H00000000  /'*< No flags (default behaviour) '/
#define orxHASHTABLE_KU32_FLAG_NOT_EXPANDABLE   &H00000001  /'*< The Hash table will not be expandable '/

Extern "Windows"

/'* @name HashTable creation/destruction.
 * @{ '/
/'* Create a new hash table and return it.
 * @param[in] _u32NbKey    Number of keys that will be inserted.
 * @param[in] _u32Flags    Flags used by the hash table
 * @param[in] _eMemType    Memory type to use
 * @return Returns the hashtable pointer or orxNULL if failed.
 '/
Declare Function orxHashTable_Create(ByVal _u32NbKey As orxU32,ByVal _u32Flags As orxU32,ByVal _eMemType As orxMEMORY_TYPE) As orxHASHTABLE Ptr

/'* Delete a hash table.
 * @param[in] _pstHashTable  Hash table to delete.
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxHashTable_Delete(ByVal _pstHashTable As orxHASHTABLE Ptr) As orxSTATUS

/'* Clear a hash table.
 * @param[in] _pstHashTable  Hash table to clear.
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxHashTable_Clear(ByVal _pstHashTable As orxHASHTABLE Ptr) As orxSTATUS
/'* @} '/

/'* Gets a hash table item counter
 * @param[in] _pstHashTable         Concerned hash table
 * @return    Item number
 '/
Declare Function orxHashTable_GetCounter(ByVal _pstHashTable As orxHASHTABLE Ptr) As orxU32

/'* @name HashTable key manipulation.
 * @{ '/
/'* Find an item in a hash table.
 * @param[in] _pstHashTable  The hash table where search.
 * @param[in] _u32Key      Key to find.
 * @return The Element associated to the key or orxNULL if not found.
 '/
Declare Function orxHashTable_Get(ByVal _pstHashTable As Const orxHASHTABLE Ptr,ByVal _u32Key As orxU32) As Any Ptr

/'* Set an item value.
 * @param[in] _pstHashTable The hash table where set.
 * @param[in] _u32Key      Key to assign.
 * @param[in] _pData       Data to assign.
 * @return      orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxHashTable_Set(ByVal _pstHashTable As orxHASHTABLE Ptr,ByVal _u32Key As orxU32,ByVal _pData As Any Ptr) As orxSTATUS


/'* Add an item value.
 * @param[in] _pstHashTable The hash table where set.
 * @param[in] _u32Key      Key to assign.
 * @param[in] _pData       Data to assign.
 * @return Returns the status of the operation. (fails if key already used)
 '/
Declare Function orxHashTable_Add(ByVal _pstHashTable As orxHASHTABLE Ptr,ByVal _u32Key As orxU32,ByVal _pData As Any Ptr) As orxSTATUS

/'* Remove an item.
 * @param[in] _pstHashTable  The hash table where remove.
 * @param[in] _u32Key      Key to remove.
 * @return Returns the status of the operation.
 '/
Declare Function orxHashTable_Remove(ByVal _pstHashTable As orxHASHTABLE Ptr,ByVal _u32Key As orxU32) As orxSTATUS
/'* @} '/


/'* @name HashTable iteration.
 * Used to pass over all elements of the hashtable.
 * @{ '/
/'* Find a the first item of the hashtable and return the iterator corresponding to the search.
 * @param[in] _pstHashTable Adress of the HashTable.
 * @param[in] _pu32Key Address where store the key, NULL to not store it.
 * @param[in] _ppData Address where store the data, NULL to not store it.
 * @return iterator of HashTable traversing. orxHANDLE_UNDEFINED if no element.
 '/
Declare Function orxHashTable_FindFirst(ByVal _pstHashTable As orxHASHTABLE Ptr,ByVal _pu32Key As orxU32 Ptr,ByVal _ppData As Any Ptr Ptr) As orxHANDLE

/'* Find a the next item of the hashtable and return the iterator corresponding to the search.
 * @param[in] _pstHashTable Adress of the HashTable.
 * @param[in] _pu32Key Address where store the key, NULL to not store it.
 * @param[in] _ppData Address where store the data, NULL to not store it.
 * @param[in] _hIterator Iterator to continue the search.
 * @return iterator of HashTable traversing. orxHANDLE_UNDEFINED if no element.
 '/
Declare Function orxHashTable_FindNext(ByVal _pstHashTable As orxHASHTABLE Ptr,ByVal _hIterator As orxHANDLE,ByVal _pu32Key As orxU32 Ptr,ByVal _ppData As Any Ptr Ptr) As orxHANDLE
/'* @} '/

/'******************************************************************************
 * DEBUG FUNCTION
 *****************************************************************************'/

#ifdef __orxDEBUG__

/'* Print the content of a Hash table
 * @param[in] _pstHashTable  Hash table to display
 '/
Declare Sub orxHashTable_DebugPrint(ByVal _pstHashTable As Const orxHASHTABLE Ptr)

#endif /' __orxDEBUG__ '/

End Extern

#endif /' _orxHASHTABLE_BI '/

/'* @} '/
