/' Orx - Portable Game Engine
 *
 * Orx is the legal property of its developers, whose names
 * are listed in the COPYRIGHT file distributed 
 * with this source distribution.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 '/

/'*
 * @file orxTree.h
 * @date 06/04/2005
 * @author iarwain@orx-project.org
 *
 * @todo
 '/

/'*
 * @addtogroup orxTree
 * 
 * Tree module
 * Module that handles trees
 *
 * @{
 '/


#ifndef _orxTREE_BI
#define _orxTREE_BI


#include "orxInclude.bi"
#include "orx/debug/orxDebug.bi"

Type orxTREE As __orxTREE_t

/'* Tree node structure
 '/
Type __orxTREE_NODE_t

  As __orxTREE_NODE_t Ptr pstParent           /'*< Parent node pointer : 4 '/
  As __orxTREE_NODE_t Ptr pstChild            /'*< First child node pointer : 8 '/
  As __orxTREE_NODE_t Ptr pstSibling          /'*< Next sibling node pointer : 12 '/
  As orxTREE      Ptr pstTree             /'*< Associated tree pointer : 16 '/

End Type

Type orxTREE_NODE As __orxTREE_NODE_t

/'* Tree structure
 '/
Type __orxTREE_t

  As orxTREE_NODE Ptr pstRoot                      /'*< Root node pointer : 4 '/
  As orxU32        u32Counter                     /'*< Node counter : 8 '/

End Type



Extern "Windows"

/'* Cleans a tree
 * @param[in]   _pstTree                        Concerned tree
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTree_Clean(ByVal _pstTree As orxTREE Ptr) As orxSTATUS

/'* Adds a node at the root of a tree
 * @param[in]   _pstTree                        Concerned tree
 * @param[in]   _pstNode                        Node to add
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTree_AddRoot(ByVal _pstTree As orxTREE Ptr, ByVal _pstNode As orxTREE_NODE Ptr) As orxSTATUS

/'* Adds a node as a parent of another one
 * @param[in]   _pstRefNode                     Reference node (add as a parent of this one)
 * @param[in]   _pstNode                        Node to add
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTree_AddParent(ByVal _pstRefNode As orxTREE_NODE Ptr, ByVal _pstNode As orxTREE_NODE Ptr) As orxSTATUS

/'* Adds a node as a child of another one
 * @param[in]   _pstRefNode                     Reference node (add as a child of this one)
 * @param[in]   _pstNode                        Node to add
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTree_AddChild(ByVal _pstRefNode As orxTREE_NODE Ptr, ByVal _pstNode As orxTREE_NODE Ptr) As orxSTATUS

/'* Moves a node as a child of another one of the same tree
 * @param[in]   _pstRefNode                     Reference node (move as a child of this one)
 * @param[in]   _pstNode                        Node to move
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTree_MoveAsChild(ByVal _pstRefNode As orxTREE_NODE Ptr, ByVal _pstNode As orxTREE_NODE Ptr) As orxSTATUS

/'* Removes a node from its tree
 * @param[in]   _pstNode                        Concerned node
 * @return orxSTATUS_SUCCESS / orxSTATUS_FAILURE
 '/
Declare Function orxTree_Remove(ByVal _pstNode As orxTREE_NODE Ptr) As orxSTATUS

End Extern 

/' *** Tree inlined accessors *** '/


/'* Gets a node tree
 * @param[in]   _pstNode                        Concerned node
 * @return orxTREE / orxNULL
 '/
#macro orxTree_GetTree(_pstNode,_result)

  /' Checks '/
  orxASSERT(_pstNode <> orxNULL)

  /' Returns it '/
  _result = (_pstNode->pstTree)
#endmacro

/'* Gets parent node
 * @param[in]   _pstNode                        Concerned node
 * @return orxTREE_NODE / orxNULL
 '/
#macro orxTree_GetParent(_pstNode,_result)

  /' Checks '/
  orxASSERT(_pstNode <> orxNULL)

  /' Returns it '/
  _result = Iif((_pstNode->pstTree <> orxNULL), _pstNode->pstParent,Cast(orxTREE_NODE Ptr,orxNULL))
#endmacro

/'* Gets first child node
 * @param[in]   _pstNode                        Concerned node
 * @return orxTREE_NODE / orxNULL
 '/
#macro orxTree_GetChild(_pstNode,_result)

  /' Checks '/
  orxASSERT(_pstNode <> orxNULL)

  /' Returns it '/
  _result = Iif((_pstNode->pstTree <> orxNULL), _pstNode->pstChild,Cast(orxTREE_NODE Ptr,orxNULL))
#endmacro

/'* Gets sibling node
 * @param[in]   _pstNode                        Concerned node
 * @return orxTREE_NODE / orxNULL
 '/
#macro orxTree_GetSibling(_pstNode,_result)

  /' Checks '/
  orxASSERT(_pstNode <> orxNULL)

  /' Returns it '/
  _result = Iif((_pstNode->pstTree <> orxNULL), _pstNode->pstSibling, Cast(orxTREE_NODE Ptr,orxNULL))
#endmacro


/'* Gets a tree root
 * @param[in]   _pstTree                        Concerned tree
 * @return orxTREE_NODE / orxNULL
 '/
#macro orxTree_GetRoot(_pstTree,_result)

  /' Checks '/
  orxASSERT(_pstTree <> orxNULL)

  /' Returns it '/
  _result = (_pstTree->pstRoot)
#endmacro

/'* Gets a tree counter
 * @param[in]   _pstTree                        Concerned tree
 * @return Number of nodes in tree
 '/
#macro orxTree_GetCounter(_pstTree,_result)

  /' Checks '/
  orxASSERT(_pstTree <> orxNULL)

  /' Returns it '/
  _result = (_pstTree->u32Counter)

#endmacro


#endif /' _orxTREE_BI '/

/'* @} '/
